/* 
/ dataSeltzer
/
/ a CGI component supporting dynamic WEB publishing
/ of (Open) Data from a SQLite/SpatiaLite DB-file
/
/ version 1.0, 2014 May 1
/
/ Author: Sandro Furieri a.furieri@lqt.it
/
/ Copyright (C) 2014  Alessandro Furieri
/
/   This program is free software: you can redistribute it and/or modify
/   it under the terms of the GNU Affero General Public License as published by
/   the Free Software Foundation, either version 3 of the License, or
/   (at your option) any later version.
/
/   This program is distributed in the hope that it will be useful,
/   but WITHOUT ANY WARRANTY; without even the implied warranty of
/   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/   GNU Affero General Public License for more details.
/
/   You should have received a copy of the GNU Affero General Public License
/   along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <memory.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <dirent.h>

#include "config.h"

#ifdef HAVE_LIBCGI_CGI_H
#include <libcgi/cgi.h>
#else
#include <cgi.h>
#endif

#ifdef HAVE_MINIZIP_ZIP_H
#include <minizip/zip.h>
#else
#include <zip.h>
#endif

#include <expat.h>
#include <sqlite3.h>
#include <spatialite.h>
#include <spatialite/gaiaaux.h>
#include <spatialite/gaiaexif.h>

#define BUFFSIZE	8192

#define myputs(x)	fputs(x, stdout)

#define FRMT64 "%lld"

#define LOG_ROTATE_DAILY		0
#define LOG_ROTATE_MONTHLY		1

#define LOG_INSULATION_MIXED	0
#define LOG_INSULATION_SEPARATE	1

#define METHOD_GET		0
#define METHOD_POST		1

#define COMPAR_NONE		0
#define COMPAR_EQ		1
#define COMPAR_NE		2
#define COMPAR_GT		3
#define COMPAR_GE		4
#define COMPAR_LT		5
#define COMPAR_LE		6
#define COMPAR_LIKE		7
#define COMPAR_NOT_LIKE		8
#define COMPAR_STARTS		9
#define COMPAR_NOT_STARTS	10
#define COMPAR_ENDS		11
#define COMPAR_NOT_ENDS		12
#define COMPAR_NULL		13
#define COMPAR_NOT_NULL		14

#define FUNCT_NONE	0
#define FUNCT_SUM	1
#define FUNCT_AVG	2
#define FUNCT_COUNT	3
#define FUNCT_MIN	4
#define FUNCT_MAX	5

#ifndef timersub
/* 
/ the MinGW compiler doesn't supports timersub
/
/ This is a copy from GNU C Library (GNU LGPL 2.1), sys/time.h.
/
*/
#define timersub(a, b, result)                                               \
 do {                                                                        \
   (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;                             \
   (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;                          \
   if ((result)->tv_usec < 0) {                                              \
     --(result)->tv_sec;                                                     \
     (result)->tv_usec += 1000000;                                           \
   }                                                                         \
 } while (0)
#endif

struct xml_meta_col
{
/* XML column meta info */
    char *name;
    char *type;
    int nullable;
    int max_length;
    int blob_cnt;
    int text_cnt;
    int int_cnt;
    int float_cnt;
    struct xml_meta_col *next;
};

struct xml_meta_columns
{
/* XML meta catalog */
    struct xml_meta_col *dflt;
    struct xml_meta_col *first;
    struct xml_meta_col *last;
};

struct auxdbf_fld
{
/* auxiliary DBF field struct */
    char already_used;
    gaiaDbfFieldPtr dbf_field;
    struct auxdbf_fld *next;
};

struct auxdbf_list
{
/* auxiliary DBF struct */
    struct auxdbf_fld *first;
    struct auxdbf_fld *last;
};

struct hidden
{
/* a struct wrapping an HiddenView */
    char *view_name;
    struct hidden *next;
};

struct filter
{
/* an auxiliary struct storing a filter */
    int filter_index;
    char *filter_column;
    int filter_comparison;
    char *filter_value;
    struct filter *next;
};

struct groupby
{
/* an auxiliary struct storing a Group By clause */
    int groupby_index;
    char *groupby_column;
    int groupby_comparison;
    char *groupby_value;
    struct groupby *next;
};

struct aggregate
{
/* an auxiliary struct storing an Aggregate Column */
    int aggregate_index;
    char *aggregate_column;
    char *aggregate_alias;
    int aggregate_function;
    int having_comparison;
    char *having_value;
    struct aggregate *next;
};

struct orderby
{
/* an auxiliary struct storing an Order By clause */
    int orderby_index;
    char *orderby_column;
    struct orderby *next;
};

struct extra_data_file
{
/* an auxiliary struct storing an extra-data files */
    char *path;
    struct extra_data_file *next;
};

struct instance
{
/* an auxiliary struct storing an Instance */
    char *name;
    char *title;
    char *path;
    struct instance *next;
};

struct config
{
/* an auxiliary struct storing the internal configuration */
    sqlite3 *db_handle;
    time_t now;
    struct timeval start;
    int bytes;
    char *logfile_dir_path;
    int logfile_rotation;
    int logfile_insulation;
    char *logfile_path;
    char *instance_name;
    char *instance_config_path;
    int enable_multi_instances;
    int enable_text_xml;
    int full_blob_xml;
    int full_xml_column_infos;
    int timestamps_localtime;
    char *db_path;
    int http_request_method;
    int max_resultset_rows;
    int max_help_resultset_rows;
    int max_rows_html_page;
    char *include_dir;
    char *temp_dir;
    char *cgi_url;
    char *icon_help_url;
    char *icon_add_url;
    char *icon_first;
    char *icon_last;
    char *icon_prev;
    char *icon_next;
    char *css_stylesheet_url;
    char *label_data_truncation;
    char *label_not_selected;
    char *label_instance;
    char *label_datasource;
    char *label_query_mode;
    char *label_mode_filter;
    char *label_mode_aggregate;
    char *label_filter_by;
    char *label_and_filter_by;
    char *label_any_value;
    char *label_equal;
    char *label_not_equal;
    char *label_greater;
    char *label_greater_equal;
    char *label_lesser;
    char *label_lesser_equal;
    char *label_like;
    char *label_not_like;
    char *label_starts_with;
    char *label_not_starts_with;
    char *label_ends_with;
    char *label_not_ends_with;
    char *label_is_null;
    char *label_is_not_null;
    char *label_output_format;
    char *label_output_html;
    char *label_output_xml;
    char *label_output_xml_zip;
    char *label_output_csv;
    char *label_output_dbf;
    char *label_output_shapefile;
    char *label_output_spatialite;
    char *label_button_reset;
    char *label_button_go;
    char *label_button_get_url;
    char *label_order_by;
    char *label_group_by;
    char *label_and_group_by;
    char *label_aggregate;
    char *label_and_aggregate;
    char *label_function_none;
    char *label_function_sum;
    char *label_function_avg;
    char *label_function_count;
    char *label_function_min;
    char *label_function_max;
    char *label_help_header;
    char *label_help_truncation;
    char *current_view_name;
    char *current_geometry_column;
    struct hidden *first_hidden;
    struct hidden *last_hidden;
    char *extra_data_dir;
    int extra_data_subdirs;
    struct extra_data_file *first_extra;
    struct extra_data_file *last_extra;
    char *arg_view;
    char *arg_query;
    int arg_count;
    int arg_count_groupby;
    int arg_count_aggregate;
    int arg_count_orderby;
    int changed_reset_all;
    int reset;
    struct filter *first_filter;
    struct filter *last_filter;
    struct groupby *first_groupby;
    struct groupby *last_groupby;
    struct aggregate *first_aggregate;
    struct aggregate *last_aggregate;
    struct orderby *first_orderby;
    struct orderby *last_orderby;
    char *arg_format;
    char *arg_select;
    int is_target_params;
    int is_target_results;
    int is_get_url;
    int is_submit;
    char *column;
    char *field;
    int is_field_help;
    int current_html_offset;
    struct instance *first_instance;
    struct instance *last_instance;
    char *CharData;
    int CharDataLen;
    int CharDataMax;
};

static void
reset_all_filters (struct config *params)
{
/* removing all filters */
    struct filter *pf;
    struct filter *pfn;
    pf = params->first_filter;
    while (pf != NULL)
      {
	  pfn = pf->next;
	  if (pf->filter_column != NULL)
	      free (pf->filter_column);
	  if (pf->filter_value != NULL)
	      free (pf->filter_value);
	  free (pf);
	  pf = pfn;
      }
    params->first_filter = NULL;
    params->last_filter = NULL;
}

static void
reset_all_aggregations (struct config *params)
{
/* removing all aggregations */
    struct groupby *pgb;
    struct groupby *pgbn;
    struct aggregate *pa;
    struct aggregate *pan;
    pgb = params->first_groupby;
    while (pgb != NULL)
      {
	  pgbn = pgb->next;
	  if (pgb->groupby_column)
	      free (pgb->groupby_column);
	  if (pgb->groupby_value)
	      free (pgb->groupby_value);
	  free (pgb);
	  pgb = pgbn;
      }
    pa = params->first_aggregate;
    while (pa != NULL)
      {
	  pan = pa->next;
	  if (pa->aggregate_column != NULL)
	      free (pa->aggregate_column);
	  if (pa->aggregate_alias != NULL)
	      sqlite3_free (pa->aggregate_alias);
	  if (pa->having_value != NULL)
	      free (pa->having_value);
	  free (pa);
	  pa = pan;
      }
    params->first_groupby = NULL;
    params->last_groupby = NULL;
    params->first_aggregate = NULL;
    params->last_aggregate = NULL;
}

static void
reset_all_orderbys (struct config *params)
{
/* removing all orderbys */
    struct orderby *po;
    struct orderby *pon;
    po = params->first_orderby;
    while (po != NULL)
      {
	  pon = po->next;
	  if (po->orderby_column != NULL)
	      free (po->orderby_column);
	  free (po);
	  po = pon;
      }
    params->first_orderby = NULL;
    params->last_orderby = NULL;
}

static void
free_config (struct config *params)
{
/* memory cleanup */
    struct hidden *ph;
    struct hidden *phn;
    struct extra_data_file *pex;
    struct extra_data_file *pexn;
    struct instance *pi;
    struct instance *pin;
    if (params->logfile_dir_path != NULL)
	free (params->logfile_dir_path);
    if (params->logfile_path != NULL)
	sqlite3_free (params->logfile_path);
    if (params->instance_name != NULL)
	free (params->instance_name);
    if (params->instance_config_path != NULL)
	free (params->instance_config_path);
    if (params->db_path != NULL)
	free (params->db_path);
    if (params->include_dir != NULL)
	free (params->include_dir);
    if (params->temp_dir != NULL)
	free (params->temp_dir);
    if (params->cgi_url != NULL)
	free (params->cgi_url);
    if (params->icon_help_url != NULL)
	free (params->icon_help_url);
    if (params->icon_add_url != NULL)
	free (params->icon_add_url);
    if (params->icon_first != NULL)
	free (params->icon_first);
    if (params->icon_last != NULL)
	free (params->icon_last);
    if (params->icon_prev != NULL)
	free (params->icon_prev);
    if (params->icon_next != NULL)
	free (params->icon_next);
    if (params->css_stylesheet_url != NULL)
	free (params->css_stylesheet_url);
    if (params->label_data_truncation != NULL)
	free (params->label_data_truncation);
    if (params->label_not_selected != NULL)
	free (params->label_not_selected);
    if (params->label_instance != NULL)
	free (params->label_instance);
    if (params->label_datasource != NULL)
	free (params->label_datasource);
    if (params->label_query_mode != NULL)
	free (params->label_query_mode);
    if (params->label_mode_filter != NULL)
	free (params->label_mode_filter);
    if (params->label_mode_aggregate != NULL)
	free (params->label_mode_aggregate);
    if (params->label_filter_by != NULL)
	free (params->label_filter_by);
    if (params->label_and_filter_by != NULL)
	free (params->label_and_filter_by);
    if (params->label_order_by != NULL)
	free (params->label_order_by);
    if (params->label_group_by != NULL)
	free (params->label_group_by);
    if (params->label_and_group_by != NULL)
	free (params->label_and_group_by);
    if (params->label_aggregate != NULL)
	free (params->label_aggregate);
    if (params->label_and_aggregate != NULL)
	free (params->label_and_aggregate);
    if (params->label_any_value != NULL)
	free (params->label_any_value);
    if (params->label_equal != NULL)
	free (params->label_equal);
    if (params->label_not_equal != NULL)
	free (params->label_not_equal);
    if (params->label_greater != NULL)
	free (params->label_greater);
    if (params->label_greater_equal != NULL)
	free (params->label_greater_equal);
    if (params->label_lesser != NULL)
	free (params->label_lesser);
    if (params->label_lesser_equal != NULL)
	free (params->label_lesser_equal);
    if (params->label_like != NULL)
	free (params->label_like);
    if (params->label_not_like != NULL)
	free (params->label_not_like);
    if (params->label_starts_with != NULL)
	free (params->label_starts_with);
    if (params->label_not_starts_with != NULL)
	free (params->label_not_starts_with);
    if (params->label_ends_with != NULL)
	free (params->label_ends_with);
    if (params->label_not_ends_with != NULL)
	free (params->label_not_ends_with);
    if (params->label_is_null != NULL)
	free (params->label_is_null);
    if (params->label_is_not_null != NULL)
	free (params->label_is_not_null);
    if (params->label_output_format != NULL)
	free (params->label_output_format);
    if (params->label_output_html != NULL)
	free (params->label_output_html);
    if (params->label_output_xml != NULL)
	free (params->label_output_xml);
    if (params->label_output_xml_zip != NULL)
	free (params->label_output_xml_zip);
    if (params->label_output_csv != NULL)
	free (params->label_output_csv);
    if (params->label_output_dbf != NULL)
	free (params->label_output_dbf);
    if (params->label_output_shapefile != NULL)
	free (params->label_output_shapefile);
    if (params->label_output_spatialite != NULL)
	free (params->label_output_spatialite);
    if (params->label_function_none != NULL)
	free (params->label_function_none);
    if (params->label_function_sum != NULL)
	free (params->label_function_sum);
    if (params->label_function_avg != NULL)
	free (params->label_function_avg);
    if (params->label_function_count != NULL)
	free (params->label_function_count);
    if (params->label_function_min != NULL)
	free (params->label_function_min);
    if (params->label_function_max != NULL)
	free (params->label_function_max);
    if (params->label_button_reset != NULL)
	free (params->label_button_reset);
    if (params->label_button_go != NULL)
	free (params->label_button_go);
    if (params->label_button_get_url != NULL)
	free (params->label_button_get_url);
    if (params->label_help_header != NULL)
	free (params->label_help_header);
    if (params->label_help_truncation != NULL)
	free (params->label_help_truncation);
    if (params->current_view_name != NULL)
	free (params->current_view_name);
    if (params->current_geometry_column != NULL)
	free (params->current_geometry_column);
    ph = params->first_hidden;
    while (ph != NULL)
      {
	  phn = ph->next;
	  if (ph->view_name != NULL)
	      free (ph->view_name);
	  free (ph);
	  ph = phn;
      }
    if (params->extra_data_dir != NULL)
	free (params->extra_data_dir);
    pex = params->first_extra;
    while (pex != NULL)
      {
	  pexn = pex->next;
	  if (pex->path != NULL)
	      free (pex->path);
	  free (pex);
	  pex = pexn;
      }
    if (params->arg_view != NULL)
	free (params->arg_view);
    if (params->arg_query != NULL)
	free (params->arg_query);
    reset_all_filters (params);
    reset_all_aggregations (params);
    reset_all_orderbys (params);
    if (params->arg_format != NULL)
	free (params->arg_format);
    if (params->arg_select != NULL)
	free (params->arg_select);
    if (params->column != NULL)
	free (params->column);
    if (params->field != NULL)
	free (params->field);
    pi = params->first_instance;
    while (pi != NULL)
      {
	  pin = pi->next;
	  if (pi->name != NULL)
	      free (pi->name);
	  if (pi->title != NULL)
	      free (pi->title);
	  if (pi->path != NULL)
	      free (pi->path);
	  free (pi);
	  pi = pin;
      }
    if (params->CharData != NULL)
	free (params->CharData);
}

static void
xmlCharData (void *data, const XML_Char * s, int len)
{
/* parsing XML char data */
    struct config *params = (struct config *) data;
    if ((params->CharDataLen + len) > params->CharDataMax)
      {
	  /* we must increase the CharData buffer size */
	  void *new_buf;
	  int new_size = params->CharDataMax;
	  while (new_size < params->CharDataLen + len)
	      new_size += BUFFSIZE;
	  new_buf = realloc (params->CharData, new_size);
	  if (new_buf)
	    {
		params->CharData = new_buf;
		params->CharDataMax = new_size;
	    }
      }
    memcpy (params->CharData + params->CharDataLen, s, len);
    params->CharDataLen += len;
}

static void
add_hidden_view (struct config *params)
{
/* adding an Hidden View */
    struct hidden *ph = malloc (sizeof (struct hidden));
    ph->view_name = malloc (params->CharDataLen + 1);
    memcpy (ph->view_name, params->CharData, params->CharDataLen);
    *(ph->view_name + params->CharDataLen) = '\0';
    ph->next = NULL;
    if (params->first_hidden == NULL)
	params->first_hidden = ph;
    if (params->last_hidden != NULL)
	params->last_hidden->next = ph;
    params->last_hidden = ph;
}

static void
add_extra_data_file (struct config *params)
{
/* adding an Extra Data File path */
    struct extra_data_file *pex = malloc (sizeof (struct extra_data_file));
    pex->path = malloc (params->CharDataLen + 1);
    memcpy (pex->path, params->CharData, params->CharDataLen);
    *(pex->path + params->CharDataLen) = '\0';
    pex->next = NULL;
    if (params->first_extra == NULL)
	params->first_extra = pex;
    if (params->last_extra != NULL)
	params->last_extra->next = pex;
    params->last_extra = pex;
}

static void
set_instance_name (struct config *params, const char *value)
{
/* setting the Instance Name */
    int len;
    if (params->instance_name != NULL)
	free (params->instance_name);
    len = strlen (value);
    params->instance_name = malloc (len + 1);
    strcpy (params->instance_name, value);
}

static void
start_main_tag (void *data, const char *el, const char **attr)
{
/* XML element starting */
    struct config *params = (struct config *) data;
    if (strcmp (el, "InstanceConfig") == 0)
      {
	  /* checking an Instance Configuration */
	  int count = 0;
	  const char *k;
	  const char *v;
	  const char *name = NULL;
	  const char *title = NULL;
	  const char *path = NULL;
	  int enabled = 1;
	  const char **attrib = attr;
	  while (*attrib != NULL)
	    {
		if ((count % 2) == 0)
		    k = *attrib;
		else
		  {
		      v = *attrib;
		      if (strcasecmp (k, "name") == 0)
			  name = v;
		      if (strcasecmp (k, "title") == 0)
			  title = v;
		      if (strcasecmp (k, "configPath") == 0)
			  path = v;
		      if (strcasecmp (k, "enabled") == 0)
			  enabled = atoi (v);
		  }
		attrib++;
		count++;
	    }
	  if (name != NULL)
	    {
		if (name != NULL && title != NULL && path != NULL && enabled)
		  {
		      /* adding an instance to the list */
		      int len;
		      struct instance *pi = malloc (sizeof (struct instance));
		      len = strlen (name);
		      pi->name = malloc (len + 1);
		      strcpy (pi->name, name);
		      len = strlen (title);
		      pi->title = malloc (len + 1);
		      strcpy (pi->title, title);
		      len = strlen (path);
		      pi->path = malloc (len + 1);
		      strcpy (pi->path, path);
		      pi->next = NULL;
		      if (params->first_instance == NULL)
			  params->first_instance = pi;
		      if (params->last_instance != NULL)
			  params->last_instance->next = pi;
		      params->last_instance = pi;
		  }
	    }
      }

    *(params->CharData) = '\0';
    params->CharDataLen = 0;
}

static void
end_main_tag (void *data, const char *el)
{
/* XML element ending */
    struct config *params = (struct config *) data;
    if (strcmp (el, "LogFileDirPath") == 0)
      {
	  if (params->logfile_dir_path != NULL)
	      free (params->logfile_dir_path);
	  params->logfile_dir_path = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->logfile_dir_path = malloc (params->CharDataLen + 1);
		memcpy (params->logfile_dir_path, params->CharData,
			params->CharDataLen);
		*(params->logfile_dir_path + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LogRotationCriterion") == 0)
      {
	  params->logfile_rotation = LOG_ROTATE_DAILY;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		if (strcasecmp (dummy, "monthly") == 0)
		    params->logfile_rotation = LOG_ROTATE_MONTHLY;
		else
		    params->logfile_rotation = LOG_ROTATE_DAILY;
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "LogInsulationCriterion") == 0)
      {
	  params->logfile_insulation = LOG_INSULATION_SEPARATE;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		if (strcasecmp (dummy, "mixed") == 0)
		    params->logfile_insulation = LOG_INSULATION_MIXED;
		else
		    params->logfile_insulation = LOG_INSULATION_SEPARATE;
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "EnableMultiInstances") == 0)
      {
	  params->enable_multi_instances = 0;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->enable_multi_instances = atoi (dummy);
		free (dummy);
	    }
	  return;
      }
}

static void
start_tag (void *data, const char *el, const char **attr)
{
/* XML element starting */
    struct config *params = (struct config *) data;
    if (el == NULL)
	el = NULL;
    if (attr == NULL)
	attr = NULL;		/* silencing stupid compiler warnings */

    *(params->CharData) = '\0';
    params->CharDataLen = 0;
}

static void
end_tag (void *data, const char *el)
{
/* XML element ending */
    struct config *params = (struct config *) data;
    if (strcmp (el, "DBPath") == 0)
      {
	  if (params->db_path != NULL)
	      free (params->db_path);
	  params->db_path = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->db_path = malloc (params->CharDataLen + 1);
		memcpy (params->db_path, params->CharData, params->CharDataLen);
		*(params->db_path + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "HttpRequestMethod") == 0)
      {
	  params->http_request_method = METHOD_GET;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		if (strcasecmp (dummy, "POST") == 0)
		    params->http_request_method = METHOD_POST;
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "MaxResultSetRows") == 0)
      {
	  params->max_resultset_rows = -1;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->max_resultset_rows = atoi (dummy);
		if (params->max_resultset_rows <= 0)
		    params->max_resultset_rows = -1;
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "MaxHelpResultSetRows") == 0)
      {
	  params->max_help_resultset_rows = -1;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->max_help_resultset_rows = atoi (dummy);
		if (params->max_help_resultset_rows <= 0)
		    params->max_help_resultset_rows = -1;
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "MaxRowsPerHtmlPage") == 0)
      {
	  params->max_rows_html_page = -1;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->max_rows_html_page = atoi (dummy);
		if (params->max_rows_html_page <= 0)
		    params->max_rows_html_page = -1;
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "HtmlIncludeDir") == 0)
      {
	  if (params->include_dir != NULL)
	      free (params->include_dir);
	  params->include_dir = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->include_dir = malloc (params->CharDataLen + 1);
		memcpy (params->include_dir, params->CharData,
			params->CharDataLen);
		*(params->include_dir + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "TempDir") == 0)
      {
	  if (params->temp_dir != NULL)
	      free (params->temp_dir);
	  params->temp_dir = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->temp_dir = malloc (params->CharDataLen + 1);
		memcpy (params->temp_dir, params->CharData,
			params->CharDataLen);
		*(params->temp_dir + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "CgiUrl") == 0)
      {
	  if (params->cgi_url != NULL)
	      free (params->cgi_url);
	  params->cgi_url = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->cgi_url = malloc (params->CharDataLen + 1);
		memcpy (params->cgi_url, params->CharData, params->CharDataLen);
		*(params->cgi_url + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "IconHelpUrl") == 0)
      {
	  if (params->icon_help_url != NULL)
	      free (params->icon_help_url);
	  params->icon_help_url = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->icon_help_url = malloc (params->CharDataLen + 1);
		memcpy (params->icon_help_url, params->CharData,
			params->CharDataLen);
		*(params->icon_help_url + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "IconAddUrl") == 0)
      {
	  if (params->icon_add_url != NULL)
	      free (params->icon_add_url);
	  params->icon_add_url = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->icon_add_url = malloc (params->CharDataLen + 1);
		memcpy (params->icon_add_url, params->CharData,
			params->CharDataLen);
		*(params->icon_add_url + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "IconFirstPage") == 0)
      {
	  if (params->icon_first != NULL)
	      free (params->icon_first);
	  params->icon_first = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->icon_first = malloc (params->CharDataLen + 1);
		memcpy (params->icon_first, params->CharData,
			params->CharDataLen);
		*(params->icon_first + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "IconLastPage") == 0)
      {
	  if (params->icon_last != NULL)
	      free (params->icon_last);
	  params->icon_last = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->icon_last = malloc (params->CharDataLen + 1);
		memcpy (params->icon_last, params->CharData,
			params->CharDataLen);
		*(params->icon_last + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "IconPrevPage") == 0)
      {
	  if (params->icon_prev != NULL)
	      free (params->icon_prev);
	  params->icon_prev = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->icon_prev = malloc (params->CharDataLen + 1);
		memcpy (params->icon_prev, params->CharData,
			params->CharDataLen);
		*(params->icon_prev + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "IconNextPage") == 0)
      {
	  if (params->icon_next != NULL)
	      free (params->icon_next);
	  params->icon_next = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->icon_next = malloc (params->CharDataLen + 1);
		memcpy (params->icon_next, params->CharData,
			params->CharDataLen);
		*(params->icon_next + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "CssStyleSheetUrl") == 0)
      {
	  if (params->css_stylesheet_url != NULL)
	      free (params->css_stylesheet_url);
	  params->css_stylesheet_url = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->css_stylesheet_url = malloc (params->CharDataLen + 1);
		memcpy (params->css_stylesheet_url, params->CharData,
			params->CharDataLen);
		*(params->css_stylesheet_url + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "EnableTextXml") == 0)
      {
	  params->enable_text_xml = 1;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->enable_text_xml = atoi (dummy);
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "FullBlobXml") == 0)
      {
	  params->full_blob_xml = 0;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->full_blob_xml = atoi (dummy);
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "FullXmlColumnInfos") == 0)
      {
	  params->full_xml_column_infos = 1;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->full_xml_column_infos = atoi (dummy);
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "TimestampsLocalTime") == 0)
      {
	  params->timestamps_localtime = 1;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->timestamps_localtime = atoi (dummy);
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "LabelDataTruncation") == 0)
      {
	  if (params->label_data_truncation != NULL)
	      free (params->label_data_truncation);
	  params->label_not_selected = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_data_truncation =
		    malloc (params->CharDataLen + 1);
		memcpy (params->label_data_truncation, params->CharData,
			params->CharDataLen);
		*(params->label_data_truncation + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelNotSelected") == 0)
      {
	  if (params->label_not_selected != NULL)
	      free (params->label_not_selected);
	  params->label_not_selected = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_not_selected = malloc (params->CharDataLen + 1);
		memcpy (params->label_not_selected, params->CharData,
			params->CharDataLen);
		*(params->label_not_selected + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelInstance") == 0)
      {
	  if (params->label_instance != NULL)
	      free (params->label_instance);
	  params->label_instance = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_instance = malloc (params->CharDataLen + 1);
		memcpy (params->label_instance, params->CharData,
			params->CharDataLen);
		*(params->label_instance + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelDataSource") == 0)
      {
	  if (params->label_datasource != NULL)
	      free (params->label_datasource);
	  params->label_datasource = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_datasource = malloc (params->CharDataLen + 1);
		memcpy (params->label_datasource, params->CharData,
			params->CharDataLen);
		*(params->label_datasource + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelQueryMode") == 0)
      {
	  if (params->label_query_mode != NULL)
	      free (params->label_query_mode);
	  params->label_query_mode = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_query_mode = malloc (params->CharDataLen + 1);
		memcpy (params->label_query_mode, params->CharData,
			params->CharDataLen);
		*(params->label_query_mode + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelModeFilter") == 0)
      {
	  if (params->label_mode_filter != NULL)
	      free (params->label_mode_filter);
	  params->label_mode_filter = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_mode_filter = malloc (params->CharDataLen + 1);
		memcpy (params->label_mode_filter, params->CharData,
			params->CharDataLen);
		*(params->label_mode_filter + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelModeAggregate") == 0)
      {
	  if (params->label_mode_aggregate != NULL)
	      free (params->label_mode_aggregate);
	  params->label_mode_aggregate = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_mode_aggregate = malloc (params->CharDataLen + 1);
		memcpy (params->label_mode_aggregate, params->CharData,
			params->CharDataLen);
		*(params->label_mode_aggregate + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelFilterBy") == 0)
      {
	  if (params->label_filter_by != NULL)
	      free (params->label_filter_by);
	  params->label_filter_by = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_filter_by = malloc (params->CharDataLen + 1);
		memcpy (params->label_filter_by, params->CharData,
			params->CharDataLen);
		*(params->label_filter_by + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelAndFilterBy") == 0)
      {
	  if (params->label_and_filter_by != NULL)
	      free (params->label_and_filter_by);
	  params->label_and_filter_by = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_and_filter_by = malloc (params->CharDataLen + 1);
		memcpy (params->label_and_filter_by, params->CharData,
			params->CharDataLen);
		*(params->label_and_filter_by + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOrderBy") == 0)
      {
	  if (params->label_order_by != NULL)
	      free (params->label_order_by);
	  params->label_order_by = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_order_by = malloc (params->CharDataLen + 1);
		memcpy (params->label_order_by, params->CharData,
			params->CharDataLen);
		*(params->label_order_by + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelGroupBy") == 0)
      {
	  if (params->label_group_by != NULL)
	      free (params->label_group_by);
	  params->label_group_by = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_group_by = malloc (params->CharDataLen + 1);
		memcpy (params->label_group_by, params->CharData,
			params->CharDataLen);
		*(params->label_group_by + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelAndGroupBy") == 0)
      {
	  if (params->label_and_group_by != NULL)
	      free (params->label_and_group_by);
	  params->label_and_group_by = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_and_group_by = malloc (params->CharDataLen + 1);
		memcpy (params->label_and_group_by, params->CharData,
			params->CharDataLen);
		*(params->label_and_group_by + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelAggregate") == 0)
      {
	  if (params->label_aggregate != NULL)
	      free (params->label_aggregate);
	  params->label_aggregate = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_aggregate = malloc (params->CharDataLen + 1);
		memcpy (params->label_aggregate, params->CharData,
			params->CharDataLen);
		*(params->label_aggregate + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelAndAggregate") == 0)
      {
	  if (params->label_and_aggregate != NULL)
	      free (params->label_and_aggregate);
	  params->label_and_aggregate = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_and_aggregate = malloc (params->CharDataLen + 1);
		memcpy (params->label_and_aggregate, params->CharData,
			params->CharDataLen);
		*(params->label_and_aggregate + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelAnyValue") == 0)
      {
	  if (params->label_any_value != NULL)
	      free (params->label_any_value);
	  params->label_any_value = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_any_value = malloc (params->CharDataLen + 1);
		memcpy (params->label_any_value, params->CharData,
			params->CharDataLen);
		*(params->label_any_value + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelEqual") == 0)
      {
	  if (params->label_equal != NULL)
	      free (params->label_equal);
	  params->label_equal = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_equal = malloc (params->CharDataLen + 1);
		memcpy (params->label_equal, params->CharData,
			params->CharDataLen);
		*(params->label_equal + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelNotEqual") == 0)
      {
	  if (params->label_not_equal != NULL)
	      free (params->label_not_equal);
	  params->label_not_equal = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_not_equal = malloc (params->CharDataLen + 1);
		memcpy (params->label_not_equal, params->CharData,
			params->CharDataLen);
		*(params->label_not_equal + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelGreater") == 0)
      {
	  if (params->label_greater != NULL)
	      free (params->label_greater);
	  params->label_greater = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_greater = malloc (params->CharDataLen + 1);
		memcpy (params->label_greater, params->CharData,
			params->CharDataLen);
		*(params->label_greater + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelGreaterEqual") == 0)
      {
	  if (params->label_greater_equal != NULL)
	      free (params->label_greater_equal);
	  params->label_greater_equal = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_greater_equal = malloc (params->CharDataLen + 1);
		memcpy (params->label_greater_equal, params->CharData,
			params->CharDataLen);
		*(params->label_greater_equal + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelLesser") == 0)
      {
	  if (params->label_lesser != NULL)
	      free (params->label_lesser);
	  params->label_lesser = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_lesser = malloc (params->CharDataLen + 1);
		memcpy (params->label_lesser, params->CharData,
			params->CharDataLen);
		*(params->label_lesser + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelLesserEqual") == 0)
      {
	  if (params->label_lesser_equal != NULL)
	      free (params->label_lesser_equal);
	  params->label_lesser_equal = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_lesser_equal = malloc (params->CharDataLen + 1);
		memcpy (params->label_lesser_equal, params->CharData,
			params->CharDataLen);
		*(params->label_lesser_equal + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelLike") == 0)
      {
	  if (params->label_like != NULL)
	      free (params->label_like);
	  params->label_like = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_like = malloc (params->CharDataLen + 1);
		memcpy (params->label_like, params->CharData,
			params->CharDataLen);
		*(params->label_like + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelNotLike") == 0)
      {
	  if (params->label_not_like != NULL)
	      free (params->label_not_like);
	  params->label_not_like = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_not_like = malloc (params->CharDataLen + 1);
		memcpy (params->label_not_like, params->CharData,
			params->CharDataLen);
		*(params->label_not_like + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelStartsWith") == 0)
      {
	  if (params->label_starts_with != NULL)
	      free (params->label_starts_with);
	  params->label_starts_with = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_starts_with = malloc (params->CharDataLen + 1);
		memcpy (params->label_starts_with, params->CharData,
			params->CharDataLen);
		*(params->label_starts_with + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelNotStartsWith") == 0)
      {
	  if (params->label_not_starts_with != NULL)
	      free (params->label_not_starts_with);
	  params->label_not_starts_with = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_not_starts_with =
		    malloc (params->CharDataLen + 1);
		memcpy (params->label_not_starts_with, params->CharData,
			params->CharDataLen);
		*(params->label_not_starts_with + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelEndsWith") == 0)
      {
	  if (params->label_ends_with != NULL)
	      free (params->label_ends_with);
	  params->label_ends_with = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_ends_with = malloc (params->CharDataLen + 1);
		memcpy (params->label_ends_with, params->CharData,
			params->CharDataLen);
		*(params->label_ends_with + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelNotEndsWith") == 0)
      {
	  if (params->label_not_ends_with != NULL)
	      free (params->label_not_ends_with);
	  params->label_not_ends_with = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_not_ends_with = malloc (params->CharDataLen + 1);
		memcpy (params->label_not_ends_with, params->CharData,
			params->CharDataLen);
		*(params->label_not_ends_with + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelIsNull") == 0)
      {
	  if (params->label_is_null != NULL)
	      free (params->label_is_null);
	  params->label_is_null = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_is_null = malloc (params->CharDataLen + 1);
		memcpy (params->label_is_null, params->CharData,
			params->CharDataLen);
		*(params->label_is_null + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelIsNotNull") == 0)
      {
	  if (params->label_is_not_null != NULL)
	      free (params->label_is_not_null);
	  params->label_is_not_null = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_is_not_null = malloc (params->CharDataLen + 1);
		memcpy (params->label_is_not_null, params->CharData,
			params->CharDataLen);
		*(params->label_is_not_null + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputFormat") == 0)
      {
	  if (params->label_output_format != NULL)
	      free (params->label_output_format);
	  params->label_output_format = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_format = malloc (params->CharDataLen + 1);
		memcpy (params->label_output_format, params->CharData,
			params->CharDataLen);
		*(params->label_output_format + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputHtml") == 0)
      {
	  if (params->label_output_html != NULL)
	      free (params->label_output_html);
	  params->label_output_html = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_html = malloc (params->CharDataLen + 1);
		memcpy (params->label_output_html, params->CharData,
			params->CharDataLen);
		*(params->label_output_html + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputXml") == 0)
      {
	  if (params->label_output_xml != NULL)
	      free (params->label_output_xml);
	  params->label_output_xml = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_xml = malloc (params->CharDataLen + 1);
		memcpy (params->label_output_xml, params->CharData,
			params->CharDataLen);
		*(params->label_output_xml + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputXmlZip") == 0)
      {
	  if (params->label_output_xml_zip != NULL)
	      free (params->label_output_xml_zip);
	  params->label_output_xml_zip = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_xml_zip = malloc (params->CharDataLen + 1);
		memcpy (params->label_output_xml_zip, params->CharData,
			params->CharDataLen);
		*(params->label_output_xml_zip + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputCsv") == 0)
      {
	  if (params->label_output_csv != NULL)
	      free (params->label_output_csv);
	  params->label_output_csv = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_csv = malloc (params->CharDataLen + 1);
		memcpy (params->label_output_csv, params->CharData,
			params->CharDataLen);
		*(params->label_output_csv + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputDbf") == 0)
      {
	  if (params->label_output_dbf != NULL)
	      free (params->label_output_dbf);
	  params->label_output_dbf = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_dbf = malloc (params->CharDataLen + 1);
		memcpy (params->label_output_dbf, params->CharData,
			params->CharDataLen);
		*(params->label_output_dbf + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputShapefile") == 0)
      {
	  if (params->label_output_shapefile != NULL)
	      free (params->label_output_shapefile);
	  params->label_output_shapefile = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_shapefile =
		    malloc (params->CharDataLen + 1);
		memcpy (params->label_output_shapefile, params->CharData,
			params->CharDataLen);
		*(params->label_output_shapefile + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelOutputSpatialite") == 0)
      {
	  if (params->label_output_spatialite != NULL)
	      free (params->label_output_spatialite);
	  params->label_output_spatialite = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_output_spatialite =
		    malloc (params->CharDataLen + 1);
		memcpy (params->label_output_spatialite, params->CharData,
			params->CharDataLen);
		*(params->label_output_spatialite + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelButtonReset") == 0)
      {
	  if (params->label_button_reset != NULL)
	      free (params->label_button_reset);
	  params->label_button_reset = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_button_reset = malloc (params->CharDataLen + 1);
		memcpy (params->label_button_reset, params->CharData,
			params->CharDataLen);
		*(params->label_button_reset + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelButtonGo") == 0)
      {
	  if (params->label_button_go != NULL)
	      free (params->label_button_go);
	  params->label_button_go = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_button_go = malloc (params->CharDataLen + 1);
		memcpy (params->label_button_go, params->CharData,
			params->CharDataLen);
		*(params->label_button_go + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelButtonGetUrl") == 0)
      {
	  if (params->label_button_get_url != NULL)
	      free (params->label_button_get_url);
	  params->label_button_get_url = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_button_get_url = malloc (params->CharDataLen + 1);
		memcpy (params->label_button_get_url, params->CharData,
			params->CharDataLen);
		*(params->label_button_get_url + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelFunctionNone") == 0)
      {
	  if (params->label_function_none != NULL)
	      free (params->label_function_none);
	  params->label_function_none = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_function_none = malloc (params->CharDataLen + 1);
		memcpy (params->label_function_none, params->CharData,
			params->CharDataLen);
		*(params->label_function_none + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelFunctionSum") == 0)
      {
	  if (params->label_function_sum != NULL)
	      free (params->label_function_sum);
	  params->label_function_sum = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_function_sum = malloc (params->CharDataLen + 1);
		memcpy (params->label_function_sum, params->CharData,
			params->CharDataLen);
		*(params->label_function_sum + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelFunctionAvg") == 0)
      {
	  if (params->label_function_avg != NULL)
	      free (params->label_function_avg);
	  params->label_function_avg = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_function_avg = malloc (params->CharDataLen + 1);
		memcpy (params->label_function_avg, params->CharData,
			params->CharDataLen);
		*(params->label_function_avg + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelFunctionCount") == 0)
      {
	  if (params->label_function_count != NULL)
	      free (params->label_function_count);
	  params->label_function_count = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_function_count = malloc (params->CharDataLen + 1);
		memcpy (params->label_function_count, params->CharData,
			params->CharDataLen);
		*(params->label_function_count + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelFunctionMin") == 0)
      {
	  if (params->label_function_min != NULL)
	      free (params->label_function_min);
	  params->label_function_min = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_function_min = malloc (params->CharDataLen + 1);
		memcpy (params->label_function_min, params->CharData,
			params->CharDataLen);
		*(params->label_function_min + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelFunctionMax") == 0)
      {
	  if (params->label_function_max != NULL)
	      free (params->label_function_max);
	  params->label_function_max = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_function_max = malloc (params->CharDataLen + 1);
		memcpy (params->label_function_max, params->CharData,
			params->CharDataLen);
		*(params->label_function_max + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelHelpPanelHeader") == 0)
      {
	  if (params->label_help_header != NULL)
	      free (params->label_help_header);
	  params->label_help_header = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_help_header = malloc (params->CharDataLen + 1);
		memcpy (params->label_help_header, params->CharData,
			params->CharDataLen);
		*(params->label_help_header + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "LabelHelpPanelTruncation") == 0)
      {
	  if (params->label_help_truncation != NULL)
	      free (params->label_help_truncation);
	  params->label_help_truncation = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->label_help_truncation =
		    malloc (params->CharDataLen + 1);
		memcpy (params->label_help_truncation, params->CharData,
			params->CharDataLen);
		*(params->label_help_truncation + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "ExtraDataDir") == 0)
      {
	  if (params->extra_data_dir != NULL)
	      free (params->extra_data_dir);
	  params->extra_data_dir = NULL;
	  if (params->CharDataLen > 0)
	    {
		params->extra_data_dir = malloc (params->CharDataLen + 1);
		memcpy (params->extra_data_dir, params->CharData,
			params->CharDataLen);
		*(params->extra_data_dir + params->CharDataLen) = '\0';
	    }
	  return;
      }
    if (strcmp (el, "ExtraDataSubdirs") == 0)
      {
	  params->extra_data_subdirs = 0;
	  if (params->CharDataLen > 0)
	    {
		char *dummy = malloc (params->CharDataLen + 1);
		memcpy (dummy, params->CharData, params->CharDataLen);
		*(dummy + params->CharDataLen) = '\0';
		params->extra_data_subdirs = atoi (dummy);
		free (dummy);
	    }
	  return;
      }
    if (strcmp (el, "HiddenView") == 0)
      {
	  if (params->CharDataLen > 0)
	      add_hidden_view (params);
	  return;
      }
    if (strcmp (el, "ExtraDataFile") == 0)
      {
	  if (params->CharDataLen > 0)
	      add_extra_data_file (params);
	  return;
      }
}

static int
parse_main_config (struct config *params)
{
/* attempting to load and parse the Main config file */
    int err = 0;
    XML_Parser parser;
    FILE *xml_file;
    char Buff[BUFFSIZE];
    int done = 0;
    int len;
    struct instance *in;

    xml_file = fopen ("./dataSeltzerMain.conf", "rb");
    if (!xml_file)
      {
	  fprintf (stderr,
		   "dataSeltzerMain: cannot open ./dataSeltzerMain.conf\n");
	  return 0;
      }

    parser = XML_ParserCreate (NULL);
    if (!parser)
      {
	  fclose (xml_file);
	  fprintf (stderr,
		   "dataSeltzerMain: couldn't allocate memory for XML parser\n");
	  return 0;
      }

    XML_SetUserData (parser, params);
/* XML parsing */
    XML_SetElementHandler (parser, start_main_tag, end_main_tag);
    XML_SetCharacterDataHandler (parser, xmlCharData);
    while (!done)
      {
	  len = fread (Buff, 1, BUFFSIZE, xml_file);
	  if (ferror (xml_file))
	    {
		fprintf (stderr, "dataSeltzerMain: XML Read error\n");
		return 0;
	    }
	  done = feof (xml_file);
	  if (!XML_Parse (parser, Buff, len, done))
	    {
		fprintf (stderr,
			 "dataSeltzerMain: XML parse error at line %d:\n%s\n",
			 (int) XML_GetCurrentLineNumber (parser),
			 XML_ErrorString (XML_GetErrorCode (parser)));
		return 0;
	    }
      }
    XML_ParserFree (parser);
    fclose (xml_file);

/* validity check */
    if (params->logfile_dir_path == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LogFileDirPath> !!\n");
	  err = 1;
      }
    if (err)
	return 0;
    if (params->first_instance == NULL)
	return 0;
    if (params->instance_name == NULL)
      {
	  /* 
	   * if no instance-name was set 
	   * we'll get the first one (default)
	   */
	  set_instance_name (params, params->first_instance->name);
      }
    in = params->first_instance;
    while (in != NULL)
      {
	  if (strcmp (in->name, params->instance_name) == 0)
	    {
		/* ok, found a valid matching Instance */
		int len = strlen (in->path);
		if (params->instance_config_path != NULL)
		    free (params->instance_config_path);
		params->instance_config_path = malloc (len + 1);
		strcpy (params->instance_config_path, in->path);
		break;
	    }
	  in = in->next;
      }
    return 1;
}

static int
parse_instance_config (struct config *params)
{
/* attempting to load and parse the Instance config file */
    int err = 0;
    XML_Parser parser;
    FILE *xml_file;
    char Buff[BUFFSIZE];
    int done = 0;
    int len;

    xml_file = fopen (params->instance_config_path, "rb");
    if (!xml_file)
      {
	  fprintf (stderr, "dataSeltzerInstance: cannot open %s\n",
		   params->instance_config_path);
	  return 0;
      }

    parser = XML_ParserCreate (NULL);
    if (!parser)
      {
	  fclose (xml_file);
	  fprintf (stderr,
		   "dataSeltzerInstance: couldn't allocate memory for XML parser\n");
	  return 0;
      }

    XML_SetUserData (parser, params);
/* XML parsing */
    XML_SetElementHandler (parser, start_tag, end_tag);
    XML_SetCharacterDataHandler (parser, xmlCharData);
    while (!done)
      {
	  len = fread (Buff, 1, BUFFSIZE, xml_file);
	  if (ferror (xml_file))
	    {
		fprintf (stderr, "dataSeltzerInstance: XML Read error\n");
		return 0;
	    }
	  done = feof (xml_file);
	  if (!XML_Parse (parser, Buff, len, done))
	    {
		fprintf (stderr,
			 "dataSeltzerInstance: XML parse error at line %d:\n%s\n",
			 (int) XML_GetCurrentLineNumber (parser),
			 XML_ErrorString (XML_GetErrorCode (parser)));
		return 0;
	    }
      }
    XML_ParserFree (parser);
    fclose (xml_file);

/* validity check */
    if (params->db_path == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <DBPath> !!\n");
	  err = 1;
      }
    if (params->include_dir == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <HtmlIncludeDir> !!\n");
	  err = 1;
      }
    if (params->temp_dir == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <TempDir> !!\n");
	  err = 1;
      }
    if (params->cgi_url == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <CgiUrl> !!\n");
	  err = 1;
      }
    if (params->icon_help_url == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <IconHelpUrl> !!\n");
	  err = 1;
      }
    if (params->icon_add_url == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <IconAddUrl> !!\n");
	  err = 1;
      }
    if (params->icon_first == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <IconFirstPage> !!\n");
	  err = 1;
      }
    if (params->icon_last == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <IconLastPage> !!\n");
	  err = 1;
      }
    if (params->icon_prev == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <IconPrevPage> !!\n");
	  err = 1;
      }
    if (params->icon_next == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <IconNextPage> !!\n");
	  err = 1;
      }
    if (params->css_stylesheet_url == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <CssStyleSheetUrl> !!\n");
	  err = 1;
      }
    if (params->label_not_selected == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelNotSelected> !!\n");
	  err = 1;
      }
    if (params->label_instance == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelInstance> !!\n");
	  err = 1;
      }
    if (params->label_datasource == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelDataSource> !!\n");
	  err = 1;
      }
    if (params->label_query_mode == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelQueryMode> !!\n");
	  err = 1;
      }
    if (params->label_filter_by == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelFilterBy> !!\n");
	  err = 1;
      }
    if (params->label_and_filter_by == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelAndFilterBy> !!\n");
	  err = 1;
      }
    if (params->label_order_by == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelOrderBy> !!\n");
	  err = 1;
      }
    if (params->label_group_by == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelGroupBy> !!\n");
	  err = 1;
      }
    if (params->label_and_group_by == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelAndGrouprBy> !!\n");
	  err = 1;
      }
    if (params->label_aggregate == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelAggregate> !!\n");
	  err = 1;
      }
    if (params->label_and_aggregate == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelAndAggregate> !!\n");
	  err = 1;
      }
    if (params->label_equal == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelEqual> !!\n");
	  err = 1;
      }
    if (params->label_not_equal == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelNotEqual> !!\n");
	  err = 1;
      }
    if (params->label_greater == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelGreater> !!\n");
	  err = 1;
      }
    if (params->label_greater_equal == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelGreaterEqual> !!\n");
	  err = 1;
      }
    if (params->label_lesser == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelLesser> !!\n");
	  err = 1;
      }
    if (params->label_lesser_equal == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelLesserEqual> !!\n");
	  err = 1;
      }
    if (params->label_like == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelLike> !!\n");
	  err = 1;
      }
    if (params->label_not_like == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelNotLike> !!\n");
	  err = 1;
      }
    if (params->label_starts_with == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelStartsWith> !!\n");
	  err = 1;
      }
    if (params->label_not_starts_with == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelNotStartsWith> !!\n");
	  err = 1;
      }
    if (params->label_ends_with == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelEndsWith> !!\n");
	  err = 1;
      }
    if (params->label_not_ends_with == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelNotEndsWith> !!\n");
	  err = 1;
      }
    if (params->label_is_null == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelIsNull> !!\n");
	  err = 1;
      }
    if (params->label_is_not_null == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelIsNotNull> !!\n");
	  err = 1;
      }
    if (params->label_output_format == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelOutputFormat> !!\n");
	  err = 1;
      }
    if (params->label_output_html == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelOutputHtml> !!\n");
	  err = 1;
      }
    if (params->label_button_reset == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelButtonReset> !!\n");
	  err = 1;
      }
    if (params->label_button_go == NULL)
      {
	  fprintf (stderr, "dataSeltzer: invalid/missing <LabelButtonGo> !!\n");
	  err = 1;
      }
    if (params->label_button_get_url == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelButtonGetUrl> !!\n");
	  err = 1;
      }
    if (params->label_function_none == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelFunctionNone> !!\n");
	  err = 1;
      }
    if (params->label_function_sum == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelFunctionSum> !!\n");
	  err = 1;
      }
    if (params->label_function_avg == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelFunctionAvg> !!\n");
	  err = 1;
      }
    if (params->label_function_count == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelFunctionCount> !!\n");
	  err = 1;
      }
    if (params->label_function_min == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelFunctionMin> !!\n");
	  err = 1;
      }
    if (params->label_function_max == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelFunctionMax> !!\n");
	  err = 1;
      }
    if (params->label_help_header == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelHelpPanelHeader> !!\n");
	  err = 1;
      }
    if (params->label_help_truncation == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: invalid/missing <LabelHelpPanelTruncation> !!\n");
	  err = 1;
      }
    if (err)
	return 0;
    return 1;
}

static int
test_hidden_view (struct config *params, const char *name)
{
/* testing for hidden View */
    struct hidden *ph = params->first_hidden;
    while (ph != NULL)
      {
	  if (strcasecmp (ph->view_name, name) == 0)
	      return 1;
	  ph = ph->next;
      }
    return 0;
}

static void
set_current_view (struct config *params, const char *name)
{
/* saving the currently selected View name */
    int len;
    if (params->current_view_name != NULL)
	free (params->current_view_name);
    params->current_view_name = NULL;
    if (name == NULL)
	return;
    len = strlen (name);
    params->current_view_name = malloc (len + 1);
    strcpy (params->current_view_name, name);
}

static void
set_current_geometry (struct config *params)
{
/* checking for an eventual Geometry column */
    int ret;
    char *sql;
    char **results;
    int rows;
    int columns;
    int i;

    if (params->current_geometry_column != NULL)
	free (params->current_geometry_column);
    params->current_geometry_column = NULL;

    sql = sqlite3_mprintf ("SELECT view_geometry "
			   "FROM views_geometry_columns "
			   "WHERE Lower(view_name) = Lower(%Q)",
			   params->current_view_name);
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows, &columns,
			   NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return;

    for (i = 1; i <= rows; i++)
      {
	  int len;
	  const char *name = results[(i * columns) + 0];
	  if (params->current_geometry_column != NULL)
	      free (params->current_geometry_column);
	  len = strlen (name);
	  params->current_geometry_column = malloc (len + 1);
	  strcpy (params->current_geometry_column, name);
      }
    sqlite3_free_table (results);
}

static int
do_test_geometry_view (sqlite3 * handle, const char *view)
{
/* testing if the current View is a Spatial one */
    int ret;
    char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    int ok = 0;
    if (view == NULL)
	return 0;

    sql = sqlite3_mprintf ("SELECT view_name "
			   "FROM views_geometry_columns "
			   "WHERE Lower(view_name) = Lower(%Q)", view);
    ret = sqlite3_get_table (handle, sql, &results, &rows, &columns, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;

    for (i = 1; i <= rows; i++)
	ok++;
    sqlite3_free_table (results);
    if (ok != 1)
	return 0;
    return 1;
}

static int
is_geometry_view (struct config *params)
{
/* testing if the current View is a Spatial one [Ordinary] */
    return do_test_geometry_view (params->db_handle, params->current_view_name);
}

static int
is_geometry_view_drilldown (struct config *params)
{
/* testing if the current View is a Spatial one [DrillDown] */
    return do_test_geometry_view (params->db_handle, params->arg_view);
}

static int
is_filter_query (struct config *params)
{
/* testing if the currently selected mode is Filter Query */
    if (params->arg_query == NULL)
	return 1;
    if (strcmp (params->arg_query, "aggregate") == 0)
	return 0;
    return 1;
}

static void
do_output_format (struct config *params)
{
/* populating the output formats list */
    char *option;
    if (params->arg_format != NULL)
      {
	  /* there is a currently selected Output Format */
	  if (strcasecmp (params->arg_format, "html") == 0)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"html\">%s</option>\n",
		   params->label_output_html);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"html\">%s</option>\n",
				   params->label_output_html);
	  myputs (option);
	  sqlite3_free (option);
	  if (params->label_output_xml != NULL)
	    {
		if (strcasecmp (params->arg_format, "xml") == 0)
		    option =
			sqlite3_mprintf
			("<option selected=\"1\" value=\"xml\">%s</option>\n",
			 params->label_output_xml);
		else
		    option =
			sqlite3_mprintf ("<option value=\"xml\">%s</option>\n",
					 params->label_output_xml);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_xml_zip != NULL)
	    {
		if (strcasecmp (params->arg_format, "xmlzip") == 0)
		    option =
			sqlite3_mprintf
			("<option selected=\"1\" value=\"xmlzip\">%s</option>\n",
			 params->label_output_xml_zip);
		else
		    option =
			sqlite3_mprintf
			("<option value=\"xmlzip\">%s</option>\n",
			 params->label_output_xml_zip);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_csv != NULL)
	    {
		if (strcasecmp (params->arg_format, "csv") == 0)
		    option =
			sqlite3_mprintf
			("<option selected=\"1\" value=\"csv\">%s</option>\n",
			 params->label_output_csv);
		else
		    option =
			sqlite3_mprintf ("<option value=\"csv\">%s</option>\n",
					 params->label_output_csv);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_dbf != NULL)
	    {
		if (strcasecmp (params->arg_format, "dbf") == 0)
		    option =
			sqlite3_mprintf
			("<option selected=\"1\" value=\"dbf\">%s</option>\n",
			 params->label_output_dbf);
		else
		    option =
			sqlite3_mprintf ("<option value=\"dbf\">%s</option>\n",
					 params->label_output_dbf);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_shapefile != NULL)
	    {
		if (is_geometry_view (params) && is_filter_query (params))
		  {
		      /* supporting Shapefile only if this View has a Geometry */
		      if (strcasecmp (params->arg_format, "shp") == 0)
			  option =
			      sqlite3_mprintf
			      ("<option selected=\"1\" value=\"shp\">%s</option>\n",
			       params->label_output_shapefile);
		      else
			  option =
			      sqlite3_mprintf
			      ("<option value=\"shp\">%s</option>\n",
			       params->label_output_shapefile);
		      myputs (option);
		      sqlite3_free (option);
		  }
	    }
	  if (params->label_output_shapefile != NULL)
	    {
		/* supporting SQLite/SpatiaLite */
		if (strcasecmp (params->arg_format, "sqlite") == 0)
		    option =
			sqlite3_mprintf
			("<option selected=\"1\" value=\"sqlite\">%s</option>\n",
			 params->label_output_spatialite);
		else
		    option =
			sqlite3_mprintf
			("<option value=\"sqlite\">%s</option>\n",
			 params->label_output_spatialite);
		myputs (option);
		sqlite3_free (option);
	    }
      }
    else
      {
	  /* no currently selected Output Format */
	  option =
	      sqlite3_mprintf
	      ("<option selected=\"1\" value=\"html\">%s</option>\n",
	       params->label_output_html);
	  myputs (option);
	  sqlite3_free (option);
	  if (params->label_output_xml != NULL)
	    {
		option =
		    sqlite3_mprintf ("<option value=\"xml\">%s</option>\n",
				     params->label_output_xml);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_xml_zip != NULL)
	    {
		option =
		    sqlite3_mprintf ("<option value=\"xmlzip\">%s</option>\n",
				     params->label_output_xml_zip);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_csv != NULL)
	    {
		option =
		    sqlite3_mprintf ("<option value=\"csv\">%s</option>\n",
				     params->label_output_csv);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_dbf != NULL)
	    {
		option =
		    sqlite3_mprintf ("<option value=\"dbf\">%s</option>\n",
				     params->label_output_dbf);
		myputs (option);
		sqlite3_free (option);
	    }
	  if (params->label_output_shapefile != NULL)
	    {
		if (is_geometry_view (params) && is_filter_query (params))
		  {
		      /* supporting Shapefile only if this View has a Geometry */
		      option =
			  sqlite3_mprintf
			  ("<option value=\"shp\">%s</option>\n",
			   params->label_output_shapefile);
		      myputs (option);
		      sqlite3_free (option);
		  }
	    }
	  if (params->label_output_spatialite != NULL)
	    {
		option =
		    sqlite3_mprintf
		    ("<option value=\"sqlite\">%s</option>\n",
		     params->label_output_spatialite);
		myputs (option);
		sqlite3_free (option);
	    }
      }
}

static int
do_output_comparisons (struct config *params, int comparison)
{
/* populating the comparison operators list */
    char *option;
    int ret = 0;
    if (comparison == COMPAR_NONE)
      {
	  /* no currently selected Comparison */
	  ret = 1;
	  option =
	      sqlite3_mprintf
	      ("<option selected=\"1\" value=\"any\">%s</option>\n",
	       params->label_any_value);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"eq\">%s</option>\n",
			       params->label_equal);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"ne\">%s</option>\n",
			       params->label_not_equal);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"gt\">%s</option>\n",
			       params->label_greater);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"ge\">%s</option>\n",
			       params->label_greater_equal);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"lt\">%s</option>\n",
			       params->label_lesser);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"le\">%s</option>\n",
			       params->label_lesser_equal);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"lk\">%s</option>\n",
			       params->label_like);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"nlk\">%s</option>\n",
			       params->label_not_like);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"sw\">%s</option>\n",
			       params->label_starts_with);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"nsw\">%s</option>\n",
			       params->label_not_starts_with);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"ew\">%s</option>\n",
			       params->label_ends_with);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"new\">%s</option>\n",
			       params->label_not_ends_with);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"in\">%s</option>\n",
			       params->label_is_null);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"inn\">%s</option>\n",
			       params->label_is_not_null);
	  myputs (option);
	  sqlite3_free (option);
      }
    else
      {
	  /* there is a currently selected Comparison */
	  option =
	      sqlite3_mprintf
	      ("<option value=\"any\">%s</option>\n", params->label_any_value);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_EQ)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"eq\">%s</option>\n",
		   params->label_equal);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"eq\">%s</option>\n",
				   params->label_equal);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_NE)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"ne\">%s</option>\n",
		   params->label_not_equal);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"ne\">%s</option>\n",
				   params->label_not_equal);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_GT)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"gt\">%s</option>\n",
		   params->label_greater);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"gt\">%s</option>\n",
				   params->label_greater);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_GE)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"ge\">%s</option>\n",
		   params->label_greater_equal);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"ge\">%s</option>\n",
				   params->label_greater_equal);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_LT)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"lt\">%s</option>\n",
		   params->label_lesser);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"lt\">%s</option>\n",
				   params->label_lesser);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_LE)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"le\">%s</option>\n",
		   params->label_lesser_equal);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"le\">%s</option>\n",
				   params->label_lesser_equal);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_LIKE)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\"value=\"lk\">%s</option>\n",
		   params->label_like);
	  else
	      option =
		  sqlite3_mprintf
		  ("<option value=\"lk\">%s</option>\n", params->label_like);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_NOT_LIKE)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"nlk\">%s</option>\n",
		   params->label_not_like);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"nlk\">%s</option>\n",
				   params->label_not_like);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_STARTS)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"sw\">%s</option>\n",
		   params->label_starts_with);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"sw\">%s</option>\n",
				   params->label_starts_with);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_NOT_STARTS)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"nsw\">%s</option>\n",
		   params->label_not_starts_with);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"nsw\">%s</option>\n",
				   params->label_not_starts_with);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_ENDS)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"ew\">%s</option>\n",
		   params->label_ends_with);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"ew\">%s</option>\n",
				   params->label_ends_with);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_NOT_ENDS)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"new\">%s</option>\n",
		   params->label_not_ends_with);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"new\">%s</option>\n",
				   params->label_not_ends_with);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_NULL)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"in\">%s</option>\n",
		   params->label_is_null);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"in\">%s</option>\n",
				   params->label_is_null);
	  myputs (option);
	  sqlite3_free (option);
	  if (comparison == COMPAR_NOT_NULL)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"inn\">%s</option>\n",
		   params->label_is_not_null);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"inn\">%s</option>\n",
				   params->label_is_not_null);
	  myputs (option);
	  sqlite3_free (option);
      }
    if (params->arg_view == NULL)
	ret = 1;
    return ret;
}

static void
do_output_columns (struct config *params, const char *column)
{
/* populating the column-names list */
    int ret;
    char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    char *option;
    char *quoted;

    if (params->current_view_name == NULL)
      {
	  option =
	      sqlite3_mprintf
	      ("<option selected=\"1\" value=\"\">%s</option>\n",
	       params->label_not_selected);
	  myputs (option);
	  sqlite3_free (option);
	  return;
      }

/* checking for an eventual Geometry */
    set_current_geometry (params);

    quoted = gaiaDoubleQuotedSql (params->current_view_name);
    sql = sqlite3_mprintf ("PRAGMA table_info(\"%s\")", quoted);
    free (quoted);
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows, &columns,
			   NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return;

    if (column == NULL)
	option =
	    sqlite3_mprintf
	    ("<option selected=\"1\" value=\"\">%s</option>\n",
	     params->label_not_selected);
    else
	option =
	    sqlite3_mprintf
	    ("<option value=\"\">%s</option>\n", params->label_not_selected);
    myputs (option);
    sqlite3_free (option);
    for (i = 1; i <= rows; i++)
      {
	  const char *name = results[(i * columns) + 1];
	  if (params->current_geometry_column != NULL)
	    {
		if (strcasecmp (params->current_geometry_column, name) == 0)
		    continue;
	    }
	  if (column != NULL)
	    {
		/* there is a currently selected View Name */
		if (strcasecmp (name, column) == 0)
		    option =
			sqlite3_mprintf
			("<option selected=\"1\" value=\"%s\">%s</option>\n",
			 name, name);
		else
		    option =
			sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
					 name, name);
	    }
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
				   name, name);
	  myputs (option);
	  sqlite3_free (option);
      }
    sqlite3_free_table (results);
}

static void
do_output_order_by (struct config *params, int index)
{
/* populating the column-names ORDER BY list */
    int ret;
    char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    char *option;
    char *quoted;
    struct orderby *orderby;
    const char *column;

    if (params->current_view_name == NULL)
      {
	  option =
	      sqlite3_mprintf
	      ("<option selected=\"1\" value=\"\">%s</option>\n",
	       params->label_not_selected);
	  myputs (option);
	  sqlite3_free (option);
	  return;
      }

/* retrieving the currently select ORDER BY clause */
    orderby = params->first_orderby;
    if (orderby == NULL)
	column = NULL;
    else
      {
	  int ind = 0;
	  while (orderby != NULL)
	    {
		if (ind == index)
		    column = orderby->orderby_column;
		orderby = orderby->next;
		ind++;
	    }
      }

/* checking for an eventual Geometry */
    set_current_geometry (params);

    quoted = gaiaDoubleQuotedSql (params->current_view_name);
    sql = sqlite3_mprintf ("PRAGMA table_info(\"%s\")", quoted);
    free (quoted);
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows, &columns,
			   NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return;
    if (column == NULL)
	option =
	    sqlite3_mprintf
	    ("<option selected=\"1\" value=\"\">%s</option>\n",
	     params->label_not_selected);
    else
	option =
	    sqlite3_mprintf
	    ("<option value=\"\">%s</option>\n", params->label_not_selected);
    myputs (option);
    sqlite3_free (option);
    for (i = 1; i <= rows; i++)
      {
	  const char *name = results[(i * columns) + 1];
	  if (params->current_geometry_column != NULL)
	    {
		if (strcasecmp (params->current_geometry_column, name) == 0)
		    continue;
	    }
	  if (column != NULL)
	    {
		/* there is a currently selected View Name */
		if (strcasecmp (name, column) == 0)
		    option =
			sqlite3_mprintf
			("<option selected=\"1\" value=\"%s\">%s</option>\n",
			 name, name);
		else
		    option =
			sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
					 name, name);
	    }
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
				   name, name);
	  myputs (option);
	  sqlite3_free (option);
      }
    sqlite3_free_table (results);
}

static void
do_view_autoselect (struct config *params)
{
/* attempting to autoselect the current view */
    int ret;
    const char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    const char *name;
    int count = 0;
    int len;

    sql = "SELECT name FROM sqlite_master WHERE type = 'view' ORDER BY name";
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows, &columns,
			   NULL);
    if (ret != SQLITE_OK)
	return;

    for (i = 1; i <= rows; i++)
      {
	  const char *xname = results[(i * columns) + 0];
	  if (test_hidden_view (params, xname) == 1)
	      continue;
	  name = xname;
	  count++;
      }
    sqlite3_free_table (results);

    if (count == 1)
      {
	  /* there is only one view being published - autoselect */
	  len = strlen (name);
	  params->arg_view = malloc (len + 1);
	  strcpy (params->arg_view, name);
      }
}

static int
check_multi_instances (struct config *params)
{
/* check is MultiInstances is really enabled */
    struct instance *pi;
    int count = 0;
    if (params->enable_multi_instances == 0)
	return 0;
    pi = params->first_instance;
    while (pi != NULL)
      {
	  count++;
	  pi = pi->next;
      }
    if (count < 2)
	return 0;
    return 1;
}

static int
check_multiple_views (struct config *params)
{
/* testing if there is a Unique View being published */
    int ret;
    const char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    int count = 0;
    sql = "SELECT name FROM sqlite_master WHERE type = 'view' ORDER BY name";
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows, &columns,
			   NULL);
    if (ret != SQLITE_OK)
	goto skip_views;

    for (i = 1; i <= rows; i++)
      {
	  const char *name = results[(i * columns) + 0];
	  if (test_hidden_view (params, name) == 1)
	      continue;
	  count++;
      }
    sqlite3_free_table (results);
  skip_views:
    if (count > 1)
	return 1;
    return 0;
}

static char *
get_unique_view_name (struct config *params)
{
/* retrieving the name of the Unique View being published */
    int ret;
    const char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    int len;
    int count = 0;
    char *xname = NULL;
    sql = "SELECT name FROM sqlite_master WHERE type = 'view' ORDER BY name";
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows, &columns,
			   NULL);
    if (ret != SQLITE_OK)
	goto skip_views;

    for (i = 1; i <= rows; i++)
      {
	  const char *name = results[(i * columns) + 0];
	  if (test_hidden_view (params, name) == 1)
	      continue;
	  count++;
	  len = strlen (name);
	  if (xname == NULL)
	    {
		xname = malloc (len + 1);
		strcpy (xname, name);
	    }
      }
    sqlite3_free_table (results);
  skip_views:
    if (count != 1)
      {
	  if (xname != NULL)
	      free (xname);
	  xname = NULL;
      }
    return xname;
}

static void
do_output_filter_frame (struct config *params)
{
/* will return an HTML page defining Query params (Filter Query) */
    int ret;
    const char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    char *path;
    char *option;
    struct filter *filter;
    int disable_compar;

/* HTML header */
    path = sqlite3_mprintf ("%s/params_header.inc", params->include_dir);
    cgi_include (path);
    sqlite3_free (path);

/* creating the <FORM> */
    if (params->http_request_method == METHOD_POST)
	myputs ("<form method=\"POST\" action=\"");
    else
	myputs ("<form method=\"GET\" action=\"");
    myputs (params->cgi_url);
    myputs ("\" target=\"params\">\n");

/* appending all HIDDEN <INPUT>s */
    myputs ("<input type=\"hidden\" name=\"in\" id=\"in\" value=\"");
    myputs (params->instance_name);
    myputs ("\">\n");
    myputs ("<input type=\"hidden\" name=\"cgi_url\" id=\"cgi_url\" value=\"");
    myputs (params->cgi_url);
    myputs ("\">\n");
    if (!check_multiple_views (params))
      {
	  /* there is only a single View being published */
	  char *name = get_unique_view_name (params);
	  set_current_view (params, name);
	  option =
	      sqlite3_mprintf
	      ("<input type=\"hidden\" name=\"cvw\" value=\"%s\">\n", name);
	  free (name);
	  myputs (option);
	  sqlite3_free (option);
      }
    else
      {
	  myputs ("<input type=\"hidden\" name=\"cvw\" value=\"");
	  if (params->arg_view != NULL)
	      myputs (params->arg_view);
	  else
	      myputs (params->label_not_selected);
	  myputs ("\">\n");
      }
    myputs ("<input type=\"hidden\" name=\"cqy\" value=\"filter\">\n");
    option =
	sqlite3_mprintf
	("<input type=\"hidden\" name=\"cnt\" id=\"cnt\" value=\"%d\">\n",
	 params->arg_count);
    myputs (option);
    sqlite3_free (option);
    option =
	sqlite3_mprintf
	("<input type=\"hidden\" name=\"cnto\" id=\"cnto\" value=\"3\">\n");
    myputs (option);
    sqlite3_free (option);
    if (!check_multiple_views (params))
      {
	  /* there is only a single View being published */
	  char *name = get_unique_view_name (params);
	  set_current_view (params, name);
	  option =
	      sqlite3_mprintf
	      ("<input type=\"hidden\" name=\"vw\" id=\"view\" value=\"%s\">\n",
	       name);
	  free (name);
	  myputs (option);
	  sqlite3_free (option);
      }

/* creating the <TABLE> */
    myputs ("<table class=\"params_table\">\n");

    if (check_multi_instances (params))
      {
	  /* HTML published instances */
	  struct instance *in;
	  myputs ("<tr><td class=\"params_label\">");
	  myputs (params->label_instance);
	  myputs ("</td><td class=\"params_background\" colspan=\"6\">\n");
	  myputs ("<select class=\"params_combo\" name=\"muin\" id=\"muin\" ");
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  in = params->first_instance;
	  while (in != NULL)
	    {
		if (params->instance_name != NULL)
		  {
		      /* there is a currently selected Instance */
		      if (strcasecmp (params->instance_name, in->name) == 0)
			{
			    option =
				sqlite3_mprintf
				("<option selected=\"1\" value=\"%s\">%s</option>\n",
				 in->name, in->title);
			}
		      else
			  option =
			      sqlite3_mprintf
			      ("<option value=\"%s\">%s</option>\n", in->name,
			       in->title);
		  }
		else
		    option =
			sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
					 in->name, in->name);
		myputs (option);
		in = in->next;
	    }
	  myputs ("</select></td></tr>\n");
      }

    if (check_multiple_views (params))
      {
	  /* HTML published Views */
	  myputs ("<tr><td class=\"params_label\">");
	  myputs (params->label_datasource);
	  myputs ("</td><td class=\"params_background\" colspan=\"6\">\n");
	  myputs ("<select class=\"params_combo\" name=\"vw\" id=\"view\" ");
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  if (params->arg_view == NULL)
	      do_view_autoselect (params);
	  if (params->arg_view == NULL)
	    {
		/* there is no currently selected View Name */
		option =
		    sqlite3_mprintf
		    ("<option selected=\"1\" value=\"\">%s</option>\n",
		     params->label_not_selected);
		set_current_view (params, NULL);
	    }
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"\">%s</option>\n",
				   params->label_not_selected);
	  myputs (option);
	  sqlite3_free (option);

	  sql =
	      "SELECT name FROM sqlite_master WHERE type = 'view' ORDER BY name";
	  ret =
	      sqlite3_get_table (params->db_handle, sql, &results, &rows,
				 &columns, NULL);
	  if (ret != SQLITE_OK)
	      goto skip_views;

	  for (i = 1; i <= rows; i++)
	    {
		const char *name = results[(i * columns) + 0];
		if (test_hidden_view (params, name) == 1)
		    continue;
		if (params->arg_view != NULL)
		  {
		      /* there is a currently selected View Name */
		      if (strcasecmp (params->arg_view, name) == 0)
			{
			    option =
				sqlite3_mprintf
				("<option selected=\"1\" value=\"%s\">%s</option>\n",
				 name, name);
			    set_current_view (params, name);
			}
		      else
			  option =
			      sqlite3_mprintf
			      ("<option value=\"%s\">%s</option>\n", name,
			       name);
		  }
		else
		    option =
			sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
					 name, name);
		myputs (option);
		sqlite3_free (option);
	    }
	  sqlite3_free_table (results);
	  myputs ("</select></td></tr>\n");
      }

  skip_views:
/* HTML query mode */
    myputs ("<tr><td class=\"params_label\">");
    myputs (params->label_query_mode);
    myputs ("</td><td class=\"params_background\" colspan=\"6\">\n");
    myputs ("<select class=\"params_combo\" name=\"q\" ");
    myputs ("onChange=\"formUpdate();return true;\">\n");
    option =
	sqlite3_mprintf
	("<option selected=\"1\" value=\"filter\">%s</option>\n",
	 params->label_mode_filter);
    myputs (option);
    sqlite3_free (option);
    option =
	sqlite3_mprintf
	("<option value=\"aggregate\">%s</option>\n",
	 params->label_mode_aggregate);
    myputs (option);
    sqlite3_free (option);
    myputs ("</select></td></tr>\n");

    filter = params->first_filter;
    while (filter != NULL)
      {
	  /* HTML filter by */
	  char *str;
	  myputs ("<tr><td class=\"params_label\">");
	  if (filter->filter_index == 0)
	      myputs (params->label_filter_by);
	  else
	      myputs (params->label_and_filter_by);
	  myputs ("</td><td class=\"params_background\">\n");
	  str =
	      sqlite3_mprintf
	      ("<select class=\"params_combo\" name=\"c%d\" id=\"c%d\" ",
	       filter->filter_index, filter->filter_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  do_output_columns (params, filter->filter_column);
	  myputs ("</select></td>\n");
	  myputs ("<td class=\"params_background\">\n");
	  str =
	      sqlite3_mprintf
	      ("<select class=\"params_combo\" name=\"x%d\" id=\"x%d\" ",
	       filter->filter_index, filter->filter_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  disable_compar =
	      do_output_comparisons (params, filter->filter_comparison);
	  myputs ("</select></td>\n");
	  myputs ("<td class=\"params_background\">\n");
	  if (disable_compar || filter->filter_column == NULL)
	    {
		myputs ("<img class=\"icon_disabled\" src=\"");
		myputs (params->icon_help_url);
		myputs ("\" alt=\"help\">\n</td>\n");
	    }
	  else
	    {
		str =
		    sqlite3_mprintf
		    ("<a href=\"#\" onClick=\"searchFieldValueFilter("
		     "'c%d','v%d'); return true;\">\n", filter->filter_index,
		     filter->filter_index);
		myputs (str);
		sqlite3_free (str);
		myputs ("<img src=\"");
		myputs (params->icon_help_url);
		myputs ("\" alt=\"help\">\n</a>\n</td>\n");
	    }
	  myputs ("<td class=\"params_background\">\n");
	  if (disable_compar || filter->filter_column == NULL)
	      str =
		  sqlite3_mprintf
		  ("<input class=\"params_input_disabled\" type=\"text\" readonly=\"readonly\" "
		   "name=\"v%d\" id=\"v%d\" size=\"30\" value=\"%s\"></td>\n",
		   filter->filter_index, filter->filter_index,
		   filter->filter_value);
	  else if (filter->filter_value != NULL)
	      str =
		  sqlite3_mprintf
		  ("<input class=\"params_input\" type=\"text\" "
		   "name=\"v%d\" id=\"v%d\" size=\"30\" value=\"%s\"></td>\n",
		   filter->filter_index, filter->filter_index,
		   filter->filter_value);
	  else
	      str =
		  sqlite3_mprintf
		  ("<input class=\"params_input\" type=\"text\" "
		   "name=\"v%d\" id=\"v%d\" size=\"30\"></td>\n",
		   filter->filter_index, filter->filter_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("<td class=\"params_background\">\n");
	  if (filter->next == NULL)
	    {
		/* the ADD-FILTER button only if this is the last filter */
		myputs ("<input type=\"image\" name=\"addf\" src=\"");
		myputs (params->icon_add_url);
		myputs ("\" onClick=\"formUpdate();return true;\">\n");
	    }
	  myputs ("</td><td class=\"params_fill\"></td></tr>\n");
	  filter = filter->next;
      }

/* HTML Order By #0 */
    myputs ("<tr><td class=\"params_label\">");
    myputs (params->label_order_by);
    myputs ("</td><td class=\"params_background\">\n");
    myputs ("<select class=\"params_combo\" name=\"o0\" ");
    myputs ("onChange=\"formUpdate();return true;\">\n");
    do_output_order_by (params, 0);
    myputs ("</select></td>\n");
/* HTML Order By #1 */
    myputs ("<td class=\"params_background\">\n");
    myputs ("<select class=\"params_combo\" name=\"o1\" ");
    myputs ("onChange=\"formUpdate();return true;\">\n");
    do_output_order_by (params, 1);
    myputs ("</select></td>\n");
/* HTML Order By #2 */
    myputs ("<td class=\"params_background\" colspan=\"4\">\n");
    myputs ("<select class=\"params_combo\" name=\"o2\" ");
    myputs ("onChange=\"formUpdate();return true;\">\n");
    do_output_order_by (params, 2);
    myputs ("</select></td></tr>\n");

/* HTML output format */
    myputs ("<tr><td class=\"params_label\">");
    myputs (params->label_output_format);
    myputs ("</td><td class=\"params_background\" colspan=\"6\">\n");
    myputs ("<select class=\"params_combo\" name=\"fmt\" ");
    myputs ("onChange=\"formUpdate();return true;\">\n");
    do_output_format (params);
    myputs ("</select></td></tr>\n");

/* HTML form buttons */
    myputs
	("<tr><td class=\"params_background_center\" colspan=\"7\" align=\"center\">\n");
    myputs
	("<input class=\"params_button\" type=\"submit\" name=\"rst\" value=\"");
    myputs (params->label_button_reset);
    myputs ("\" onClick=\"formResetAll();return true;\">\n");
    myputs
	("<input class=\"params_button\" type=\"submit\" name=\"go\" value=\"");
    myputs (params->label_button_go);
    myputs ("\" onClick=\"formResults();return true;\">\n");
    myputs
	("<input class=\"params_button\" type=\"submit\" name=\"gu\" value=\"");
    myputs (params->label_button_get_url);
    myputs ("\" onClick=\"formResults();return true;\">\n");
    myputs ("</td></tr>\n");

/* closing both <TABLE> and <FORM> */
    myputs ("</table>\n");
    myputs ("</form>\n");

/* HTML footer */
    path = sqlite3_mprintf ("%s/params_footer.inc", params->include_dir);
    cgi_include (path);
    sqlite3_free (path);
}

static void
do_output_functions (struct config *params, int function)
{
/* populating the aggregate functions list */
    char *option;
    if (function == FUNCT_NONE)
      {
	  /* no currently selected Function */
	  option =
	      sqlite3_mprintf
	      ("<option selected=\"1\" value=\"any\">%s</option>\n",
	       params->label_function_none);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"sum\">%s</option>\n",
			       params->label_function_sum);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"avg\">%s</option>\n",
			       params->label_function_avg);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"count\">%s</option>\n",
			       params->label_function_count);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"min\">%s</option>\n",
			       params->label_function_min);
	  myputs (option);
	  sqlite3_free (option);
	  option =
	      sqlite3_mprintf ("<option value=\"max\">%s</option>\n",
			       params->label_function_max);
	  myputs (option);
	  sqlite3_free (option);
      }
    else
      {
	  /* there is a currently selected Function */
	  option =
	      sqlite3_mprintf
	      ("<option value=\"any\">%s</option>\n",
	       params->label_function_none);
	  myputs (option);
	  sqlite3_free (option);
	  if (function == FUNCT_SUM)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"sum\">%s</option>\n",
		   params->label_function_sum);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"sum\">%s</option>\n",
				   params->label_function_sum);
	  myputs (option);
	  sqlite3_free (option);
	  if (function == FUNCT_AVG)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"avg\">%s</option>\n",
		   params->label_function_avg);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"avg\">%s</option>\n",
				   params->label_function_avg);
	  myputs (option);
	  sqlite3_free (option);
	  if (function == FUNCT_COUNT)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"count\">%s</option>\n",
		   params->label_function_count);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"count\">%s</option>\n",
				   params->label_function_count);
	  myputs (option);
	  sqlite3_free (option);
	  if (function == FUNCT_MIN)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"min\">%s</option>\n",
		   params->label_function_min);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"min\">%s</option>\n",
				   params->label_function_min);
	  myputs (option);
	  sqlite3_free (option);
	  if (function == FUNCT_MAX)
	      option =
		  sqlite3_mprintf
		  ("<option selected=\"1\" value=\"max\">%s</option>\n",
		   params->label_function_max);
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"max\">%s</option>\n",
				   params->label_function_max);
	  myputs (option);
	  sqlite3_free (option);
      }
}

static void
do_output_aggregate_frame (struct config *params)
{
/* will return an HTML page defining Query params (Aggregate Query) */
    int ret;
    const char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    char *path;
    char *option;
    struct groupby *groupby;
    struct aggregate *aggregate;
    int disable_compar;

/* HTML header */
    path = sqlite3_mprintf ("%s/params_header.inc", params->include_dir);
    cgi_include (path);
    sqlite3_free (path);

/* creating the <FORM> */
    if (params->http_request_method == METHOD_POST)
	myputs ("<form method=\"POST\" action=\"");
    else
	myputs ("<form method=\"GET\" action=\"");
    myputs (params->cgi_url);
    myputs ("\" target=\"params\">\n");

/* appending all HIDDEN <INPUT>s */
    myputs ("<input type=\"hidden\" name=\"in\" id=\"in\" value=\"");
    myputs (params->instance_name);
    myputs ("\">\n");
    myputs ("<input type=\"hidden\" name=\"cgi_url\" id=\"cgi_url\" value=\"");
    myputs (params->cgi_url);
    myputs ("\">\n");
    if (!check_multiple_views (params))
      {
	  /* there is only a single View being published */
	  char *name = get_unique_view_name (params);
	  set_current_view (params, name);
	  option =
	      sqlite3_mprintf
	      ("<input type=\"hidden\" name=\"cvw\" value=\"%s\">\n", name);
	  free (name);
	  myputs (option);
	  sqlite3_free (option);
      }
    else
      {
	  myputs ("<input type=\"hidden\" name=\"cvw\" id=\"view\" value=\"");
	  if (params->arg_view != NULL)
	      myputs (params->arg_view);
	  else
	      myputs (params->label_not_selected);
	  myputs ("\">\n");
      }
    myputs ("<input type=\"hidden\" name=\"cqy\" value=\"aggregate\">\n");
    option =
	sqlite3_mprintf
	("<input type=\"hidden\" name=\"cntg\" id=\"cntg\" value=\"%d\">\n",
	 params->arg_count_groupby);
    myputs (option);
    sqlite3_free (option);
    option =
	sqlite3_mprintf
	("<input type=\"hidden\" name=\"cnta\" value=\"%d\">\n",
	 params->arg_count_aggregate);
    myputs (option);
    sqlite3_free (option);
    if (!check_multiple_views (params))
      {
	  /* there is only a single View being published */
	  char *name = get_unique_view_name (params);
	  set_current_view (params, name);
	  option =
	      sqlite3_mprintf
	      ("<input type=\"hidden\" name=\"vw\" id=\"view\" value=\"%s\">\n",
	       name);
	  free (name);
	  myputs (option);
	  sqlite3_free (option);
      }

/* creating the <TABLE> */
    myputs ("<table class=\"params_table\">\n");

    if (check_multi_instances (params))
      {
	  /* HTML published instances */
	  struct instance *in;
	  myputs ("<tr><td class=\"params_label\">");
	  myputs (params->label_instance);
	  myputs ("</td><td class=\"params_background\" colspan=\"6\">\n");
	  myputs ("<select class=\"params_combo\" name=\"muin\" id=\"muin\" ");
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  in = params->first_instance;
	  while (in != NULL)
	    {
		if (params->instance_name != NULL)
		  {
		      /* there is a currently selected Instance */
		      if (strcasecmp (params->instance_name, in->name) == 0)
			{
			    option =
				sqlite3_mprintf
				("<option selected=\"1\" value=\"%s\">%s</option>\n",
				 in->name, in->title);
			}
		      else
			  option =
			      sqlite3_mprintf
			      ("<option value=\"%s\">%s</option>\n", in->name,
			       in->title);
		  }
		else
		    option =
			sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
					 in->name, in->name);
		myputs (option);
		in = in->next;
	    }
	  myputs ("</select></td></tr>\n");
      }

    if (check_multiple_views (params))
      {
	  /* HTML published Views */
	  myputs ("<tr><td class=\"params_label\">");
	  myputs (params->label_datasource);
	  myputs ("</td><td class=\"params_background\" colspan=\"7\">\n");
	  myputs ("<select class=\"params_combo\" name=\"vw\" id=\"view\" ");
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  if (params->arg_view == NULL)
	      do_view_autoselect (params);
	  if (params->arg_view == NULL)
	    {
		/* there is no currently selected View Name */
		option =
		    sqlite3_mprintf
		    ("<option selected=\"1\" value=\"\">%s</option>\n",
		     params->label_not_selected);
		set_current_view (params, NULL);
	    }
	  else
	      option =
		  sqlite3_mprintf ("<option value=\"\">%s</option>\n",
				   params->label_not_selected);
	  myputs (option);
	  sqlite3_free (option);

	  sql =
	      "SELECT name FROM sqlite_master WHERE type = 'view' ORDER BY name";
	  ret =
	      sqlite3_get_table (params->db_handle, sql, &results, &rows,
				 &columns, NULL);
	  if (ret != SQLITE_OK)
	      goto skip_views;

	  for (i = 1; i <= rows; i++)
	    {
		const char *name = results[(i * columns) + 0];
		if (test_hidden_view (params, name) == 1)
		    continue;
		if (params->arg_view != NULL)
		  {
		      /* there is a currently selected View Name */
		      if (strcasecmp (params->arg_view, name) == 0)
			{
			    option =
				sqlite3_mprintf
				("<option selected=\"1\" value=\"%s\">%s</option>\n",
				 name, name);
			    set_current_view (params, name);
			}
		      else
			  option =
			      sqlite3_mprintf
			      ("<option value=\"%s\">%s</option>\n", name,
			       name);
		  }
		else
		    option =
			sqlite3_mprintf ("<option value=\"%s\">%s</option>\n",
					 name, name);
		myputs (option);
		sqlite3_free (option);
	    }
	  sqlite3_free_table (results);
	  myputs ("</select></td></tr>\n");
      }

  skip_views:
/* HTML query mode */
    myputs ("<tr><td class=\"params_label\">");
    myputs (params->label_query_mode);
    myputs ("</td><td class=\"params_background\" colspan=\"7\">\n");
    myputs ("<select class=\"params_combo\" name=\"q\" ");
    myputs ("onChange=\"formUpdate();return true;\">\n");
    option =
	sqlite3_mprintf
	("<option value=\"filter\">%s</option>\n", params->label_mode_filter);
    myputs (option);
    sqlite3_free (option);
    option =
	sqlite3_mprintf
	("<option selected=\"1\" value=\"aggregate\">%s</option>\n",
	 params->label_mode_aggregate);
    myputs (option);
    sqlite3_free (option);
    myputs ("</select></td></tr>\n");

    groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  /* HTML group by */
	  char *str;
	  myputs ("<tr><td class=\"params_label\">");
	  if (groupby->groupby_index == 0)
	      myputs (params->label_group_by);
	  else
	      myputs (params->label_and_group_by);
	  myputs ("</td><td class=\"params_background\">\n");
	  str =
	      sqlite3_mprintf
	      ("<select class=\"params_combo\" name=\"gc%d\" id=\"gc%d\" ",
	       groupby->groupby_index, groupby->groupby_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  do_output_columns (params, groupby->groupby_column);
	  myputs ("</select></td>\n");
	  myputs ("<td class=\"params_background\">\n");
	  str =
	      sqlite3_mprintf
	      ("<select class=\"params_combo\" name=\"gx%d\" id=\"gx%d\" ",
	       groupby->groupby_index, groupby->groupby_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  disable_compar =
	      do_output_comparisons (params, groupby->groupby_comparison);
	  myputs ("</select></td>\n");
	  myputs ("<td class=\"params_background\">\n");
	  if (disable_compar || groupby->groupby_column == NULL)
	    {
		myputs ("<img class=\"icon_disabled\" src=\"");
		myputs (params->icon_help_url);
		myputs ("\" alt=\"help\"></td>\n");
	    }
	  else
	    {
		str =
		    sqlite3_mprintf
		    ("<a href=\"#\" onClick=\"searchFieldValueAggregate("
		     "'gc%d','gv%d'); return true;\">\n",
		     groupby->groupby_index, groupby->groupby_index);
		myputs (str);
		sqlite3_free (str);
		myputs ("<img src=\"");
		myputs (params->icon_help_url);
		myputs ("\" alt=\"help\"></a></td>\n");
	    }
	  myputs ("<td class=\"params_background\">\n");
	  if (disable_compar || groupby->groupby_column == NULL)
	    {
		str =
		    sqlite3_mprintf
		    ("<input class=\"params_input_disabled\" type=\"text\" readonly=\"readonly\" "
		     "name=\"gv%d\" id=\"gv%d\" size=\"30\" value=\"%s\"></td>\n",
		     groupby->groupby_index, groupby->groupby_index,
		     groupby->groupby_value);
	    }
	  else if (groupby->groupby_value != NULL)
	    {
		str =
		    sqlite3_mprintf
		    ("<input class=\"params_input\" type=\"text\" "
		     "name=\"gv%d\" id=\"gv%d\" size=\"30\" value=\"%s\"></td>\n",
		     groupby->groupby_index, groupby->groupby_index,
		     groupby->groupby_value);
	    }
	  else
	      str =
		  sqlite3_mprintf
		  ("<input class=\"params_input\" type=\"text\" "
		   "name=\"gv%d\" id=\"gv%d\"size=\"30\"></td>\n",
		   groupby->groupby_index, groupby->groupby_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("<td class=\"params_background\">\n");
	  if (groupby->next == NULL)
	    {
		/* the ADD-GROUP BY button only if this is the last Group By clause */
		myputs ("<input type=\"image\" name=\"addg\" src=\"");
		myputs (params->icon_add_url);
		myputs ("\" onClick=\"formUpdate();return true;\">\n");
	    }
	  myputs ("</td><td class=\"params_fill\"></td></tr>\n");
	  groupby = groupby->next;
      }

    aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  /* HTML aggregated columns */
	  char *str;
	  myputs ("<tr><td class=\"params_label\">");
	  if (aggregate->aggregate_index == 0)
	      myputs (params->label_aggregate);
	  else
	      myputs (params->label_and_aggregate);
	  myputs ("</td><td class=\"params_background\">\n");
	  str =
	      sqlite3_mprintf
	      ("<select class=\"params_combo\" name=\"ac%d\" ",
	       aggregate->aggregate_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  do_output_columns (params, aggregate->aggregate_column);
	  myputs ("</select></td>\n");
	  myputs ("<td class=\"params_background\">\n");
	  str =
	      sqlite3_mprintf
	      ("<select class=\"params_combo\" name=\"af%d\" ",
	       aggregate->aggregate_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  do_output_functions (params, aggregate->aggregate_function);
	  myputs ("</select></td>\n");
	  myputs ("<td class=\"params_background_right\" colspan=\"2\">\n");
	  str =
	      sqlite3_mprintf
	      ("<select class=\"params_combo\" name=\"hc%d\" ",
	       aggregate->aggregate_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("onChange=\"formUpdate();return true;\">\n");
	  disable_compar =
	      do_output_comparisons (params, aggregate->having_comparison);
	  myputs ("</select></td>\n");
	  myputs ("<td class=\"params_background\">\n");
	  if (disable_compar || aggregate->aggregate_column == NULL)
	      str =
		  sqlite3_mprintf
		  ("<input class=\"params_input_disabled\" type=\"text\" readonly=\"readonly\" "
		   "name=\"hv%d\" size=\"30\" value=\"%s\"></td>\n",
		   aggregate->aggregate_index, aggregate->having_value);
	  else if (aggregate->having_value != NULL)
	      str =
		  sqlite3_mprintf
		  ("<input class=\"params_input\" type=\"text\" "
		   "name=\"hv%d\" size=\"30\" value=\"%s\"></td>\n",
		   aggregate->aggregate_index, aggregate->having_value);
	  else
	      str =
		  sqlite3_mprintf
		  ("<input class=\"params_input\" type=\"text\" "
		   "name=\"hv%d\" size=\"30\"></td>\n",
		   aggregate->aggregate_index);
	  myputs (str);
	  sqlite3_free (str);
	  myputs ("<td class=\"params_background\">\n");
	  if (aggregate->next == NULL)
	    {
		/* the ADD-AGGREGATE button only if this is the last Aggregate clause */
		myputs ("<input type=\"image\" name=\"adda\" src=\"");
		myputs (params->icon_add_url);
		myputs ("\" onClick=\"formUpdate();return true;\">\n");
	    }
	  myputs ("</td><td class=\"params_fill\"></td></tr>\n");
	  aggregate = aggregate->next;
      }

/* HTML output format */
    myputs ("<tr><td class=\"params_label\">");
    myputs (params->label_output_format);
    myputs ("</td><td class=\"params_background\" colspan=\"7\">\n");
    myputs ("<select class=\"params_combo\" name=\"fmt\" ");
    myputs ("onChange=\"formUpdate();return true;\">\n");
    do_output_format (params);
    myputs ("</select></td></tr>\n");

/* HTML form buttons */
    myputs ("<tr><td class=\"params_background_center\" colspan=\"8\">\n");
    myputs
	("<input class=\"params_button\" type=\"submit\" name=\"rst\" value=\"");
    myputs (params->label_button_reset);
    myputs ("\" onClick=\"formResetAll();return true;\">\n");
    myputs
	("<input class=\"params_button\" type=\"submit\" name=\"go\" value=\"");
    myputs (params->label_button_go);
    myputs ("\" onClick=\"formResults();return true;\">\n");
    myputs
	("<input class=\"params_button\" type=\"submit\" name=\"gu\" value=\"");
    myputs (params->label_button_get_url);
    myputs ("\" onClick=\"formResults();return true;\">\n");
    myputs ("</td></tr>\n");

/* closing both <TABLE> and <FORM> */
    myputs ("</table>\n");
    myputs ("</form>\n");

/* HTML footer */
    path = sqlite3_mprintf ("%s/params_footer.inc", params->include_dir);
    cgi_include (path);
    sqlite3_free (path);
}

static void
do_output_params_frame (struct config *params)
{
/* will return an HTML page defining Query params */
    if (!is_filter_query (params))
      {
	  /* Aggregate Query */
	  do_output_aggregate_frame (params);
	  return;
      }
/* Filter Query */
    do_output_filter_frame (params);
}

static char *
get_current_timestamp (struct config *params)
{
/* will return the current time */
    int ret;
    char **results;
    int rows;
    int columns;
    int i;
    char *timestamp = NULL;

    if (params->timestamps_localtime)
	ret =
	    sqlite3_get_table (params->db_handle,
			       "SELECT datetime('now', 'localtime')", &results,
			       &rows, &columns, NULL);
    else
	ret =
	    sqlite3_get_table (params->db_handle, "SELECT datetime('now')",
			       &results, &rows, &columns, NULL);
    if (ret != SQLITE_OK)
	goto deflt;

    for (i = 1; i <= rows; i++)
      {
	  int len;
	  const char *str = results[(i * columns) + 0];
	  len = strlen (str);
	  timestamp = malloc (len + 1);
	  strcpy (timestamp, str);
      }
    sqlite3_free_table (results);
    if (timestamp == NULL)
	goto deflt;
    return timestamp;

  deflt:
    timestamp = malloc (20);
    strcpy (timestamp, "0000-00-00 00:00:00");
    return timestamp;
}

static char *
do_prepare_filtered_sql (struct config *params)
{
/* will create an SQL Query statement of the Filter type */
    char *sql;
    char *quoted;
    struct filter *filter;
    struct orderby *orderby;
    int first = 1;

/* preparing the SQL statement */
    quoted = gaiaDoubleQuotedSql (params->arg_view);
    if (params->arg_select != NULL)
	sql =
	    sqlite3_mprintf ("SELECT %s FROM \"%s\"", params->arg_select,
			     quoted);
    else
	sql = sqlite3_mprintf ("SELECT * FROM \"%s\"", quoted);
    free (quoted);

    filter = params->first_filter;
    while (filter != NULL)
      {
	  /* adding filters if required */
	  if (filter->filter_column != NULL
	      && filter->filter_comparison != COMPAR_NONE)
	    {
		int needs_value = 1;
		int starts = 0;
		int ends = 0;
		char *save_sql = sql;
		const char *compar = "";
		switch (filter->filter_comparison)
		  {
		  case COMPAR_EQ:
		      compar = "=";
		      break;
		  case COMPAR_NE:
		      compar = "<>";
		      break;
		  case COMPAR_GT:
		      compar = ">";
		      break;
		  case COMPAR_GE:
		      compar = ">=";
		      break;
		  case COMPAR_LT:
		      compar = "<";
		      break;
		  case COMPAR_LE:
		      compar = "<=";
		      break;
		  case COMPAR_LIKE:
		      compar = "LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_STARTS:
		      compar = "LIKE";
		      starts = 1;
		      break;
		  case COMPAR_ENDS:
		      compar = "LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NOT_LIKE:
		      compar = "NOT LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_NOT_STARTS:
		      compar = "NOT LIKE";
		      starts = 1;
		      break;
		  case COMPAR_NOT_ENDS:
		      compar = "NOT LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NULL:
		      compar = "IS NULL";
		      needs_value = 0;
		      break;
		  case COMPAR_NOT_NULL:
		      compar = "IS NOT NULL";
		      needs_value = 0;
		      break;
		  };
		if (needs_value && filter->filter_value == NULL)
		  {
		      /* skipping an incomplete filter */
		      filter = filter->next;
		      continue;
		  }
		quoted = gaiaDoubleQuotedSql (filter->filter_column);
		if (needs_value)
		  {
		      /* this comparison operator requires a value */
		      if (starts || ends)
			{
			    /* some special value is required (LIKE something) */
			    char *value;
			    if (!starts && ends)
				value =
				    sqlite3_mprintf ("%%%s",
						     filter->filter_value);
			    else if (starts && !ends)
				value =
				    sqlite3_mprintf ("%s%%",
						     filter->filter_value);
			    else
				value =
				    sqlite3_mprintf ("%%%s%%",
						     filter->filter_value);
			    if (first)
			      {
				  /* first filter */
				  sql =
				      sqlite3_mprintf ("%s WHERE \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
				  first = 0;
			      }
			    else
			      {
				  /* any other subsequent filter */
				  sql =
				      sqlite3_mprintf ("%s AND \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
			      }
			    sqlite3_free (value);
			}
		      else
			{
			    /* an ordinary value is required */
			    if (first)
			      {
				  /* first filter */
				  sql =
				      sqlite3_mprintf ("%s WHERE \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       filter->filter_value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
				  first = 0;
			      }
			    else
			      {
				  /* any other subsequent filter */
				  sql =
				      sqlite3_mprintf ("%s AND \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       filter->filter_value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
			      }
			}
		  }
		else
		  {
		      /* no value is required by this comparison oparator */
		      if (first)
			{
			    /* first filter */
			    sql =
				sqlite3_mprintf ("%s WHERE \"%s\" %s", save_sql,
						 quoted, compar);
			    sqlite3_free (save_sql);
			    save_sql = sql;
			    first = 0;
			}
		      else
			{
			    /* any other subsequent filter */
			    sql =
				sqlite3_mprintf ("%s AND \"%s\" %s", save_sql,
						 quoted, compar);
			    sqlite3_free (save_sql);
			    save_sql = sql;
			}
		  }
		free (quoted);
	    }
	  filter = filter->next;
      }

    first = 1;
    orderby = params->first_orderby;
    while (orderby != NULL)
      {
	  /* adding ORDER BY clauses if required */
	  if (orderby->orderby_column != NULL)
	    {
		char *save_sql = sql;
		quoted = gaiaDoubleQuotedSql (orderby->orderby_column);
		if (first)
		  {
		      /* first ORDER BY column */
		      sql =
			  sqlite3_mprintf ("%s ORDER BY \"%s\"", save_sql,
					   quoted);
		      sqlite3_free (save_sql);
		      first = 0;
		  }
		else
		  {
		      /* any other subsequent ORDER BY column */
		      sql = sqlite3_mprintf ("%s, \"%s\"", save_sql, quoted);
		      sqlite3_free (save_sql);
		  }
		free (quoted);
	    }
	  orderby = orderby->next;
      }
    return sql;
}

static char *
create_columns_list (struct config *params)
{
/* creating the columns list for Aggregate SQL query */
    char *list = NULL;
    char *save_list;
    int first = 1;
    char *colname;
    char *quoted;
    struct groupby *groupby;
    struct aggregate *aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  /* resetting all alias names */
	  if (aggregate->aggregate_alias != NULL)
	    {
		sqlite3_free (aggregate->aggregate_alias);
		aggregate->aggregate_alias = NULL;
	    }
	  aggregate = aggregate->next;
      }

/* retrieving View's columns - GROUP BY columns */
    groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  quoted = gaiaDoubleQuotedSql (groupby->groupby_column);
	  if (first)
	    {
		colname = sqlite3_mprintf ("\"%s\"", quoted);
		first = 0;
	    }
	  else
	      colname = sqlite3_mprintf (", \"%s\"", quoted);
	  free (quoted);
	  if (list == NULL)
	      list = sqlite3_mprintf ("%s", colname);
	  else
	    {
		save_list = list;
		list = sqlite3_mprintf ("%s%s", save_list, colname);
		sqlite3_free (save_list);
	    }
	  sqlite3_free (colname);
	  groupby = groupby->next;
      }

/* retrieving View's columns - Aggregate columns */
    aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  if (aggregate->aggregate_function != FUNCT_NONE)
	    {
		/* applying an aggregate function to this column */
		const char *function = "";
		switch (aggregate->aggregate_function)
		  {
		  case FUNCT_SUM:
		      function = "sum";
		      break;
		  case FUNCT_AVG:
		      function = "avg";
		      break;
		  case FUNCT_COUNT:
		      function = "count";
		      break;
		  case FUNCT_MIN:
		      function = "min";
		      break;
		  case FUNCT_MAX:
		      function = "max";
		      break;
		  };
		quoted = gaiaDoubleQuotedSql (aggregate->aggregate_column);
		if (first)
		  {
		      colname =
			  sqlite3_mprintf ("%s(\"%s\") AS \"%s(%s)\"", function,
					   quoted, function,
					   aggregate->aggregate_column);
		      first = 0;
		  }
		else
		    colname =
			sqlite3_mprintf (", %s(\"%s\") AS \"%s(%s)\"", function,
					 quoted, function,
					 aggregate->aggregate_column);
		if (aggregate->aggregate_alias != NULL)
		    sqlite3_free (aggregate->aggregate_alias);
		aggregate->aggregate_alias =
		    sqlite3_mprintf ("%s(%s)", function,
				     aggregate->aggregate_column);
		free (quoted);
	    }
	  else
	    {
		/* plain column - no aggregate function */
		quoted = gaiaDoubleQuotedSql (aggregate->aggregate_column);
		if (first)
		  {
		      colname = sqlite3_mprintf ("\"%s\"", quoted);
		      first = 0;
		  }
		else
		    colname = sqlite3_mprintf (", \"%s\"", quoted);
		if (aggregate->aggregate_alias != NULL)
		    sqlite3_free (aggregate->aggregate_alias);
		aggregate->aggregate_alias =
		    sqlite3_mprintf ("%s", aggregate->aggregate_column);
		free (quoted);
	    }
	  if (list == NULL)
	      list = sqlite3_mprintf ("%s", colname);
	  else
	    {
		save_list = list;
		list = sqlite3_mprintf ("%s%s", save_list, colname);
		sqlite3_free (save_list);
	    }
	  sqlite3_free (colname);
	  aggregate = aggregate->next;
      }
    return list;
}

static void
do_output_field_help (struct config *params)
{
/* will create a popup HTML page supporting Field Help */
    char *sql;
    char *quoted1;
    char *quoted2;
    int ret;
    sqlite3_stmt *stmt;
    int nrow = 0;
    int truncation = 0;
    struct filter *filter;
    char *where = NULL;
    char *item;

/* HTML header */
    myputs
	("<!DOCTYPE html PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n");
    myputs ("<html>\n<head>\n");
    myputs
	("<meta http-equiv='content-type' content='text/html; charset=UTF-8'>\n");
    myputs ("<title>dataSeltzer Field Help</title>\n");
    myputs ("<link rel=\"stylesheet\" type=\"text/css\" href=\"");
    myputs (params->css_stylesheet_url);
    myputs ("\">\n");
    myputs ("<script language=\"javascript\">\n\n");
    myputs ("function setFieldValue(value)\n");
    myputs ("{\n// setting a Filed Value\n");
    myputs ("\topener.document.getElementById('");
    myputs (params->field);
    myputs ("').value = value;\n");
    myputs ("\twindow.close();\n");
    myputs ("}\n\n");
    myputs ("</script>\n");
    myputs ("</head>\n<body>\n");
    myputs ("<h1>");
    if (params->arg_view == NULL)
	myputs ("NULL");
    else
	myputs (params->arg_view);
    myputs ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
    if (params->column == NULL)
	myputs ("NULL");
    else
	myputs (params->column);
    myputs ("</h1>\n<ul>\n");
    filter = params->first_filter;
    while (filter != NULL)
      {
	  /* adding filters if required */
	  if (filter->filter_column != NULL
	      && filter->filter_comparison != COMPAR_NONE)
	    {

		int needs_value = 1;
		int starts = 0;
		int ends = 0;
		const char *compar = "";
		const char *sql_compar = "";
		char *value;
		if (strcasecmp (filter->filter_column, params->column) == 0)
		  {
		      /* skipping the target column itself */
		      filter = filter->next;
		      continue;
		  }
		switch (filter->filter_comparison)
		  {
		  case COMPAR_EQ:
		      compar = params->label_equal;
		      sql_compar = "=";
		      break;
		  case COMPAR_NE:
		      compar = params->label_not_equal;
		      sql_compar = "<>";
		      break;
		  case COMPAR_GT:
		      compar = params->label_greater;
		      sql_compar = ">";
		      break;
		  case COMPAR_GE:
		      compar = params->label_greater_equal;
		      sql_compar = ">=";
		      break;
		  case COMPAR_LT:
		      compar = params->label_lesser;
		      sql_compar = "<";
		      break;
		  case COMPAR_LE:
		      compar = params->label_lesser_equal;
		      sql_compar = "<=";
		      break;
		  case COMPAR_LIKE:
		      compar = params->label_like;
		      sql_compar = "LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_STARTS:
		      compar = params->label_starts_with;
		      sql_compar = "LIKE";
		      starts = 1;
		      break;
		  case COMPAR_ENDS:
		      compar = params->label_ends_with;
		      sql_compar = "LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NOT_LIKE:
		      compar = params->label_not_like;
		      sql_compar = "NOT LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_NOT_STARTS:
		      compar = params->label_not_starts_with;
		      sql_compar = "NOT LIKE";
		      starts = 1;
		      break;
		  case COMPAR_NOT_ENDS:
		      compar = params->label_not_ends_with;
		      sql_compar = "NOT LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NULL:
		      compar = params->label_is_null;
		      sql_compar = "IS NULL";
		      needs_value = 0;
		      break;
		  case COMPAR_NOT_NULL:
		      compar = params->label_is_not_null;
		      sql_compar = "IS NOT NULL";
		      needs_value = 0;
		      break;
		  };
		if (needs_value && filter->filter_value == NULL)
		  {
		      /* skipping an incomplete filter */
		      filter = filter->next;
		      continue;
		  }
		if (needs_value)
		    item =
			sqlite3_mprintf ("<li>%s %s %s</li>\n",
					 filter->filter_column, compar,
					 filter->filter_value);
		else
		    item =
			sqlite3_mprintf ("<li>%s %s</li>\n",
					 filter->filter_column, compar);
		myputs (item);
		sqlite3_free (item);
		if (where == NULL)
		  {
		      if (needs_value)
			{
			    if (starts && ends)
			      {
				  value =
				      sqlite3_mprintf ("%%%s%%",
						       filter->filter_value);
				  where =
				      sqlite3_mprintf (" %s %s %Q",
						       filter->filter_column,
						       sql_compar, value);
				  sqlite3_free (value);
			      }
			    else if (starts)
			      {
				  value =
				      sqlite3_mprintf ("%s%%",
						       filter->filter_value);
				  where =
				      sqlite3_mprintf (" %s %s %Q",
						       filter->filter_column,
						       sql_compar, value);
				  sqlite3_free (value);
			      }
			    else if (ends)
			      {
				  value =
				      sqlite3_mprintf ("%%%s",
						       filter->filter_value);
				  where =
				      sqlite3_mprintf (" %s %s %Q",
						       filter->filter_column,
						       sql_compar, value);
				  sqlite3_free (value);
			      }
			    else
				where =
				    sqlite3_mprintf (" %s %s %Q",
						     filter->filter_column,
						     sql_compar,
						     filter->filter_value);
			}
		      else
			  where =
			      sqlite3_mprintf (" %s %s", filter->filter_column,
					       sql_compar);
		  }
		else
		  {
		      char *old_where = where;
		      if (needs_value)
			{
			    if (starts && ends)
			      {
				  value =
				      sqlite3_mprintf ("%%%s%%",
						       filter->filter_value);
				  where =
				      sqlite3_mprintf ("%s AND %s %s %Q",
						       old_where,
						       filter->filter_column,
						       sql_compar, value);
				  sqlite3_free (value);
			      }
			    else if (starts)
			      {
				  value =
				      sqlite3_mprintf ("%s%%",
						       filter->filter_value);
				  where =
				      sqlite3_mprintf ("%s AND %s %s %Q",
						       old_where,
						       filter->filter_column,
						       sql_compar, value);
				  sqlite3_free (value);
			      }
			    else if (ends)
			      {
				  value =
				      sqlite3_mprintf ("%%%s",
						       filter->filter_value);
				  where =
				      sqlite3_mprintf ("%s AND %s %s %Q",
						       old_where,
						       filter->filter_column,
						       sql_compar, value);
				  sqlite3_free (value);
			      }
			    else
				where =
				    sqlite3_mprintf ("%s AND %s %s %Q",
						     old_where,
						     filter->filter_column,
						     sql_compar,
						     filter->filter_value);
			}
		      else
			  where =
			      sqlite3_mprintf ("%s AND %s %s", old_where,
					       old_where, filter->filter_column,
					       sql_compar);
		      sqlite3_free (old_where);
		  }
	    }
	  filter = filter->next;
      }
    myputs ("</ul>\n<h2>");
    myputs (params->label_help_header);
    myputs ("</h2>\n");

/* creating the <UL> container */
    myputs ("<table class=\"results_table\">\n");

/* preparing the SQL statement */
    quoted1 = gaiaDoubleQuotedSql (params->column);
    quoted2 = gaiaDoubleQuotedSql (params->arg_view);
    if (where == NULL)
	sql =
	    sqlite3_mprintf ("SELECT Count(*), \"%s\" FROM \"%s\" "
			     "GROUP BY \"%s\" ORDER BY \"%s\"",
			     quoted1, quoted2, quoted1, quoted1);
    else
      {
	  sql =
	      sqlite3_mprintf ("SELECT Count(*), \"%s\" FROM \"%s\" "
			       "WHERE %s GROUP BY \"%s\" ORDER BY \"%s\"",
			       quoted1, quoted2, where, quoted1, quoted1);
	  sqlite3_free (where);
      }
    free (quoted1);
    free (quoted2);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	goto stop;
    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* Column Values */
		const char *text_value;
		sqlite3_int64 int_value;
		double dbl_value;
		int type = sqlite3_column_type (stmt, 1);
		const char *count_class = "result_num_even_row";
		const char *label_class = "result_even_row";
		int count = sqlite3_column_int (stmt, 0);
		if (params->max_help_resultset_rows > 0
		    && nrow > params->max_help_resultset_rows)
		  {
		      truncation = 1;
		      break;
		  }
		if ((nrow % 2) == 1)
		  {
		      count_class = "result_num_odd_row";
		      label_class = "result_odd_row";
		  }
		switch (type)
		  {
		  case SQLITE_TEXT:
		      text_value = (const char *) sqlite3_column_text (stmt, 1);
		      item =
			  sqlite3_mprintf
			  ("<tr><td class=\"%s\">(%d)</td><td class=\"%s\"><a href=\"#\" onClick=\"setFieldValue('%s');"
			   "return true;\">%s</a></td></tr>\n", count_class,
			   count, label_class, text_value, text_value);
		      myputs (item);
		      sqlite3_free (item);
		      nrow++;
		      break;
		  case SQLITE_INTEGER:
		      int_value = sqlite3_column_int64 (stmt, 1);
		      item =
			  sqlite3_mprintf
			  ("<tr><td class=\"%s\">(%d)</td><td class=\"%s\"><a href=\"#\" onClick=\"setFieldValue(%lld);"
			   "return true;\">%lld</a></td></tr>\n", count_class,
			   count, label_class, int_value, int_value);
		      myputs (item);
		      sqlite3_free (item);
		      nrow++;
		      break;
		  case SQLITE_FLOAT:
		      dbl_value = sqlite3_column_double (stmt, 1);
		      item =
			  sqlite3_mprintf
			  ("<tr><td class=\"%s\">(%d)</td><td class=\"%s\"><a href=\"#\" onClick=\"setFieldValue(%1.5f);"
			   "return true;\">%1.5f</a></td></tr>\n", count_class,
			   count, label_class, dbl_value, dbl_value);
		      myputs (item);
		      sqlite3_free (item);
		      nrow++;
		      break;
		  };
	    }
	  else
	      break;
      }
    sqlite3_finalize (stmt);

  stop:
/* HTML footer */
    myputs ("</table>\n");
    if (truncation)
      {
	  myputs ("<h1>");
	  myputs (params->label_help_truncation);
	  myputs ("</h1>\n");
      }
    myputs ("</body>\n</html>\n");
}

static char *
do_prepare_aggregated_sql (struct config *params)
{
/* will create an SQL Query statement of the Aggregate type */
    char *sql;
    char *quoted;
    char *columns;
    struct groupby *groupby;
    struct aggregate *aggregate;
    int first = 1;

/* preparing the SQL statement */
    quoted = gaiaDoubleQuotedSql (params->arg_view);
    columns = create_columns_list (params);
    sql = sqlite3_mprintf ("SELECT %s FROM \"%s\"", columns, quoted);
    free (quoted);
    sqlite3_free (columns);

    groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  /* adding WHERE filters if required */
	  if (groupby->groupby_column != NULL
	      && groupby->groupby_comparison != COMPAR_NONE)
	    {
		int needs_value = 1;
		int starts = 0;
		int ends = 0;
		char *save_sql = sql;
		const char *compar = "";
		switch (groupby->groupby_comparison)
		  {
		  case COMPAR_EQ:
		      compar = "=";
		      break;
		  case COMPAR_NE:
		      compar = "<>";
		      break;
		  case COMPAR_GT:
		      compar = ">";
		      break;
		  case COMPAR_GE:
		      compar = ">=";
		      break;
		  case COMPAR_LT:
		      compar = "<";
		      break;
		  case COMPAR_LE:
		      compar = "<=";
		      break;
		  case COMPAR_LIKE:
		      compar = "LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_STARTS:
		      compar = "LIKE";
		      starts = 1;
		      break;
		  case COMPAR_ENDS:
		      compar = "LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NOT_LIKE:
		      compar = "NOT LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_NOT_STARTS:
		      compar = "NOT LIKE";
		      starts = 1;
		      break;
		  case COMPAR_NOT_ENDS:
		      compar = "NOT LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NULL:
		      compar = "IS NULL";
		      needs_value = 0;
		      break;
		  case COMPAR_NOT_NULL:
		      compar = "IS NOT NULL";
		      needs_value = 0;
		      break;
		  };
		if (needs_value && groupby->groupby_value == NULL)
		  {
		      /* skipping an incomplete filter */
		      groupby = groupby->next;
		      continue;
		  }
		quoted = gaiaDoubleQuotedSql (groupby->groupby_column);
		if (needs_value)
		  {
		      /* this comparison operator requires a value */
		      if (starts || ends)
			{
			    /* some special value is required (LIKE something) */
			    char *value;
			    if (!starts && ends)
				value =
				    sqlite3_mprintf ("%%%s",
						     groupby->groupby_value);
			    else if (starts && !ends)
				value =
				    sqlite3_mprintf ("%s%%",
						     groupby->groupby_value);
			    else
				value =
				    sqlite3_mprintf ("%%%s%%",
						     groupby->groupby_value);
			    if (first)
			      {
				  /* first filter */
				  sql =
				      sqlite3_mprintf ("%s WHERE \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
				  first = 0;
			      }
			    else
			      {
				  /* any other subsequent filter */
				  sql =
				      sqlite3_mprintf ("%s AND \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
			      }
			    sqlite3_free (value);
			}
		      else
			{
			    /* an ordinary value is required */
			    if (first)
			      {
				  /* first filter */
				  sql =
				      sqlite3_mprintf ("%s WHERE \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       groupby->groupby_value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
				  first = 0;
			      }
			    else
			      {
				  /* any other subsequent filter */
				  sql =
				      sqlite3_mprintf ("%s AND \"%s\" %s %Q",
						       save_sql, quoted, compar,
						       groupby->groupby_value);
				  sqlite3_free (save_sql);
				  save_sql = sql;
			      }
			}
		  }
		else
		  {
		      /* no value is required by this comparison oparator */
		      if (first)
			{
			    /* first filter */
			    sql =
				sqlite3_mprintf ("%s WHERE \"%s\" %s", save_sql,
						 quoted, compar);
			    sqlite3_free (save_sql);
			    save_sql = sql;
			    first = 0;
			}
		      else
			{
			    /* any other subsequent filter */
			    sql =
				sqlite3_mprintf ("%s AND \"%s\" %s", save_sql,
						 quoted, compar);
			    sqlite3_free (save_sql);
			    save_sql = sql;
			}
		  }
		free (quoted);
	    }
	  groupby = groupby->next;
      }

    groupby = params->first_groupby;
    first = 1;
    while (groupby != NULL)
      {
	  /* adding GROUP BY clauses if required */
	  if (groupby->groupby_column != NULL)
	    {
		char *save_sql = sql;
		quoted = gaiaDoubleQuotedSql (groupby->groupby_column);
		if (first)
		  {
		      /* first GROUP BY clause */
		      sql =
			  sqlite3_mprintf ("%s GROUP BY \"%s\"", save_sql,
					   quoted);
		      sqlite3_free (save_sql);
		      save_sql = sql;
		      first = 0;
		  }
		else
		  {
		      /* any other subsequent GROUP BY clause */
		      sql = sqlite3_mprintf ("%s, \"%s\"", save_sql, quoted);
		      sqlite3_free (save_sql);
		      save_sql = sql;
		  }
		free (quoted);
	    }
	  groupby = groupby->next;
      }

    aggregate = params->first_aggregate;
    first = 1;
    while (aggregate != NULL)
      {
	  /* adding HAVING filters if required */
	  if (aggregate->aggregate_column != NULL
	      && aggregate->having_comparison != COMPAR_NONE)
	    {
		int needs_value = 1;
		int starts = 0;
		int ends = 0;
		char *save_sql = sql;
		const char *compar = "";
		switch (aggregate->having_comparison)
		  {
		  case COMPAR_EQ:
		      compar = "=";
		      break;
		  case COMPAR_NE:
		      compar = "<>";
		      break;
		  case COMPAR_GT:
		      compar = ">";
		      break;
		  case COMPAR_GE:
		      compar = ">=";
		      break;
		  case COMPAR_LT:
		      compar = "<";
		      break;
		  case COMPAR_LE:
		      compar = "<=";
		      break;
		  case COMPAR_LIKE:
		      compar = "LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_STARTS:
		      compar = "LIKE";
		      starts = 1;
		      break;
		  case COMPAR_ENDS:
		      compar = "LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NOT_LIKE:
		      compar = "NOT LIKE";
		      starts = 1;
		      ends = 1;
		      break;
		  case COMPAR_NOT_STARTS:
		      compar = "NOT LIKE";
		      starts = 1;
		      break;
		  case COMPAR_NOT_ENDS:
		      compar = "NOT LIKE";
		      ends = 1;
		      break;
		  case COMPAR_NULL:
		      compar = "IS NULL";
		      needs_value = 0;
		      break;
		  case COMPAR_NOT_NULL:
		      compar = "IS NOT NULL";
		      needs_value = 0;
		      break;
		  };
		if (needs_value && aggregate->having_value == NULL)
		  {
		      /* skipping an incomplete filter */
		      aggregate = aggregate->next;
		      continue;
		  }
		if (aggregate->aggregate_alias != NULL)
		  {
		      /* special case: aggregate function column */
		      if (needs_value)
			{
			    /* this comparison operator requires a value */
			    if (starts || ends)
			      {
				  /* some special value is required (LIKE something) */
				  char *value;
				  if (!starts && ends)
				      value =
					  sqlite3_mprintf ("%%%s",
							   aggregate->
							   having_value);
				  else if (starts && !ends)
				      value =
					  sqlite3_mprintf ("%s%%",
							   aggregate->
							   having_value);
				  else
				      value =
					  sqlite3_mprintf ("%%%s%%",
							   aggregate->
							   having_value);
				  if (first)
				    {
					/* first filter */
					sql =
					    sqlite3_mprintf
					    ("%s HAVING \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     value);
					sqlite3_free (save_sql);
					save_sql = sql;
					first = 0;
				    }
				  else
				    {
					/* any other subsequent filter */
					sql =
					    sqlite3_mprintf
					    ("%s AND \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     value);
					sqlite3_free (save_sql);
					save_sql = sql;
				    }
				  sqlite3_free (value);
			      }
			    else
			      {
				  /* an ordinary value is required */
				  if (first)
				    {
					/* first filter */
					sql =
					    sqlite3_mprintf
					    ("%s HAVING \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     aggregate->having_value);
					sqlite3_free (save_sql);
					save_sql = sql;
					first = 0;
				    }
				  else
				    {
					/* any other subsequent filter */
					sql =
					    sqlite3_mprintf
					    ("%s AND \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     aggregate->having_value);
					sqlite3_free (save_sql);
					save_sql = sql;
				    }
			      }
			}
		      else
			{
			    /* no value is required by this comparison oparator */
			    if (first)
			      {
				  /* first filter */
				  sql =
				      sqlite3_mprintf ("%s HAVING \"%s\" %s",
						       save_sql,
						       aggregate->
						       aggregate_alias, compar);
				  sqlite3_free (save_sql);
				  save_sql = sql;
				  first = 0;
			      }
			    else
			      {
				  /* any other subsequent filter */
				  sql =
				      sqlite3_mprintf ("%s AND \"%s\" %s",
						       save_sql,
						       aggregate->
						       aggregate_alias, compar);
				  sqlite3_free (save_sql);
				  save_sql = sql;
			      }
			}
		  }
		else
		  {
		      /* an ordinary column */
		      if (needs_value)
			{
			    /* this comparison operator requires a value */
			    if (starts || ends)
			      {
				  /* some special value is required (LIKE something) */
				  char *value;
				  if (!starts && ends)
				      value =
					  sqlite3_mprintf ("%%%s",
							   aggregate->
							   having_value);
				  else if (starts && !ends)
				      value =
					  sqlite3_mprintf ("%s%%",
							   aggregate->
							   having_value);
				  else
				      value =
					  sqlite3_mprintf ("%%%s%%",
							   aggregate->
							   having_value);
				  if (first)
				    {
					/* first filter */
					sql =
					    sqlite3_mprintf
					    ("%s HAVING \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     value);
					sqlite3_free (save_sql);
					save_sql = sql;
					first = 0;
				    }
				  else
				    {
					/* any other subsequent filter */
					sql =
					    sqlite3_mprintf
					    ("%s AND \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     value);
					sqlite3_free (save_sql);
					save_sql = sql;
				    }
				  sqlite3_free (value);
			      }
			    else
			      {
				  /* an ordinary value is required */
				  if (first)
				    {
					/* first filter */
					sql =
					    sqlite3_mprintf
					    ("%s HAVING \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     aggregate->having_value);
					sqlite3_free (save_sql);
					save_sql = sql;
					first = 0;
				    }
				  else
				    {
					/* any other subsequent filter */
					sql =
					    sqlite3_mprintf
					    ("%s AND \"%s\" %s %s", save_sql,
					     aggregate->aggregate_alias, compar,
					     aggregate->having_value);
					sqlite3_free (save_sql);
					save_sql = sql;
				    }
			      }
			}
		      else
			{
			    /* no value is required by this comparison oparator */
			    if (first)
			      {
				  /* first filter */
				  sql =
				      sqlite3_mprintf ("%s HAVING \"%s\" %s",
						       save_sql,
						       aggregate->aggregate_alias,
						       compar);
				  sqlite3_free (save_sql);
				  save_sql = sql;
				  first = 0;
			      }
			    else
			      {
				  /* any other subsequent filter */
				  sql =
				      sqlite3_mprintf ("%s AND \"%s\" %s",
						       save_sql,
						       aggregate->aggregate_alias,
						       compar);
				  sqlite3_free (save_sql);
				  save_sql = sql;
			      }
			}
		  }
	    }
	  aggregate = aggregate->next;
      }
    return sql;
}

static char *
do_prepare_sql (struct config *params)
{
/* will create the appropriate SQL Query statement */
    if (strcmp (params->arg_query, "aggregate") == 0)
	return do_prepare_aggregated_sql (params);
    return do_prepare_filtered_sql (params);
}

static char *
clean_uri (const char *dirty)
{
/* cleaning a request URI */
    char *clean;
    int len;
    const char *p = dirty;
    const char *x = NULL;
    while (*p != '\0')
      {
	  if (*p == '?')
	    {
		x = p;
		break;
	    }
	  p++;
      }
    if (x == NULL)
      {
	  len = strlen (dirty);
	  clean = malloc (len + 1);
	  strcpy (clean, dirty);
      }
    else
      {
	  len = x - dirty;
	  clean = malloc (len + 1);
	  memcpy (clean, dirty, len);
	  *(clean + len) = '\0';
      }
    return clean;
}

static char *
do_build_get_url (struct config *params, int for_html)
{
/* will build the GetURL arguments */
    char *url;
    char *save_url;
    char *clean_1;
    char *clean_2;
    int count = 0;
    char *amp = "&";
    struct filter *filter;
    struct groupby *groupby;
    struct aggregate *aggregate;

    if (for_html)
	amp = "&amp;";
/* building the GetURL */
    clean_1 = cgi_escape_special_chars (params->instance_name);
    clean_2 = cgi_escape_special_chars (params->arg_view);
    url = sqlite3_mprintf ("?in=%s%svw=%s", clean_1, amp, clean_2);
    save_url = url;
    free (clean_1);
    free (clean_2);
    clean_1 = cgi_escape_special_chars (params->arg_query);
    url = sqlite3_mprintf ("%s%sq=%s", save_url, amp, clean_1);
    sqlite3_free (save_url);
    save_url = url;
    free (clean_1);

    if (strcmp (params->arg_query, "aggregate") == 0)
      {
	  /* Aggregate request */
	  groupby = params->first_groupby;
	  while (groupby != NULL)
	    {
		/* counting how many GROUP BY clauses are really set */
		if (groupby->groupby_column != NULL)
		    count++;
		groupby = groupby->next;
	    }
	  url = sqlite3_mprintf ("%s%scntg=%d", save_url, amp, count);
	  sqlite3_free (save_url);
	  save_url = url;

	  count = 0;
	  groupby = params->first_groupby;
	  while (groupby != NULL)
	    {
		if (groupby->groupby_column != NULL)
		  {
		      /* exporting GROUP BY clauses */
		      const char *compar = "";
		      switch (groupby->groupby_comparison)
			{
			case COMPAR_NONE:
			    compar = "none";
			    break;
			case COMPAR_EQ:
			    compar = "eq";
			    break;
			case COMPAR_NE:
			    compar = "ne";
			    break;
			case COMPAR_GT:
			    compar = "gt";
			    break;
			case COMPAR_GE:
			    compar = "ge";
			    break;
			case COMPAR_LT:
			    compar = "lt";
			    break;
			case COMPAR_LE:
			    compar = "le";
			    break;
			case COMPAR_LIKE:
			    compar = "lk";
			    break;
			case COMPAR_NOT_LIKE:
			    compar = "nlk";
			    break;
			case COMPAR_STARTS:
			    compar = "sw";
			    break;
			case COMPAR_NOT_STARTS:
			    compar = "nsw";
			    break;
			case COMPAR_ENDS:
			    compar = "ew";
			    break;
			case COMPAR_NOT_ENDS:
			    compar = "new";
			    break;
			case COMPAR_NULL:
			    compar = "in";
			    break;
			case COMPAR_NOT_NULL:
			    compar = "inn";
			    break;
			};
		      clean_1 =
			  cgi_escape_special_chars (groupby->groupby_column);
		      if (groupby->groupby_value == NULL)
			  clean_2 = cgi_escape_special_chars ("");
		      else
			  clean_2 =
			      cgi_escape_special_chars (groupby->groupby_value);
		      url =
			  sqlite3_mprintf
			  ("%s%sgc%d=%s%sgx%d=%s%sgv%d=%s",
			   save_url, amp, count, clean_1, amp, count, compar,
			   amp, count, clean_2);
		      sqlite3_free (save_url);
		      save_url = url;
		      free (clean_1);
		      free (clean_2);
		      count++;
		  }
		groupby = groupby->next;
	    }

	  count = 0;
	  aggregate = params->first_aggregate;
	  while (aggregate != NULL)
	    {
		/* counting how many Aggregate/HAVING clauses are really set */
		if (aggregate->aggregate_column != NULL)
		    count++;
		aggregate = aggregate->next;
	    }
	  url = sqlite3_mprintf ("%s%scnta=%d", save_url, amp, count);
	  sqlite3_free (save_url);
	  save_url = url;

	  count = 0;
	  aggregate = params->first_aggregate;
	  while (aggregate != NULL)
	    {
		if (aggregate->aggregate_column != NULL)
		  {
		      /* exporting every Aggregate/Having clause */
		      const char *compar = "";
		      const char *function = "";
		      switch (aggregate->aggregate_function)
			{
			case FUNCT_NONE:
			    function = "none";
			    break;
			case FUNCT_SUM:
			    function = "sum";
			    break;
			case FUNCT_AVG:
			    function = "avg";
			    break;
			case FUNCT_COUNT:
			    function = "count";
			    break;
			case FUNCT_MAX:
			    function = "max";
			    break;
			case FUNCT_MIN:
			    function = "min";
			    break;
			};
		      switch (aggregate->having_comparison)
			{
			case COMPAR_NONE:
			    compar = "none";
			    break;
			case COMPAR_EQ:
			    compar = "eq";
			    break;
			case COMPAR_NE:
			    compar = "ne";
			    break;
			case COMPAR_GT:
			    compar = "gt";
			    break;
			case COMPAR_GE:
			    compar = "ge";
			    break;
			case COMPAR_LT:
			    compar = "lt";
			    break;
			case COMPAR_LE:
			    compar = "le";
			    break;
			case COMPAR_LIKE:
			    compar = "lk";
			    break;
			case COMPAR_NOT_LIKE:
			    compar = "nlk";
			    break;
			case COMPAR_STARTS:
			    compar = "sw";
			    break;
			case COMPAR_NOT_STARTS:
			    compar = "nsw";
			    break;
			case COMPAR_ENDS:
			    compar = "ew";
			    break;
			case COMPAR_NOT_ENDS:
			    compar = "new";
			    break;
			case COMPAR_NULL:
			    compar = "in";
			    break;
			case COMPAR_NOT_NULL:
			    compar = "inn";
			    break;
			};
		      clean_1 =
			  cgi_escape_special_chars
			  (aggregate->aggregate_column);
		      if (aggregate->having_value == NULL)
			  clean_2 = cgi_escape_special_chars ("");
		      else
			  clean_2 =
			      cgi_escape_special_chars
			      (aggregate->having_value);
		      url =
			  sqlite3_mprintf
			  ("%s%sac%d=%s%saf%d=%s%shc%d=%s%shv%d=%s",
			   save_url, amp, count, clean_1, amp, count, function,
			   amp, count, compar, amp, count, clean_2);
		      sqlite3_free (save_url);
		      save_url = url;
		      free (clean_1);
		      free (clean_2);
		      count++;
		  }
		aggregate = aggregate->next;
	    }
      }
    else
      {
	  /* Filer request */
	  filter = params->first_filter;
	  while (filter != NULL)
	    {
		/* counting how many filters are really set */
		if (filter->filter_column != NULL
		    && filter->filter_comparison != COMPAR_NONE)
		  {
		      /* testing if a value is required by this comparison */
		      int need_value = 1;
		      switch (filter->filter_comparison)
			{
			case COMPAR_NULL:
			case COMPAR_NOT_NULL:
			    need_value = 0;
			    break;
			};
		      if (!need_value
			  || (need_value && filter->filter_value != NULL))
			  count++;
		  }
		filter = filter->next;
	    }
	  url = sqlite3_mprintf ("%s%scnt=%d", save_url, amp, count);
	  sqlite3_free (save_url);
	  save_url = url;

	  count = 0;
	  filter = params->first_filter;
	  while (filter != NULL)
	    {
		if (filter->filter_column != NULL
		    && filter->filter_comparison != COMPAR_NONE)
		  {
		      /* testing if a value is required by this comparison */
		      int need_value = 1;
		      switch (filter->filter_comparison)
			{
			case COMPAR_NULL:
			case COMPAR_NOT_NULL:
			    need_value = 0;
			    break;
			};
		      if (!need_value
			  || (need_value && filter->filter_value != NULL))
			{
			    /* exporting every real filter */
			    const char *compar = "";
			    switch (filter->filter_comparison)
			      {
			      case COMPAR_EQ:
				  compar = "eq";
				  break;
			      case COMPAR_NE:
				  compar = "ne";
				  break;
			      case COMPAR_GT:
				  compar = "gt";
				  break;
			      case COMPAR_GE:
				  compar = "ge";
				  break;
			      case COMPAR_LT:
				  compar = "lt";
				  break;
			      case COMPAR_LE:
				  compar = "le";
				  break;
			      case COMPAR_LIKE:
				  compar = "lk";
				  break;
			      case COMPAR_NOT_LIKE:
				  compar = "nlk";
				  break;
			      case COMPAR_STARTS:
				  compar = "sw";
				  break;
			      case COMPAR_NOT_STARTS:
				  compar = "nsw";
				  break;
			      case COMPAR_ENDS:
				  compar = "ew";
				  break;
			      case COMPAR_NOT_ENDS:
				  compar = "new";
				  break;
			      case COMPAR_NULL:
				  compar = "in";
				  break;
			      case COMPAR_NOT_NULL:
				  compar = "inn";
				  break;
			      };
			    clean_1 =
				cgi_escape_special_chars
				(filter->filter_column);
			    clean_2 =
				cgi_escape_special_chars (filter->filter_value);
			    url =
				sqlite3_mprintf
				("%s%sc%d=%s%sx%d=%s%sv%d=%s",
				 save_url, amp, count, clean_1, amp, count,
				 compar, amp, count, clean_2);
			    sqlite3_free (save_url);
			    save_url = url;
			    free (clean_1);
			    free (clean_2);
			    count++;
			}
		  }
		filter = filter->next;
	    }
      }

    clean_1 = cgi_escape_special_chars (params->arg_format);
    url = sqlite3_mprintf ("%s%sfmt=%s%sgo=go", save_url, amp, clean_1, amp);
    sqlite3_free (save_url);
    free (clean_1);
    return url;
}

static void
do_update_log (struct config *params)
{
/* updating the logfile */
    struct tm *now;
    struct timeval end;
    struct timeval diff;
    int millis;
    const char *month[12] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
    };
    FILE *log = fopen (params->logfile_path, "a");
    if (log != NULL)
      {
	  const char *remote_addr = NULL;
	  const char *request_method = NULL;
	  const char *request_uri = NULL;
	  char *request_uri2 = NULL;
	  char *query_string = NULL;
	  remote_addr = getenv ("REMOTE_ADDR");
	  if (remote_addr == NULL)
	      remote_addr = "-";
	  request_method = getenv ("REQUEST_METHOD");
	  if (request_method == NULL)
	      request_method = "-";
	  if (params->timestamps_localtime)
	      now = localtime (&(params->now));
	  else
	      now = gmtime (&(params->now));
	  gettimeofday (&end, NULL);
	  timersub (&end, &(params->start), &diff);
	  millis = (diff.tv_sec * 1000) + (diff.tv_usec / 1000);
	  request_uri = getenv ("REQUEST_URI");
	  if (request_uri == NULL)
	      request_uri = "-";
	  request_uri2 = clean_uri (request_uri);
	  query_string = do_build_get_url (params, 0);
	  fprintf (log,
		   "\"%s\",\"[%02d/%s/%04d:%02d:%02d:%02d]\",%d,\"%s\",\"%s\",\"%s\",200,%d\n",
		   remote_addr, now->tm_mday, month[now->tm_mon],
		   now->tm_year + 1900, now->tm_hour, now->tm_min, now->tm_sec,
		   millis, request_method, request_uri2, query_string,
		   params->bytes);
	  free (request_uri2);
	  sqlite3_free (query_string);
	  fclose (log);
      }
}

static char *
find_current_col_value (const char *column, sqlite3_stmt * stmt)
{
/* fetching the current column value */
    char *value = NULL;
    int i;
    int n_columns = sqlite3_column_count (stmt);
    for (i = 0; i < n_columns; i++)
      {
	  /* Column Values */
	  if (strcasecmp (column, sqlite3_column_name (stmt, i)) == 0)
	    {
		/* ok, this is the target column */
		int type = sqlite3_column_type (stmt, i);
		switch (type)
		  {
		  case SQLITE_TEXT:
		      value =
			  sqlite3_mprintf ("%s",
					   (const char *)
					   sqlite3_column_text (stmt, i));
		      break;
		  case SQLITE_INTEGER:
		      value =
			  sqlite3_mprintf ("%lld\n",
					   sqlite3_column_int64 (stmt, i));
		      break;
		  case SQLITE_FLOAT:
		      value =
			  sqlite3_mprintf ("1.5f\n",
					   sqlite3_column_double (stmt, i));
		      break;
		  case SQLITE_NULL:
		  case SQLITE_BLOB:
		  default:
		      break;
		  };
	    }
      }
    return value;
}

static char *
do_prepare_base_drilldown_url (struct config *params, sqlite3_stmt * stmt)
{
/* preparing basic/common DrillDown URL */
    char *url;
    char *save_url;
    char *clean_1;
    char *clean_2;
    int count;
    struct groupby *groupby;
    struct aggregate *aggregate;

    clean_1 = cgi_escape_special_chars (params->instance_name);
    clean_2 = cgi_escape_special_chars (params->arg_view);
    url =
	sqlite3_mprintf ("%s?in=%s&amp;vw=%s", params->cgi_url, clean_1,
			 clean_2);
    save_url = url;
    free (clean_1);
    free (clean_2);
    url = sqlite3_mprintf ("%s&amp;q=filter", save_url);
    sqlite3_free (save_url);
    save_url = url;
    count = 0;
    groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  /* counting how many GROUP BY clauses are really set */
	  if (groupby->groupby_column != NULL)
	      count++;
	  groupby = groupby->next;
      }
    url = sqlite3_mprintf ("%s&amp;cnt=%d", save_url, count);
    sqlite3_free (save_url);
    save_url = url;

    count = 0;
    groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  if (groupby->groupby_column != NULL)
	    {
		/* exporting GROUP BY clauses as WHERE clauses */
		char *col_value =
		    find_current_col_value (groupby->groupby_column, stmt);
		clean_1 = cgi_escape_special_chars (groupby->groupby_column);
		if (col_value == NULL)
		  {
		      /* NULL value */
		      url =
			  sqlite3_mprintf ("%s&amp;c%d=%s&amp;x%d=in&amp;v%d=",
					   save_url, count, clean_1, count,
					   count);
		  }
		else
		  {
		      clean_2 = cgi_escape_special_chars (col_value);
		      url =
			  sqlite3_mprintf
			  ("%s&amp;c%d=%s&amp;x%d=eq&amp;v%d=%s", save_url,
			   count, clean_1, count, count, clean_2);
		      free (clean_2);
		      sqlite3_free (col_value);
		  }
		sqlite3_free (save_url);
		save_url = url;
		free (clean_1);
		count++;
	    }
	  groupby = groupby->next;
      }

/* ORDER BY clauses */
    count = 0;
    groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  if (groupby->groupby_column != NULL)
	    {
		clean_1 = cgi_escape_special_chars (groupby->groupby_column);
		save_url = url;
		url =
		    sqlite3_mprintf ("%s&amp;o%d=%s", save_url, count, clean_1);
		sqlite3_free (save_url);
		save_url = url;
		free (clean_1);
		count++;
	    }
	  groupby = groupby->next;
      }
    aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  if (aggregate->aggregate_column != NULL)
	    {
		clean_1 =
		    cgi_escape_special_chars (aggregate->aggregate_column);
		save_url = url;
		url =
		    sqlite3_mprintf ("%s&amp;o%d=%s", save_url, count, clean_1);
		sqlite3_free (save_url);
		save_url = url;
		free (clean_1);
		count++;
	    }
	  aggregate = aggregate->next;
      }
    save_url = url;
    url = sqlite3_mprintf ("%s&amp;cnto=%d", save_url, count);
    sqlite3_free (save_url);
    return url;
}

static int
is_full_anchor (const char *string)
{
/* checking for a full anchor tag */
    char start = '\0';
    char end = '\0';
    int len = strlen (string);
    const char *p = string;
    while (*p != '\0')
      {
	  if (*p == ' ' || *p == '\t')
	    {
		p++;
		continue;
	    }
	  start = *p;
	  break;
      }
    p = string + len - 1;
    while (p >= string)
      {
	  if (*p == ' ' || *p == '\t')
	    {
		p--;
		continue;
	    }
	  end = *p;
	  break;
      }
    if (start == '<' && end == '>')
	return 1;
    return 0;
}

static int
do_sniff_resultset (sqlite3_stmt * stmt, int max_resultset_rows,
		    int *truncation)
{
/* first pass: checking the resultset */
    int ret;
    int nrow = 0;

    *truncation = 0;
    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* ok, found a valid row */ ;
		if (max_resultset_rows > 0 && nrow >= max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		nrow++;
	    }
	  else
	      break;
      }
    sqlite3_reset (stmt);
    return nrow;
}

static void
do_output_html_resultset (struct config *params)
{
/* will return the HTML resultset page */
    int ret;
    sqlite3_stmt *stmt = NULL;
    char *sql;
    char *item;
    int n_columns;
    int i;
    int nrow = 0;
    char *timestamp;
    int truncation = 0;
    int drill_down = 0;
    char *query_string;
    char *url;
    int tot_rows;
    int page_no;
    int tot_pages;
    int last_row;
    int next_offset;
    int prev_offset;
    int last_offset;
    int current_offset = 0;
    int end_offset;
    int start = 0;
    int end = 0;

    if (strcmp (params->arg_query, "aggregate") == 0)
	drill_down = 1;

/* HTML header */
    myputs
	("<!DOCTYPE html PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n");
    myputs ("<html>\n<head>\n");
    myputs
	("<meta http-equiv='content-type' content='text/html; charset=UTF-8'>\n");
    myputs ("<title>dataSeltzer resultset</title>\n");
    myputs ("<link rel=\"stylesheet\" type=\"text/css\" href=\"");
    myputs (params->css_stylesheet_url);
    myputs ("\">\n");
    myputs ("</head>\n<body>\n");
    myputs ("<h1>[");
    timestamp = get_current_timestamp (params);
    myputs (timestamp);
    free (timestamp);
    myputs
	("]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;dataset: ");
    myputs (params->arg_view);
    myputs ("</h1>\n");

/* executing the SQL Query statement */
    sql = do_prepare_sql (params);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	goto stop;

    tot_rows =
	do_sniff_resultset (stmt, params->max_resultset_rows, &truncation);
    end_offset = tot_rows;
    if (truncation)
      {
	  myputs ("<h1>");
	  myputs (params->label_data_truncation);
	  myputs ("</h1>\n");
      }

    if (params->max_rows_html_page > 0)
      {
	  /* backwards / forwards hyperlinks */
	  tot_pages = tot_rows / params->max_rows_html_page;
	  if ((tot_pages * params->max_rows_html_page) < tot_rows)
	      tot_pages++;
	  last_row = params->current_html_offset + params->max_rows_html_page;
	  if (last_row > tot_rows)
	      last_row = tot_rows;
	  current_offset = params->current_html_offset;
	  page_no = current_offset / params->max_rows_html_page;
	  prev_offset = current_offset - params->max_rows_html_page;
	  next_offset = current_offset + params->max_rows_html_page;
	  last_offset = (tot_pages - 1) * params->max_rows_html_page;
	  if (current_offset == 0)
	      start = 1;
	  if (current_offset == last_offset)
	      end = 1;
	  if (prev_offset < 0)
	      prev_offset = 0;
	  if (next_offset > tot_rows)
	      next_offset = (tot_pages - 1) * params->max_rows_html_page;
	  end_offset = next_offset - 1;
	  if (end)
	      end_offset = tot_rows;
	  myputs
	      ("<table class=\"results_table\" width=\"75%\"><tr><td align=\"left\">\n");
	  query_string = do_build_get_url (params, 1);
	  url =
	      sqlite3_mprintf ("%s%s&amp;hoff=0", params->cgi_url,
			       query_string);
	  if (start)
	    {
		myputs
		    ("<td align=\"left\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_first);
		myputs ("\" alt=\"prev\"></a></td>");
		myputs
		    ("<td align=\"left\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_prev);
		myputs ("\" alt=\"prev\"></a></td>");
	    }
	  else
	    {
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_first);
		myputs ("\" alt=\"first\"></a></td>");
		myputs ("<td align=\"left\">\n");
		query_string = do_build_get_url (params, 1);
		url =
		    sqlite3_mprintf ("%s%s&amp;hoff=%d", params->cgi_url,
				     query_string, prev_offset);
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_prev);
		myputs ("\" alt=\"prev\"></a></td>");
	    }
	  item =
	      sqlite3_mprintf
	      ("<td class=\"result_page_count\" align=\"center\">Page %d/%d "
	       "[rows: %d - %d / total rows: %d]</td>\n",
	       page_no + 1, tot_pages, params->current_html_offset + 1,
	       last_row, tot_rows);
	  myputs (item);
	  sqlite3_free (item);
	  if (end)
	    {
		myputs
		    ("<td align=\"right\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_next);
		myputs ("\" alt=\"prev\"></a></td>");
		myputs
		    ("<td align=\"right\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_last);
		myputs ("\" alt=\"prev\"></a></td>");
	    }
	  else
	    {
		myputs ("<td align=\"right\">\n");
		query_string = do_build_get_url (params, 1);
		url =
		    sqlite3_mprintf ("%s%s&amp;hoff=%d", params->cgi_url,
				     query_string, next_offset);
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_next);
		myputs ("\" alt=\"next\"></a></td><td align=\"right\">\n");
		url =
		    sqlite3_mprintf ("%s%s&amp;hoff=%d", params->cgi_url,
				     query_string, last_offset);
		sqlite3_free (query_string);
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_last);
		myputs ("\" alt=\"last\"></a>\n");
		myputs ("</td>");
	    }
	  myputs ("</tr></table><br>\n");
      }

/* creating the <TABLE> */
    myputs ("<table class=\"results_table\">\n");
    n_columns = sqlite3_column_count (stmt);
    myputs ("<tr>\n");
    for (i = 0; i < n_columns; i++)
      {
	  /* Table Header: Column Names */
	  const char *colname = sqlite3_column_name (stmt, i);
	  item =
	      sqlite3_mprintf ("<td class=\"results_title\">%s</td>\n",
			       colname);
	  myputs (item);
	  sqlite3_free (item);
      }
    if (drill_down)
      {
	  /* adding the Drill-Down title */
	  item =
	      sqlite3_mprintf ("<td class=\"results_title\">%s</td>\n",
			       "DrillDown");
	  myputs (item);
	  sqlite3_free (item);
      }
    myputs ("</tr>\n");
    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* ok, found a valid row */
		const char *colname;
		int hyperlink;
		if (nrow < current_offset)
		  {
		      nrow++;
		      continue;
		  }
		if (nrow > end_offset)
		    break;
		myputs ("<tr>\n");
		for (i = 0; i < n_columns; i++)
		  {
		      /* Column Values */
		      const char *text_value;
		      sqlite3_int64 int_value;
		      double dbl_value;
		      int type = sqlite3_column_type (stmt, i);
		      switch (type)
			{
			case SQLITE_NULL:
			    if ((nrow % 2) == 0)
				myputs
				    ("<td class=\"result_odd_row\">NULL</td>\n");
			    else
				myputs
				    ("<td class=\"result_even_row\">NULL</td>\n");
			    break;
			case SQLITE_BLOB:
			    if ((nrow % 2) == 0)
				myputs
				    ("<td class=\"result_odd_row\">BLOB</td>\n");
			    else
				myputs
				    ("<td class=\"result_even_row\">BLOB</td>\n");
			    break;
			case SQLITE_TEXT:
			    hyperlink = 0;
			    colname = sqlite3_column_name (stmt, i);
			    if (strlen (colname) >= 3)
			      {
				  if ((colname[0] == 'h' || colname[0] == 'H')
				      && (colname[1] == 'l'
					  || colname[1] == 'L')
				      && colname[2] == '_')
				      hyperlink = 1;
			      }
			    text_value =
				(const char *) sqlite3_column_text (stmt, i);
			    if (hyperlink)
			      {
				  /* special case: HYPERLINK */
				  if (is_full_anchor (text_value))
				    {
					if ((nrow % 2) == 0)
					    item =
						sqlite3_mprintf
						("<td class=\"result_odd_row\">%s</td>\n",
						 text_value);
					else
					    item =
						sqlite3_mprintf
						("<td class=\"result_even_row\">%s</td>\n",
						 text_value);
				    }
				  else
				    {
					if ((nrow % 2) == 0)
					    item =
						sqlite3_mprintf
						("<td class=\"result_odd_row\">"
						 "<a href=\"%s\" target=\"_blank\">%s</a></td>\n",
						 text_value, text_value);
					else
					    item =
						sqlite3_mprintf
						("<td class=\"result_even_row\">"
						 "<a href=\"%s\" target=\"_blank\">%s</a></td>\n",
						 text_value, text_value);
				    }
			      }
			    else
			      {
				  /* ordinary TEXT value */
				  if ((nrow % 2) == 0)
				      item =
					  sqlite3_mprintf
					  ("<td class=\"result_odd_row\">%s</td>\n",
					   text_value);
				  else
				      item =
					  sqlite3_mprintf
					  ("<td class=\"result_even_row\">%s</td>\n",
					   text_value);
			      }
			    myputs (item);
			    sqlite3_free (item);
			    break;
			case SQLITE_INTEGER:
			    int_value = sqlite3_column_int64 (stmt, i);
			    if ((nrow % 2) == 0)
				item =
				    sqlite3_mprintf
				    ("<td class=\"result_num_odd_row\">%lld</td>\n",
				     int_value);
			    else
				item =
				    sqlite3_mprintf
				    ("<td class=\"result_num_even_row\">%lld</td>\n",
				     int_value);
			    myputs (item);
			    sqlite3_free (item);
			    break;
			case SQLITE_FLOAT:
			    dbl_value = sqlite3_column_double (stmt, i);
			    if ((nrow % 2) == 0)
				item =
				    sqlite3_mprintf
				    ("<td class=\"result_num_odd_row\">%1.5f</td>\n",
				     dbl_value);
			    else
				item =
				    sqlite3_mprintf
				    ("<td class=\"result_num_even_row\">%1.5f</td>\n",
				     dbl_value);
			    myputs (item);
			    sqlite3_free (item);
			    break;
			default:
			    myputs ("<td></td>\n");
			    break;
			};
		  }
		if (drill_down)
		  {
		      /* adding Drill-Down hyperlinks */
		      char *href_html;
		      char *href_xml;
		      char *href_xml_zip;
		      char *href_csv;
		      char *href_shp;
		      char *url;
		      char *save_url;

		      /* preparing the HTML hyperlink */
		      url = do_prepare_base_drilldown_url (params, stmt);
		      save_url = url;
		      url =
			  sqlite3_mprintf ("%s&amp;fmt=html&amp;go=go",
					   save_url);
		      sqlite3_free (save_url);
		      href_html =
			  sqlite3_mprintf
			  ("<a class=\"drill_down_anchor\" href=\"%s\" target=\"_blank\">HTML</a> ",
			   url);
		      sqlite3_free (url);

		      /* preparing the XML hyperlink */
		      url = do_prepare_base_drilldown_url (params, stmt);
		      save_url = url;
		      url =
			  sqlite3_mprintf ("%s&amp;fmt=xml&amp;go=go",
					   save_url);
		      sqlite3_free (save_url);
		      href_xml =
			  sqlite3_mprintf
			  ("<a class=\"drill_down_anchor\" href=\"%s\" target=\"_blank\">XML</a> ",
			   url);
		      sqlite3_free (url);

		      /* preparing the XML-ZIP hyperlink */
		      url = do_prepare_base_drilldown_url (params, stmt);
		      save_url = url;
		      url =
			  sqlite3_mprintf ("%s&amp;fmt=xmlzip&amp;go=go",
					   save_url);
		      sqlite3_free (save_url);
		      href_xml_zip =
			  sqlite3_mprintf
			  ("<a class=\"drill_down_anchor\" href=\"%s\">XML-zip</a> ",
			   url);
		      sqlite3_free (url);

		      /* preparing the CSV hyperlink */
		      url = do_prepare_base_drilldown_url (params, stmt);
		      save_url = url;
		      url =
			  sqlite3_mprintf ("%s&amp;fmt=csv&amp;go=go",
					   save_url);
		      sqlite3_free (save_url);
		      href_csv =
			  sqlite3_mprintf
			  ("<a class=\"drill_down_anchor\" href=\"%s\">CSV</a> ",
			   url);
		      sqlite3_free (url);

		      if (is_geometry_view_drilldown (params))
			{
			    /* preparing the SHP hyperlink */
			    url = do_prepare_base_drilldown_url (params, stmt);
			    save_url = url;
			    url =
				sqlite3_mprintf
				("%s&amp;fmt=shp&amp;go=go", save_url);
			    sqlite3_free (save_url);
			    href_shp =
				sqlite3_mprintf
				("<a class=\"drill_down_anchor\" href=\"%s\">SHP</a> ",
				 url);
			    sqlite3_free (url);
			}
		      else
			{
			    /* unsupported SHP: just a placeholder */
			    href_shp = sqlite3_mprintf (" ");
			}

		      /* inserting Drill-Down <TD> */
		      if ((nrow % 2) == 0)
			  item =
			      sqlite3_mprintf
			      ("<td class=\"result_num_odd_row\">"
			       "%s %s %s %s %s</td>\n",
			       href_html, href_xml, href_xml_zip,
			       href_csv, href_shp);
		      else
			  item =
			      sqlite3_mprintf
			      ("<td class=\"result_num_even_row\">"
			       "%s %s %s %s %s</td>\n",
			       href_html, href_xml, href_xml_zip,
			       href_csv, href_shp);
		      sqlite3_free (href_html);
		      sqlite3_free (href_xml);
		      sqlite3_free (href_xml_zip);
		      sqlite3_free (href_csv);
		      sqlite3_free (href_shp);
		      myputs (item);
		      sqlite3_free (item);
		  }
		myputs ("</tr>\n");
		nrow++;
	    }
	  else
	      break;
      }
    sqlite3_finalize (stmt);

/* HTML footer */
  stop:
    myputs ("</table>\n");

    if (params->max_rows_html_page > 0)
      {
	  /* backwards / forwards hyperlinks */
	  myputs
	      ("<br><br><table class=\"results_table\" width=\"75%\"><tr><td align=\"left\">\n");
	  query_string = do_build_get_url (params, 1);
	  url =
	      sqlite3_mprintf ("%s%s&amp;hoff=0", params->cgi_url,
			       query_string);
	  if (start)
	    {
		myputs
		    ("<td align=\"left\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_first);
		myputs ("\" alt=\"prev\"></a></td>");
		myputs
		    ("<td align=\"left\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_prev);
		myputs ("\" alt=\"prev\"></a></td>");
	    }
	  else
	    {
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_first);
		myputs ("\" alt=\"first\"></a></td>");
		myputs ("<td align=\"left\">\n");
		query_string = do_build_get_url (params, 1);
		url =
		    sqlite3_mprintf ("%s%s&amp;hoff=%d", params->cgi_url,
				     query_string, prev_offset);
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_prev);
		myputs ("\" alt=\"prev\"></a></td>");
	    }
	  item =
	      sqlite3_mprintf
	      ("<td class=\"result_page_count\" align=\"center\">Page %d/%d "
	       "[rows: %d - %d / total rows: %d]</td>\n",
	       page_no + 1, tot_pages, params->current_html_offset + 1,
	       last_row, tot_rows);
	  myputs (item);
	  sqlite3_free (item);
	  if (end)
	    {
		myputs
		    ("<td align=\"right\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_next);
		myputs ("\" alt=\"prev\"></a></td>");
		myputs
		    ("<td align=\"right\"><img class=\"icon_disabled\" src=\"");
		myputs (params->icon_last);
		myputs ("\" alt=\"prev\"></a></td>");
	    }
	  else
	    {
		myputs ("<td align=\"right\">\n");
		query_string = do_build_get_url (params, 1);
		url =
		    sqlite3_mprintf ("%s%s&amp;hoff=%d", params->cgi_url,
				     query_string, next_offset);
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_next);
		myputs ("\" alt=\"next\"></a></td><td align=\"right\">\n");
		url =
		    sqlite3_mprintf ("%s%s&amp;hoff=%d", params->cgi_url,
				     query_string, last_offset);
		sqlite3_free (query_string);
		myputs ("<a href=\"");
		myputs (url);
		myputs ("\"><img src=\"");
		myputs (params->icon_last);
		myputs ("\" alt=\"last\"></a>\n");
		myputs ("</td>");
	    }
	  myputs ("</tr></table><br>\n");
      }

    if (truncation)
      {
	  myputs ("<h1>");
	  myputs (params->label_data_truncation);
	  myputs ("</h1>\n");
      }
    myputs ("</body>\n</html>\n");
}

static char *
make_timestamp2 (const char *timestamp)
{
/* building a timestamp in the format YYYYMMDDHHMMSS */
    char *str = malloc (15);
    if (strlen (timestamp) != 19)
	strcpy (str, "00000000000000");
    else
      {
	  str[0] = timestamp[0];
	  str[1] = timestamp[1];
	  str[2] = timestamp[2];
	  str[3] = timestamp[3];
	  str[4] = timestamp[5];
	  str[5] = timestamp[6];
	  str[6] = timestamp[8];
	  str[7] = timestamp[9];
	  str[8] = timestamp[11];
	  str[9] = timestamp[12];
	  str[10] = timestamp[14];
	  str[11] = timestamp[15];
	  str[12] = timestamp[17];
	  str[13] = timestamp[18];
	  str[14] = '\0';
      }
    return str;
}

static char *
make_filename (const char *path)
{
/* extracting a filename from a path */
    char *fn;
    int len;
    const char *p = path;
    const char *start = path;
    while (*p != '\0')
      {
	  if (*p == '/' || *p == '\\')
	      start = p + 1;
	  p++;
      }
    len = strlen (start);
    fn = malloc (len + 1);
    strcpy (fn, start);
    return fn;
}

static void
add_file_to_zipfile (zipFile zip, const char *arg_view,
		     const char *timestamp2, const char *path,
		     zip_fileinfo * zip_info)
{
/* attempting to insert an extra-file into the ZipFile */
    FILE *in;
    char *filename;
    char *zip_filename;
    int ret;
    unsigned char buf[BUFFSIZE];

    in = fopen (path, "rb");
    if (in == NULL)
      {
	  fprintf (stderr, "dataSeltzer: unable to read \"%s\"\n", path);
	  return;
      }
    filename = make_filename (path);
    zip_filename = sqlite3_mprintf ("%s_%s/%s", arg_view, timestamp2, filename);
    free (filename);
    ret =
	zipOpenNewFileInZip (zip, zip_filename, zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (zip_filename);
    if (ret != Z_OK)
      {
	  fprintf (stderr,
		   "dataSeltzer: unable to create a file within ZipFile: error %d\n",
		   ret);
	  fclose (in);
	  return;
      }
    while (1)
      {
	  /* copying the extra-file into the zip */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  zipWriteInFileInZip (zip, buf, nb);
      }
/* closing both in and out files */
    zipCloseFileInZip (zip);
    fclose (in);
}

static void
do_write_metainfo (zipFile * zip, struct config *params, int truncation,
		   const char *timestamp2, zip_fileinfo * zip_info)
{
/* inserting download metainfo */
    char *zip_filename;
    int ret;
    const char *str;
    char *url;
    char *query_string;
    char *timestamp;

    zip_filename =
	sqlite3_mprintf ("%s_%s/download_metainfo.txt", params->arg_view,
			 timestamp2);
    ret =
	zipOpenNewFileInZip (zip, zip_filename, zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (zip_filename);
    str = "Download Metainfo\r\n";
    if (ret != Z_OK)
	return;
    zipWriteInFileInZip (zip, str, strlen (str));
    str = "===========================\r\n\r\n";
    zipWriteInFileInZip (zip, str, strlen (str));
    query_string = do_build_get_url (params, 1);
    url =
	sqlite3_mprintf ("RequestURL: %s%s\r\n\r\n", params->cgi_url,
			 query_string);
    sqlite3_free (query_string);
    zipWriteInFileInZip (zip, url, strlen (url));
    sqlite3_free (url);
    url = sqlite3_mprintf ("Dataset: %s\r\n\r\n", params->arg_view);
    zipWriteInFileInZip (zip, url, strlen (url));
    sqlite3_free (url);
    timestamp = get_current_timestamp (params);
    url = sqlite3_mprintf ("Timestamp: %s\r\n", timestamp);
    free (timestamp);
    zipWriteInFileInZip (zip, url, strlen (url));
    sqlite3_free (url);
    if (truncation)
      {
	  url = sqlite3_mprintf ("\r\n%s\r\n", params->label_data_truncation);
	  zipWriteInFileInZip (zip, url, strlen (url));
	  sqlite3_free (url);
      }
    zipCloseFileInZip (zip);
}

static int
test_regular_file (const char *path, zip_fileinfo * zip_info)
{
/* testing for a regular file */
    struct stat sb;
    struct tm *timestamp;
    if (stat (path, &sb) < 0)
	return 0;
    if ((sb.st_mode & S_IFMT) == S_IFREG)
      {
	  timestamp = gmtime (&(sb.st_mtime));
	  zip_info->tmz_date.tm_year = (1900 + timestamp->tm_year) - 1980;
	  zip_info->tmz_date.tm_mon = timestamp->tm_mon;
	  zip_info->tmz_date.tm_mday = timestamp->tm_mday;
	  zip_info->tmz_date.tm_hour = timestamp->tm_hour;
	  zip_info->tmz_date.tm_min = timestamp->tm_min;
	  zip_info->tmz_date.tm_sec = timestamp->tm_sec;
	  zip_info->dosDate = 0;
	  zip_info->internal_fa = 0;
	  zip_info->external_fa = 0;
	  return 1;
      }
    return 0;
}

static char *
do_clean_xml (const char *dirty)
{
/* cleaning an XML string */
    char *clean;
    char *p_clean;
    int len = strlen (dirty);
    int clean_len = 0;
    int i;

    for (i = 0; i < len; i++)
      {
	  /* precomputing the clean length */
	  if (*(dirty + i) == '&')
	      clean_len += 5;
	  else if (*(dirty + i) == '>')
	      clean_len += 4;
	  else if (*(dirty + i) == '<')
	      clean_len += 4;
	  else if (*(dirty + i) == '"')
	      clean_len += 6;
	  else
	      clean_len += 1;
      }
    clean = malloc (clean_len + 1);
    p_clean = clean;
    for (i = 0; i < len; i++)
      {
	  int o;
	  char buf[2];
	  const char *out;
	  if (*(dirty + i) == '&')
	      out = "&amp;";
	  else if (*(dirty + i) == '>')
	      out = "&gt;";
	  else if (*(dirty + i) == '<')
	      out = "&lt;";
	  else if (*(dirty + i) == '"')
	      out = "&quot;";
	  else
	    {
		buf[0] = *(dirty + i);
		buf[1] = '\0';
		out = buf;
	    }
	  for (o = 0; o < (int) strlen (out); o++)
	      *p_clean++ = *(out + o);
      }
    *p_clean = '\0';
    return clean;
}

static struct xml_meta_col *
find_xml_meta_col (struct xml_meta_columns *meta, const char *name)
{
/* retrieving a column by name */
    struct xml_meta_col *pc;
    if (meta == NULL)
	return NULL;
    pc = meta->first;
    while (pc != NULL)
      {
	  if (strcasecmp (name, pc->name) == 0)
	      return pc;
	  pc = pc->next;
      }
    return meta->dflt;
}

static void
xml_meta_col_null (struct xml_meta_columns *meta, const char *name)
{
/* registering a NULL value */
    int len;
    struct xml_meta_col *pc = find_xml_meta_col (meta, name);
    if (pc != meta->dflt)
      {
	  pc->nullable = 1;
	  return;
      }
/* inserting a further column */
    pc = malloc (sizeof (struct xml_meta_col));
    len = strlen (name);
    pc->name = malloc (len + 1);
    strcpy (pc->name, name);
    pc->type = NULL;
    pc->nullable = 1;
    pc->blob_cnt = 0;
    pc->text_cnt = 0;
    pc->int_cnt = 0;
    pc->float_cnt = 0;
    pc->max_length = -1;
    pc->next = NULL;
    if (meta->first == NULL)
	meta->first = pc;
    if (meta->last != NULL)
	meta->last->next = pc;
    meta->last = pc;
}

static void
xml_meta_col_blob (struct xml_meta_columns *meta, const char *name)
{
/* registering a BLOB value */
    int len;
    struct xml_meta_col *pc = find_xml_meta_col (meta, name);
    if (pc != meta->dflt)
      {
	  pc->blob_cnt += 1;
	  return;
      }
/* inserting a further column */
    pc = malloc (sizeof (struct xml_meta_col));
    len = strlen (name);
    pc->name = malloc (len + 1);
    strcpy (pc->name, name);
    pc->type = NULL;
    pc->nullable = 0;
    pc->blob_cnt = 1;
    pc->text_cnt = 0;
    pc->int_cnt = 0;
    pc->float_cnt = 0;
    pc->max_length = -1;
    pc->next = NULL;
    if (meta->first == NULL)
	meta->first = pc;
    if (meta->last != NULL)
	meta->last->next = pc;
    meta->last = pc;
}

static void
xml_meta_col_text (struct xml_meta_columns *meta, const char *name, int bytes)
{
/* registering an TEXT value */
    int len;
    struct xml_meta_col *pc = find_xml_meta_col (meta, name);
    if (pc != meta->dflt)
      {
	  pc->text_cnt += 1;
	  if (bytes > pc->max_length)
	      pc->max_length = bytes;
	  return;
      }
/* inserting a further column */
    pc = malloc (sizeof (struct xml_meta_col));
    len = strlen (name);
    pc->name = malloc (len + 1);
    strcpy (pc->name, name);
    pc->type = NULL;
    pc->nullable = 0;
    pc->blob_cnt = 0;
    pc->text_cnt = 1;
    pc->int_cnt = 0;
    pc->float_cnt = 0;
    pc->max_length = bytes;
    pc->next = NULL;
    if (meta->first == NULL)
	meta->first = pc;
    if (meta->last != NULL)
	meta->last->next = pc;
    meta->last = pc;
}

static void
xml_meta_col_int (struct xml_meta_columns *meta, const char *name)
{
/* registering an INTEGER value */
    int len;
    struct xml_meta_col *pc = find_xml_meta_col (meta, name);
    if (pc != meta->dflt)
      {
	  pc->int_cnt += 1;
	  return;
      }
/* inserting a further column */
    pc = malloc (sizeof (struct xml_meta_col));
    len = strlen (name);
    pc->name = malloc (len + 1);
    strcpy (pc->name, name);
    pc->type = NULL;
    pc->nullable = 0;
    pc->blob_cnt = 0;
    pc->text_cnt = 0;
    pc->int_cnt = 1;
    pc->float_cnt = 0;
    pc->max_length = -1;
    pc->next = NULL;
    if (meta->first == NULL)
	meta->first = pc;
    if (meta->last != NULL)
	meta->last->next = pc;
    meta->last = pc;
}

static void
xml_meta_col_float (struct xml_meta_columns *meta, const char *name)
{
/* registering a FLOAT value */
    int len;
    struct xml_meta_col *pc = find_xml_meta_col (meta, name);
    if (pc != meta->dflt)
      {
	  pc->float_cnt += 1;
	  return;
      }
/* inserting a further column */
    pc = malloc (sizeof (struct xml_meta_col));
    len = strlen (name);
    pc->name = malloc (len + 1);
    strcpy (pc->name, name);
    pc->type = NULL;
    pc->nullable = 0;
    pc->blob_cnt = 0;
    pc->text_cnt = 0;
    pc->int_cnt = 0;
    pc->float_cnt = 1;
    pc->max_length = -1;
    pc->next = NULL;
    if (meta->first == NULL)
	meta->first = pc;
    if (meta->last != NULL)
	meta->last->next = pc;
    meta->last = pc;
}

static void
xml_meta_eval_col (struct xml_meta_col *pc)
{
/* evaluating an XML column datatype */
    int len;
    const char *type;
    if (pc->int_cnt > 0 && pc->float_cnt == 0 && pc->text_cnt == 0
	&& pc->blob_cnt == 0)
      {
	  type = "INTEGER";
	  if (pc->type != NULL)
	      free (pc->type);
	  len = strlen (type);
	  pc->type = malloc (len + 1);
	  strcpy (pc->type, type);
	  return;
      }
    if (pc->float_cnt > 0 && pc->text_cnt == 0 && pc->blob_cnt == 0)
      {
	  type = "DOUBLE";
	  if (pc->type != NULL)
	      free (pc->type);
	  len = strlen (type);
	  pc->type = malloc (len + 1);
	  strcpy (pc->type, type);
	  return;
      }
    if (pc->int_cnt == 0 && pc->float_cnt == 0 && pc->text_cnt == 0
	&& pc->blob_cnt > 0)
      {
	  type = "BLOB";
	  if (pc->type != NULL)
	      free (pc->type);
	  len = strlen (type);
	  pc->type = malloc (len + 1);
	  strcpy (pc->type, type);
	  return;
      }
    if (pc->int_cnt == 0 && pc->float_cnt == 0 && pc->text_cnt > 0
	&& pc->blob_cnt == 0)
      {
	  type = "TEXT";
	  if (pc->type != NULL)
	      free (pc->type);
	  len = strlen (type);
	  pc->type = malloc (len + 1);
	  strcpy (pc->type, type);
	  return;
      }
    type = "UNKNOWN";
    if (pc->type != NULL)
	free (pc->type);
    len = strlen (type);
    pc->type = malloc (len + 1);
    strcpy (pc->type, type);
}

static void
xml_meta_eval (struct xml_meta_columns *meta)
{
/* setting up the final data-types */
    struct xml_meta_col *pc;
    if (meta == NULL)
	return;
    pc = meta->first;
    while (pc != NULL)
      {
	  xml_meta_eval_col (pc);
	  pc = pc->next;
      }
}

static struct xml_meta_columns *
build_xml_simple_cols (struct config *params, sqlite3_stmt * stmt)
{
/* building an XML metacatalog - simplified */
    int ret;
    char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    int c;
    char *quoted;
    int n_columns;
    int len;
    const char *name;
    const char *type;
    int not_null;
    struct xml_meta_col *pc;
    struct xml_meta_columns *meta = malloc (sizeof (struct xml_meta_columns));
    meta->first = NULL;
    meta->last = NULL;
/* inserting the default column */
    pc = malloc (sizeof (struct xml_meta_col));
    pc->name = NULL;
    name = "UNKNOWN";
    len = strlen (name);
    pc->type = malloc (len + 1);
    strcpy (pc->type, name);
    pc->nullable = 1;
    pc->max_length = -1;
    pc->next = NULL;
    meta->dflt = pc;

    quoted = gaiaDoubleQuotedSql (params->arg_view);
    sql = sqlite3_mprintf ("PRAGMA table_info(\"%s\")", quoted);
    free (quoted);
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows, &columns,
			   NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return meta;

    n_columns = sqlite3_column_count (stmt);
    for (c = 0; c < n_columns; c++)
      {
	  for (i = 1; i <= rows; i++)
	    {
		name = results[(i * columns) + 1];
		type = results[(i * columns) + 2];
		not_null = atoi (results[(i * columns) + 3]);
		if (strcasecmp (name, sqlite3_column_name (stmt, c)) != 0)
		    continue;
		pc = malloc (sizeof (struct xml_meta_col));
		len = strlen (name);
		pc->name = malloc (len + 1);
		strcpy (pc->name, name);
		if (*type == '\0')
		    type = "UNKNOWN";
		len = strlen (type);
		pc->type = malloc (len + 1);
		strcpy (pc->type, type);
		if (not_null)
		    pc->nullable = 0;
		else
		    pc->nullable = 1;
		pc->max_length = -1;
		pc->next = NULL;
		if (meta->first == NULL)
		    meta->first = pc;
		if (meta->last != NULL)
		    meta->last->next = pc;
		meta->last = pc;
	    }
      }
    sqlite3_free_table (results);

    return meta;
}

static struct xml_meta_columns *
build_xml_meta_cols (struct config *params, sqlite3_stmt * stmt)
{
/* building an XML metacatalog */
    int len;
    const char *name;
    int ret;
    int nrow = 0;
    int n_columns;
    int i;
    struct xml_meta_col *pc;
    struct xml_meta_columns *meta;

    if (params->full_xml_column_infos == 0)
	return build_xml_simple_cols (params, stmt);

    meta = malloc (sizeof (struct xml_meta_columns));
    meta->first = NULL;
    meta->last = NULL;
/* inserting the default column */
    pc = malloc (sizeof (struct xml_meta_col));
    pc->name = NULL;
    name = "UNKNOWN";
    len = strlen (name);
    pc->type = malloc (len + 1);
    strcpy (pc->type, name);
    pc->nullable = 1;
    pc->max_length = -1;
    pc->next = NULL;
    meta->dflt = pc;

    n_columns = sqlite3_column_count (stmt);
    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* ok, found a valid row */
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		    break;
		for (i = 0; i < n_columns; i++)
		  {
		      /* sniffing cell values */
		      const char *colname = sqlite3_column_name (stmt, i);
		      int type = sqlite3_column_type (stmt, i);
		      switch (type)
			{
			case SQLITE_BLOB:
			    xml_meta_col_blob (meta, colname);
			    break;
			case SQLITE_TEXT:
			    xml_meta_col_text (meta, colname,
					       sqlite3_column_bytes (stmt, i));
			    break;
			case SQLITE_INTEGER:
			    xml_meta_col_int (meta, colname);
			    break;
			case SQLITE_FLOAT:
			    xml_meta_col_float (meta, colname);
			    break;
			case SQLITE_NULL:
			default:
			    xml_meta_col_null (meta, colname);
			    break;
			};
		  }
		nrow++;
	    }
	  else
	      break;
      }
/* resetting the SQL statement */
    sqlite3_reset (stmt);
    xml_meta_eval (meta);
    return meta;
}

static void
destroy_xml_meta_cols (struct xml_meta_columns *meta)
{
/* memory cleanup - destroying an XML metacatalog */
    struct xml_meta_col *pc;
    struct xml_meta_col *pcn;
    if (meta == NULL)
	return;
    pc = meta->first;
    while (pc != NULL)
      {
	  pcn = pc->next;
	  if (pc->name != NULL)
	      free (pc->name);
	  if (pc->type != NULL)
	      free (pc->type);
	  free (pc);
	  pc = pcn;
      }
    if (meta->dflt != NULL)
      {
	  pc = meta->dflt;
	  if (pc->type != NULL)
	      free (pc->type);
	  free (pc);
      }
    free (meta);
}

static char *
create_hex_blob (unsigned char *p_blob, int n_bytes)
{
/* creating an Hex-Blob string */
    int i;
    char byte[3];
    char *hex = malloc ((n_bytes * 2) + 1);
    char *p = hex;
    for (i = 0; i < n_bytes; i++)
      {
	  sprintf (byte, "%02X", *(p_blob + i));
	  *p++ = byte[0];
	  *p++ = byte[1];
      }
    *p = '\0';
    return hex;
}

static int
do_output_xml_resultset (struct config *params, int *truncation)
{
/* will return the XML resultset (text/xml) */
    int ret;
    sqlite3_stmt *stmt = NULL;
    char *sql;
    char *item;
    int n_columns;
    int i;
    char *temp_path;
    time_t now;
    int bytes = 0;
    FILE *xml;
    unsigned char buf[BUFFSIZE];
    char *timestamp = NULL;
    int nrow = 0;
    char *url;
    char *query_string;
    char *clean_xml;
    struct xml_meta_columns *meta_cols;
    char *rowid_name = NULL;
    int ok;

    *truncation = 0;
/* executing the SQL Query statement */
    sql = do_prepare_sql (params);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;

/* creating the temporary XML file */
    time (&now);
    timestamp = get_current_timestamp (params);
    temp_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.xml", params->temp_dir, now, getpid ());
    xml = fopen (temp_path, "w+");
    if (xml == NULL)
      {
	  fprintf (stderr, "Unable to create a ZipFile\n");
	  sqlite3_free (temp_path);
	  return 0;
      }

/* printing the XML header */
    fprintf (xml, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
    fprintf (xml, "<dataSeltzerResultSet>\n");
    fprintf (xml, "<dataset>%s</dataset>\n", params->arg_view);
    fprintf (xml, "<timestamp>%s</timestamp>\n", timestamp);
    query_string = do_build_get_url (params, 1);
    url = sqlite3_mprintf ("%s%s", params->cgi_url, query_string);
    sqlite3_free (query_string);
    fprintf (xml, "<requestURL>%s</requestURL>\n", url);
    sqlite3_free (url);
    fprintf (xml, "<columnNames>\n");

    meta_cols = build_xml_meta_cols (params, stmt);
    n_columns = sqlite3_column_count (stmt);
    for (i = 0; i < n_columns; i++)
      {
	  /* Header: Column Names */
	  const char *colname = sqlite3_column_name (stmt, i);
	  struct xml_meta_col *col = find_xml_meta_col (meta_cols, colname);
	  clean_xml = do_clean_xml (colname);
	  if (col->max_length > 0)
	      fprintf (xml,
		       "<column name=\"%s\" type=\"%s\" nullable=\"%d\" maxLength=\"%d\" />\n",
		       clean_xml, col->type, col->nullable, col->max_length);
	  else
	      fprintf (xml,
		       "<column name=\"%s\" type=\"%s\" nullable=\"%d\" />\n",
		       clean_xml, col->type, col->nullable);
	  free (clean_xml);
      }
    rowid_name = sqlite3_mprintf ("ufid");
    ok = 1;
    while (ok)
      {
	  int alias = 1;
	  ok = 0;
	  for (i = 0; i < n_columns; i++)
	    {
		/* checking for RowIdName unicity */
		const char *colname = sqlite3_column_name (stmt, i);
		if (strcasecmp (colname, rowid_name) == 0)
		  {
		      sqlite3_free (rowid_name);
		      rowid_name = sqlite3_mprintf ("ufid_%d", alias++);
		      ok = 1;
		      break;
		  }
	    }
      }
    destroy_xml_meta_cols (meta_cols);

    fprintf (xml, "</columnNames>\n");
    fprintf (xml, "<resultSetRows>\n");

    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* ok, found a valid row */
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		fprintf (xml, "<dataRow>\n");
		fprintf (xml, "<%s isNull=\"0\">%d</%s>\n", rowid_name,
			 nrow + 1, rowid_name);
		for (i = 0; i < n_columns; i++)
		  {
		      /* exporting every cell */
		      unsigned char *p_blob;
		      int n_bytes;
		      gaiaOutBuffer out_buf;
		      gaiaGeomCollPtr geo;
		      char *out_blob;
		      char *colname =
			  do_clean_xml (sqlite3_column_name (stmt, i));
		      int type = sqlite3_column_type (stmt, i);
		      switch (type)
			{
			case SQLITE_NULL:
			    fprintf (xml, "<%s isNull=\"1\" />\n", colname);
			    break;
			case SQLITE_BLOB:
			    if (params->full_blob_xml)
			      {
				  p_blob =
				      (unsigned char *)
				      sqlite3_column_blob (stmt, i);
				  n_bytes = sqlite3_column_bytes (stmt, i);
				  switch (gaiaGuessBlobType (p_blob, n_bytes))
				    {
				    case GAIA_GEOMETRY_BLOB:
					geo =
					    gaiaFromSpatiaLiteBlobWkb (p_blob,
								       n_bytes);
					if (geo)
					  {
					      gaiaOutBufferInitialize
						  (&out_buf);
					      gaiaOutWkt (&out_buf, geo);
					      if (out_buf.Error
						  || out_buf.Buffer == NULL)
						  out_blob =
						      create_hex_blob (p_blob,
								       n_bytes);
					      else
						{
						    out_blob = out_buf.Buffer;
						    out_buf.Buffer = NULL;
						}
					      gaiaFreeGeomColl (geo);
					      gaiaOutBufferReset (&out_buf);
					  }
					break;
				    default:
					out_blob =
					    create_hex_blob (p_blob, n_bytes);
					break;
				    };
				  fprintf (xml,
					   "<%s isNull=\"0\">%s</%s>\n",
					   colname, out_blob, colname);
				  free (out_blob);
			      }
			    else
				fprintf (xml,
					 "<%s isNull=\"0\">BLOB</%s>\n",
					 colname, colname);
			    break;
			case SQLITE_TEXT:
			    clean_xml =
				do_clean_xml ((const char *)
					      sqlite3_column_text (stmt, i));
			    fprintf (xml,
				     "<%s isNull=\"0\">%s</%s>\n",
				     colname, (const char *) clean_xml,
				     colname);
			    free (clean_xml);
			    break;
			case SQLITE_INTEGER:
			    fprintf (xml,
				     "<%s isNull=\"0\">%lld</%s>\n",
				     colname, sqlite3_column_int64 (stmt, i),
				     colname);
			    break;
			case SQLITE_FLOAT:
			    fprintf (xml,
				     "<%s isNull=\"0\">%1.18f</%s>\n",
				     colname, sqlite3_column_double (stmt, i),
				     colname);
			    break;
			default:
			    fprintf (xml,
				     "<%s isNull=\"0\">*****</%s>\n",
				     colname, colname);
			    break;
			};
		      free (colname);
		  }
		fprintf (xml, "</dataRow>\n");
		nrow++;
	    }
	  else
	      break;
      }
    sqlite3_free (rowid_name);
    if (*truncation)
	fprintf (xml, "<dataTruncation>%s</dataTruncation>\n",
		 params->label_data_truncation);
    sqlite3_finalize (stmt);

/* printing the XML footer */
    fprintf (xml, "</resultSetRows>\n");
    fprintf (xml, "</dataSeltzerResultSet>\n");

/* sending HTTP headers */
    cgi_send_header ("Content-Type: text/xml");
    bytes = ftell (xml);
    item = sqlite3_mprintf ("Content-Length: %d\n", bytes);
    params->bytes = bytes;
    cgi_send_header (item);
    sqlite3_free (item);

    rewind (xml);
    while (1)
      {
	  /* updloading the XML file */
	  int nb = fread (buf, 1, BUFFSIZE, xml);
	  if (nb <= 0)
	      break;
	  fwrite (buf, 1, nb, stdout);
      }

    fclose (xml);
    unlink (temp_path);
    sqlite3_free (temp_path);
    free (timestamp);
/* updating the LOG */
    do_update_log (params);
    return 1;
}

static char *
to_lower (const char *in)
{
/* converting a string to lowercase */
    int len = strlen (in);
    char *out = malloc (len + 1);
    char *p = out;
    int i;

    for (i = 0; i < len; i++)
      {
	  if (*(in + i) >= 'A' && *(in + i) <= 'Z')
	      *p++ = *(in + i) - 'A' + 'a';
	  else
	      *p++ = *(in + i);
      }
    *p = '\0';
    return out;
}

static int
do_output_xmlzip_resultset (struct config *params, int *truncation)
{
/* will return the XML resultset zipfile */
    int ret;
    sqlite3_stmt *stmt = NULL;
    char *sql;
    char *item;
    int n_columns;
    int i;
    zipFile zip;
    char *temp_path;
    char *file_path;
    time_t now;
    char *timestamp = NULL;
    char *timestamp2 = NULL;
    const char *line;
    char *xline;
    int bytes = 0;
    FILE *in;
    unsigned char buf[BUFFSIZE];
    void *md5;
    char *md5_checksum;
    struct extra_data_file *extra;
    zip_fileinfo zip_info;
    struct tm *tms;
    int nrow = 0;
    char *url;
    char *query_string;
    char *clean_xml;
    struct xml_meta_columns *meta_cols;
    char *rowid_name = NULL;
    int ok;

    *truncation = 0;
/* executing the SQL Query statement */
    sql = do_prepare_sql (params);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;

/* creating the temporary zipfile */
    time (&now);
    temp_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.zip", params->temp_dir, now, getpid ());
    zip = zipOpen (temp_path, APPEND_STATUS_CREATE);
    if (zip == NULL)
      {
	  fprintf (stderr, "Unable to create a ZipFile\n");
	  sqlite3_free (temp_path);
	  return 0;
      }

/* creating a file within the ZipFile */
    timestamp = get_current_timestamp (params);
    timestamp2 = make_timestamp2 (timestamp);
    file_path =
	sqlite3_mprintf ("%s_%s/%s_%s.xml", params->arg_view, timestamp2,
			 params->arg_view, timestamp2);
    tms = gmtime (&now);
    zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
    zip_info.tmz_date.tm_mon = tms->tm_mon;
    zip_info.tmz_date.tm_mday = tms->tm_mday;
    zip_info.tmz_date.tm_hour = tms->tm_hour;
    zip_info.tmz_date.tm_min = tms->tm_min;
    zip_info.tmz_date.tm_sec = tms->tm_sec;
    zip_info.dosDate = 0;
    zip_info.internal_fa = 0;
    zip_info.external_fa = 0;
    ret =
	zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (file_path);
    if (ret != Z_OK)
      {
	  free (timestamp);
	  fprintf (stderr,
		   "Unable to create a file within ZipFile: error %d\n", ret);
	  goto error;
      }

/* printing the XML header */
    line = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
    zipWriteInFileInZip (zip, line, strlen (line));
    line = "<dataSeltzerResultSet>\n";
    zipWriteInFileInZip (zip, line, strlen (line));
    item = sqlite3_mprintf ("<dataset>%s</dataset>\n", params->arg_view);
    zipWriteInFileInZip (zip, item, strlen (item));
    sqlite3_free (item);
    item = sqlite3_mprintf ("<timestamp>%s</timestamp>\n", timestamp);
    zipWriteInFileInZip (zip, item, strlen (item));
    sqlite3_free (item);
    query_string = do_build_get_url (params, 1);
    url = sqlite3_mprintf ("%s%s", params->cgi_url, query_string);
    sqlite3_free (query_string);
    item = sqlite3_mprintf ("<requestURL>%s</requestURL>\n", url);
    sqlite3_free (url);
    zipWriteInFileInZip (zip, item, strlen (item));
    sqlite3_free (item);
    line = "<columnNames>\n";
    zipWriteInFileInZip (zip, line, strlen (line));


    meta_cols = build_xml_meta_cols (params, stmt);
    n_columns = sqlite3_column_count (stmt);
    for (i = 0; i < n_columns; i++)
      {
	  /* Header: Column Names */
	  const char *colname = sqlite3_column_name (stmt, i);
	  struct xml_meta_col *col = find_xml_meta_col (meta_cols, colname);
	  clean_xml = do_clean_xml (colname);
	  if (col->max_length > 0)
	      item =
		  sqlite3_mprintf
		  ("<column name=\"%s\" type=\"%s\" nullable=\"%d\" maxLength=\"%d\" />\n",
		   clean_xml, col->type, col->nullable, col->max_length);
	  else
	      item =
		  sqlite3_mprintf
		  ("<column name=\"%s\" type=\"%s\" nullable=\"%d\" />\n",
		   clean_xml, col->type, col->nullable);
	  free (clean_xml);
	  zipWriteInFileInZip (zip, item, strlen (item));
	  sqlite3_free (item);
      }
    rowid_name = sqlite3_mprintf ("ufid");
    ok = 1;
    while (ok)
      {
	  int alias = 1;
	  ok = 0;
	  for (i = 0; i < n_columns; i++)
	    {
		/* checking for RowIdName unicity */
		const char *colname = sqlite3_column_name (stmt, i);
		if (strcasecmp (colname, rowid_name) == 0)
		  {
		      sqlite3_free (rowid_name);
		      rowid_name = sqlite3_mprintf ("ufid_%d", alias++);
		      ok = 1;
		      break;
		  }
	    }
      }
    destroy_xml_meta_cols (meta_cols);

    line = "</columnNames>\n";
    zipWriteInFileInZip (zip, line, strlen (line));
    line = "<resultSetRows>\n";
    zipWriteInFileInZip (zip, line, strlen (line));

    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* ok, found a valid row */
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		line = "<dataRow>\n";
		zipWriteInFileInZip (zip, line, strlen (line));
		xline =
		    sqlite3_mprintf ("<%s isNull=\"0\">%d</%s>\n", rowid_name,
				     nrow + 1, rowid_name);
		zipWriteInFileInZip (zip, xline, strlen (xline));
		sqlite3_free (xline);
		for (i = 0; i < n_columns; i++)
		  {
		      /* exporting every cell */
		      unsigned char *p_blob;
		      int n_bytes;
		      gaiaOutBuffer out_buf;
		      gaiaGeomCollPtr geo;
		      char *out_blob;
		      char *colname =
			  do_clean_xml (sqlite3_column_name (stmt, i));
		      int type = sqlite3_column_type (stmt, i);
		      switch (type)
			{
			case SQLITE_NULL:
			    item =
				sqlite3_mprintf
				("%s isNull=\"1\" />\n", colname);
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			case SQLITE_BLOB:
			    if (params->full_blob_xml)
			      {
				  p_blob =
				      (unsigned char *)
				      sqlite3_column_blob (stmt, i);
				  n_bytes = sqlite3_column_bytes (stmt, i);
				  switch (gaiaGuessBlobType (p_blob, n_bytes))
				    {
				    case GAIA_GEOMETRY_BLOB:
					geo =
					    gaiaFromSpatiaLiteBlobWkb (p_blob,
								       n_bytes);
					if (geo)
					  {
					      gaiaOutBufferInitialize
						  (&out_buf);
					      gaiaOutWkt (&out_buf, geo);
					      if (out_buf.Error
						  || out_buf.Buffer == NULL)
						  out_blob =
						      create_hex_blob (p_blob,
								       n_bytes);
					      else
						{
						    out_blob = out_buf.Buffer;
						    out_buf.Buffer = NULL;
						}
					      gaiaFreeGeomColl (geo);
					      gaiaOutBufferReset (&out_buf);
					  }
					break;
				    default:
					out_blob =
					    create_hex_blob (p_blob, n_bytes);
					break;
				    };
				  item =
				      sqlite3_mprintf
				      ("<%s isNull=\"0\">%s</%s>\n",
				       colname, out_blob, colname);
				  free (out_blob);
				  zipWriteInFileInZip (zip, item,
						       strlen (item));
				  sqlite3_free (item);
			      }
			    else
				item =
				    sqlite3_mprintf
				    ("<%s isNull=\"0\">BLOB</%s>\n",
				     colname, colname);
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			case SQLITE_TEXT:
			    clean_xml =
				do_clean_xml ((const char *)
					      sqlite3_column_text (stmt, i));
			    item =
				sqlite3_mprintf
				("<%s isNull=\"0\">%s</%s>\n",
				 colname, (const char *) clean_xml, colname);
			    free (clean_xml);
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			case SQLITE_INTEGER:
			    item =
				sqlite3_mprintf
				("<%s isNull=\"0\">%lld</%s>\n",
				 colname, sqlite3_column_int64 (stmt, i),
				 colname);
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			case SQLITE_FLOAT:
			    item =
				sqlite3_mprintf
				("<%s isNull=\"0\">%1.18f</%s>\n",
				 colname, sqlite3_column_double (stmt, i),
				 colname);
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			default:
			    item =
				sqlite3_mprintf
				("<%s isNull=\"0\">*****</%s>\n",
				 colname, colname);
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			};
		      free (colname);
		  }
		line = "</dataRow>\n";
		zipWriteInFileInZip (zip, line, strlen (line));
		nrow++;
	    }
	  else
	      break;
      }
    sqlite3_free (rowid_name);
    if (*truncation)
      {
	  item =
	      sqlite3_mprintf ("<dataTruncation>%s</dataTruncation>\n",
			       params->label_data_truncation);
	  zipWriteInFileInZip (zip, item, strlen (item));
	  sqlite3_free (item);
      }
    sqlite3_finalize (stmt);

/* printing the XML footer */
    line = "</resultSetRows>\n";
    zipWriteInFileInZip (zip, line, strlen (line));
    line = "</dataSeltzerResultSet>\n";
    zipWriteInFileInZip (zip, line, strlen (line));


/* closing the XML file in ZipFile */
    zipCloseFileInZip (zip);

/* inserting download metainfo */
    do_write_metainfo (zip, params, *truncation, timestamp2, &zip_info);

    if (params->extra_data_dir != NULL)
      {
	  /* attempting to retrieve datasource-specific extra-files */
	  struct dirent *entry;
	  DIR *dir;
	  char *path;

	  if (params->extra_data_subdirs)
	    {
		/* scanning the datasource specific subdir (if any) */
		char *view = to_lower (params->arg_view);
		char *dir_path =
		    sqlite3_mprintf ("%s/%s", params->extra_data_dir, view);
		free (view);
		dir = opendir (dir_path);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    path =
				sqlite3_mprintf ("%s/%s", dir_path,
						 entry->d_name);
			    if (!test_regular_file (path, &zip_info))
			      {
				  sqlite3_free (path);
				  continue;
			      }
			    add_file_to_zipfile (zip, params->arg_view,
						 timestamp2, path, &zip_info);
			    sqlite3_free (path);
			}
		      closedir (dir);
		  }
		sqlite3_free (dir_path);
	    }
	  else
	    {
		/* scanning the generic extra_data_dir */
		dir = opendir (params->extra_data_dir);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    if (strncmp
				(entry->d_name, params->arg_view,
				 strlen (params->arg_view)) == 0)
			      {
				  /* attempting to include every file starting with the View name */
				  path =
				      sqlite3_mprintf ("%s/%s",
						       params->extra_data_dir,
						       entry->d_name);
				  if (!test_regular_file (path, &zip_info))
				    {
					sqlite3_free (path);
					continue;
				    }
				  add_file_to_zipfile (zip,
						       params->arg_view,
						       timestamp2, path,
						       &zip_info);
				  sqlite3_free (path);
			      }
			}
		      closedir (dir);
		  }
	    }
      }

    extra = params->first_extra;
    while (extra != NULL)
      {
	  /* inserting any extra-file into the ZipFile */
	  if (!test_regular_file (extra->path, &zip_info))
	    {
		extra = extra->next;
		continue;
	    }
	  add_file_to_zipfile (zip, params->arg_view, timestamp2,
			       extra->path, &zip_info);
	  extra = extra->next;
      }


/* final steps */
    zipClose (zip, NULL);
    zip = NULL;
    in = fopen (temp_path, "rb");
    if (in == NULL)
	goto error;
    md5 = gaiaCreateMD5Checksum ();
    while (1)
      {
	  /* computing Content-Length and MD5 checksum */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  bytes += nb;
	  gaiaUpdateMD5Checksum (md5, buf, nb);
      }
    md5_checksum = gaiaFinalizeMD5Checksum (md5);
    gaiaFreeMD5Checksum (md5);

/* sending HTTP headers */
    cgi_send_header ("Content-Type: application/zip");
    file_path =
	sqlite3_mprintf ("%s_%s_%s.zip", params->arg_view, timestamp2,
			 md5_checksum);
    free (md5_checksum);
    item =
	sqlite3_mprintf ("Content-Disposition: attachment; filename=%s",
			 file_path);
    sqlite3_free (file_path);
    cgi_send_header (item);
    sqlite3_free (item);
    item = sqlite3_mprintf ("Content-Length: %d\n", bytes);
    params->bytes = bytes;
    cgi_send_header (item);
    sqlite3_free (item);

    rewind (in);
    while (1)
      {
	  /* updloading the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  fwrite (buf, 1, nb, stdout);
      }

    fclose (in);
    unlink (temp_path);
    sqlite3_free (temp_path);
    free (timestamp);
    free (timestamp2);
/* updating the LOG */
    do_update_log (params);
    return 1;

  error:
    if (stmt != NULL)
	sqlite3_finalize (stmt);
    if (zip != NULL)
	zipClose (zip, NULL);
    unlink (temp_path);
    sqlite3_free (temp_path);
    if (timestamp != NULL)
	free (timestamp);
    if (timestamp2 != NULL)
	free (timestamp2);
    return 0;
}

static int
do_output_csv_resultset (struct config *params, int *truncation)
{
/* will return the CSV resultset zipfile */
    int ret;
    sqlite3_stmt *stmt = NULL;
    char *sql;
    char *item;
    int n_columns;
    int i;
    zipFile zip;
    char *temp_path;
    char *file_path;
    time_t now;
    char *timestamp = NULL;
    char *timestamp2 = NULL;
    int bytes = 0;
    FILE *in;
    unsigned char buf[BUFFSIZE];
    void *md5;
    char *md5_checksum;
    struct extra_data_file *extra;
    char *quoted;
    zip_fileinfo zip_info;
    struct tm *tms;
    int nrow = 0;

    *truncation = 0;
/* executing the SQL Query statement */
    sql = do_prepare_sql (params);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;

/* creating the temporary zipfile */
    time (&now);
    temp_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.zip", params->temp_dir, now, getpid ());
    zip = zipOpen (temp_path, APPEND_STATUS_CREATE);
    if (zip == NULL)
      {
	  fprintf (stderr, "Unable to create a ZipFile\n");
	  sqlite3_free (temp_path);
	  return 0;
      }

/* creating a file within the ZipFile */
    timestamp = get_current_timestamp (params);
    timestamp2 = make_timestamp2 (timestamp);
    file_path =
	sqlite3_mprintf ("%s_%s/%s_%s.csv", params->arg_view, timestamp2,
			 params->arg_view, timestamp2);
    tms = gmtime (&now);
    zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
    zip_info.tmz_date.tm_mon = tms->tm_mon;
    zip_info.tmz_date.tm_mday = tms->tm_mday;
    zip_info.tmz_date.tm_hour = tms->tm_hour;
    zip_info.tmz_date.tm_min = tms->tm_min;
    zip_info.tmz_date.tm_sec = tms->tm_sec;
    zip_info.dosDate = 0;
    zip_info.internal_fa = 0;
    zip_info.external_fa = 0;
    ret =
	zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (file_path);
    if (ret != Z_OK)
      {
	  free (timestamp);
	  fprintf (stderr,
		   "Unable to create a file within ZipFile: error %d\n", ret);
	  goto error;
      }

/* printing the CSV header */
    n_columns = sqlite3_column_count (stmt);
    for (i = 0; i < n_columns; i++)
      {
	  /* Header: Column Names */
	  const char *colname = sqlite3_column_name (stmt, i);
	  if (i == 0)
	      item = sqlite3_mprintf ("\"%s\"", colname);
	  else
	      item = sqlite3_mprintf (",\"%s\"", colname);
	  zipWriteInFileInZip (zip, item, strlen (item));
	  sqlite3_free (item);
      }
    zipWriteInFileInZip (zip, "\r\n", 2);

    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* ok, found a valid row */
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		for (i = 0; i < n_columns; i++)
		  {
		      /* exporting every cell */
		      int type = sqlite3_column_type (stmt, i);
		      switch (type)
			{
			case SQLITE_NULL:
			    if (i > 0)
				zipWriteInFileInZip (zip, ",", 1);
			    break;
			case SQLITE_BLOB:
			    if (i == 0)
				item = "BLOB";
			    else
				item = ",BLOB";
			    zipWriteInFileInZip (zip, item, strlen (item));
			    break;
			case SQLITE_TEXT:
			    quoted =
				gaiaDoubleQuotedSql ((const char *)
						     sqlite3_column_text
						     (stmt, i));
			    if (i == 0)
				item = sqlite3_mprintf ("\"%s\"", quoted);
			    else
				item = sqlite3_mprintf (",\"%s\"", quoted);
			    free (quoted);
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			case SQLITE_INTEGER:
			    if (i == 0)
				item = sqlite3_mprintf ("%lld",
							sqlite3_column_int64
							(stmt, i));
			    else
				item = sqlite3_mprintf (",%lld",
							sqlite3_column_int64
							(stmt, i));
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			case SQLITE_FLOAT:
			    if (i == 0)
				item = sqlite3_mprintf ("%1.18f",
							sqlite3_column_double
							(stmt, i));
			    else
				item = sqlite3_mprintf (",%1.18f",
							sqlite3_column_double
							(stmt, i));
			    zipWriteInFileInZip (zip, item, strlen (item));
			    sqlite3_free (item);
			    break;
			default:
			    if (i == 0)
				item = sqlite3_mprintf ("*****");
			    else
				item = sqlite3_mprintf (",*****");
			    zipWriteInFileInZip (zip, item, strlen (item));
			    break;
			};
		  }
		zipWriteInFileInZip (zip, "\r\n", 2);
		nrow++;
	    }
	  else
	      break;
      }
    sqlite3_finalize (stmt);

/* closing the CSV file in ZipFile */
    zipCloseFileInZip (zip);

/* inserting download metainfo */
    do_write_metainfo (zip, params, *truncation, timestamp2, &zip_info);

    if (params->extra_data_dir != NULL)
      {
	  /* attempting to retrieve datasource-specific extra-files */
	  struct dirent *entry;
	  DIR *dir;
	  char *path;

	  if (params->extra_data_subdirs)
	    {
		/* scanning the datasource specific subdir (if any) */
		char *view = to_lower (params->arg_view);
		char *dir_path =
		    sqlite3_mprintf ("%s/%s", params->extra_data_dir, view);
		free (view);
		dir = opendir (dir_path);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    path =
				sqlite3_mprintf ("%s/%s", dir_path,
						 entry->d_name);
			    if (!test_regular_file (path, &zip_info))
			      {
				  sqlite3_free (path);
				  continue;
			      }
			    add_file_to_zipfile (zip, params->arg_view,
						 timestamp2, path, &zip_info);
			    sqlite3_free (path);
			}
		      closedir (dir);
		  }
		sqlite3_free (dir_path);
	    }
	  else
	    {
		/* scanning the generic extra_data_dir */
		dir = opendir (params->extra_data_dir);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    if (strncmp
				(entry->d_name, params->arg_view,
				 strlen (params->arg_view)) == 0)
			      {
				  /* attempting to include every file starting with the View name */
				  path =
				      sqlite3_mprintf ("%s/%s",
						       params->extra_data_dir,
						       entry->d_name);
				  if (!test_regular_file (path, &zip_info))
				    {
					sqlite3_free (path);
					continue;
				    }
				  add_file_to_zipfile (zip,
						       params->arg_view,
						       timestamp2, path,
						       &zip_info);
				  sqlite3_free (path);
			      }
			}
		      closedir (dir);
		  }
	    }
      }

    extra = params->first_extra;
    while (extra != NULL)
      {
	  /* inserting any extra-file into the ZipFile */
	  if (!test_regular_file (extra->path, &zip_info))
	    {
		extra = extra->next;
		continue;
	    }
	  add_file_to_zipfile (zip, params->arg_view, timestamp2,
			       extra->path, &zip_info);
	  extra = extra->next;
      }


/* final steps */
    zipClose (zip, NULL);
    zip = NULL;
    in = fopen (temp_path, "rb");
    if (in == NULL)
	goto error;
    md5 = gaiaCreateMD5Checksum ();
    while (1)
      {
	  /* computing Content-Length and MD5 checksum */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  bytes += nb;
	  gaiaUpdateMD5Checksum (md5, buf, nb);
      }
    md5_checksum = gaiaFinalizeMD5Checksum (md5);
    gaiaFreeMD5Checksum (md5);

/* sending HTTP headers */
    cgi_send_header ("Content-Type: application/zip");
    file_path =
	sqlite3_mprintf ("%s_%s_%s.zip", params->arg_view, timestamp2,
			 md5_checksum);
    free (md5_checksum);
    item =
	sqlite3_mprintf ("Content-Disposition: attachment; filename=%s",
			 file_path);
    sqlite3_free (file_path);
    cgi_send_header (item);
    sqlite3_free (item);
    item = sqlite3_mprintf ("Content-Length: %d\n", bytes);
    params->bytes = bytes;
    cgi_send_header (item);
    sqlite3_free (item);

    rewind (in);
    while (1)
      {
	  /* updloading the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  fwrite (buf, 1, nb, stdout);
      }

    fclose (in);
    unlink (temp_path);
    sqlite3_free (temp_path);
    free (timestamp);
    free (timestamp2);
/* updating the LOG */
    do_update_log (params);
    return 1;

  error:
    if (stmt != NULL)
	sqlite3_finalize (stmt);
    if (zip != NULL)
	zipClose (zip, NULL);
    unlink (temp_path);
    sqlite3_free (temp_path);
    if (timestamp != NULL)
	free (timestamp);
    if (timestamp2 != NULL)
	free (timestamp2);
    return 0;
}

static struct auxdbf_list *
alloc_auxdbf (gaiaDbfListPtr dbf_list)
{
/* allocating the auxiliary DBF struct */
    gaiaDbfFieldPtr fld;
    struct auxdbf_fld *fld_ex;
    struct auxdbf_list *auxdbf = malloc (sizeof (struct auxdbf_list));
    auxdbf->first = NULL;
    auxdbf->last = NULL;
    fld = dbf_list->First;
    while (fld)
      {
	  fld_ex = malloc (sizeof (struct auxdbf_fld));
	  fld_ex->already_used = 0;
	  fld_ex->dbf_field = fld;
	  fld_ex->next = NULL;
	  if (auxdbf->first == NULL)
	      auxdbf->first = fld_ex;
	  if (auxdbf->last != NULL)
	      auxdbf->last->next = fld_ex;
	  auxdbf->last = fld_ex;
	  fld = fld->Next;
      }
    return auxdbf;
}

static void
free_auxdbf (struct auxdbf_list *auxdbf)
{
/* freeing an auxiliary DBF struct */
    struct auxdbf_fld *n_fld;
    struct auxdbf_fld *fld = auxdbf->first;
    while (fld != NULL)
      {
	  n_fld = fld->next;
	  free (fld);
	  fld = n_fld;
      }
    free (auxdbf);
}

static gaiaDbfFieldPtr
getDbfField (struct auxdbf_list *aux, char *name)
{
/* find a DBF attribute by name */
    struct auxdbf_fld *fld = aux->first;
    while (fld)
      {
	  if (strcasecmp (fld->dbf_field->Name, name) == 0)
	    {
		fld->already_used = 1;
		return fld->dbf_field;
	    }
	  fld = fld->next;
      }
    fld = aux->first;
    while (fld)
      {
	  if (fld->already_used)
	    {
		fld = fld->next;
		continue;
	    }
	  if (strncasecmp (fld->dbf_field->Name, name, 9) == 0)
	    {
		fld->already_used = 1;
		return fld->dbf_field;
	    }
	  fld = fld->next;
      }
    return NULL;
}

static int
get_shape_params (struct config *params, char **geometry, int *shape, int *srid)
{
/* attempting to qualify the Geometry column */
    int ret;
    char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    char *geomcol;
    int invalid = 0;

    sql =
	sqlite3_mprintf ("SELECT g.geometry_type, v.view_geometry, g.srid "
			 "FROM views_geometry_columns AS v "
			 "JOIN geometry_columns AS g ON (g.f_table_name = v.f_table_name "
			 "AND g.f_geometry_column = v.f_geometry_column) "
			 "WHERE Lower(v.view_name) = Lower(%Q)",
			 params->arg_view);
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows,
			   &columns, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;
    if (rows > 1)
	return 0;

    for (i = 1; i <= rows; i++)
      {
	  int len;
	  const char *name = results[(i * columns) + 0];
	  switch (atoi (name))
	    {
	    case 1:
		*shape = GAIA_POINT;
		break;
	    case 1001:
		*shape = GAIA_POINTZ;
		break;
	    case 2001:
		*shape = GAIA_POINTM;
		break;
	    case 3001:
		*shape = GAIA_POINTZM;
		break;
	    case 2:
		*shape = GAIA_LINESTRING;
		break;
	    case 1002:
		*shape = GAIA_LINESTRINGZ;
		break;
	    case 2002:
		*shape = GAIA_LINESTRINGM;
		break;
	    case 3002:
		*shape = GAIA_LINESTRINGZM;
		break;
	    case 3:
		*shape = GAIA_POLYGON;
		break;
	    case 1003:
		*shape = GAIA_POLYGONZ;
		break;
	    case 2003:
		*shape = GAIA_POLYGONM;
		break;
	    case 3003:
		*shape = GAIA_POLYGONZM;
		break;
	    case 4:
		*shape = GAIA_MULTIPOINT;
		break;
	    case 1004:
		*shape = GAIA_MULTIPOINTZ;
		break;
	    case 2004:
		*shape = GAIA_MULTIPOINTM;
		break;
	    case 3004:
		*shape = GAIA_MULTIPOINTZM;
		break;
	    case 5:
		*shape = GAIA_MULTILINESTRING;
		break;
	    case 1005:
		*shape = GAIA_MULTILINESTRINGZ;
		break;
	    case 2005:
		*shape = GAIA_MULTILINESTRINGM;
		break;
	    case 3005:
		*shape = GAIA_MULTILINESTRINGZM;
		break;
	    case 6:
		*shape = GAIA_MULTIPOLYGON;
		break;
	    case 1006:
		*shape = GAIA_MULTIPOLYGONZ;
		break;
	    case 2006:
		*shape = GAIA_MULTIPOLYGONM;
		break;
	    case 3006:
		*shape = GAIA_MULTIPOLYGONZM;
		break;
	    default:
		invalid = 1;
		break;
	    };
	  name = results[(i * columns) + 1];
	  len = strlen (name);
	  geomcol = malloc (len + 1);
	  strcpy (geomcol, name);
	  name = results[(i * columns) + 2];
	  *srid = atoi (name);
      }
    sqlite3_free_table (results);
    if (invalid)
	return 0;
    *geometry = geomcol;
    return 1;
}

static void
get_geom_params (struct config *params, char **geometry, char **type,
		 char **dims, int *srid)
{
/* attempting to qualify the Geometry column */
    int ret;
    char *sql;
    char **results;
    int rows;
    int columns;
    int i;
    int len;
    char *geomcol;
    const char *xtype;
    const char *xdims;
    int xsrid;
    int invalid = 0;

    *geometry = NULL;
    *type = NULL;
    *dims = NULL;
    *srid = -1;

    sql =
	sqlite3_mprintf ("SELECT g.geometry_type, v.view_geometry, g.srid "
			 "FROM views_geometry_columns AS v "
			 "JOIN geometry_columns AS g ON (g.f_table_name = v.f_table_name "
			 "AND g.f_geometry_column = v.f_geometry_column) "
			 "WHERE Lower(v.view_name) = Lower(%Q)",
			 params->arg_view);
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows,
			   &columns, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return;
    if (rows > 1)
	return;

    for (i = 1; i <= rows; i++)
      {
	  const char *name = results[(i * columns) + 0];
	  switch (atoi (name))
	    {
	    case 1:
		xtype = "POINT";
		xdims = "XY";
		break;
	    case 1001:
		xtype = "POINT";
		xdims = "XYZ";
		break;
	    case 2001:
		xtype = "POINT";
		xdims = "XYM";
		break;
	    case 3001:
		xtype = "POINT";
		xdims = "XYZM";
		break;
	    case 2:
		xtype = "LINESTRING";
		xdims = "XY";
		break;
	    case 1002:
		xtype = "LINESTRING";
		xdims = "XYZ";
		break;
	    case 2002:
		xtype = "LINESTRING";
		xdims = "XYM";
		break;
	    case 3002:
		xtype = "LINESTRING";
		xdims = "XYZM";
		break;
	    case 3:
		xtype = "POLYGON";
		xdims = "XY";
		break;
	    case 1003:
		xtype = "POLYGON";
		xdims = "XYZ";
		break;
	    case 2003:
		xtype = "POLYGON";
		xdims = "XYM";
		break;
	    case 3003:
		xtype = "POLYGON";
		xdims = "XYZM";
		break;
	    case 4:
		xtype = "MULTIPOINT";
		xdims = "XY";
		break;
	    case 1004:
		xtype = "MULTIPOINT";
		xdims = "XYZ";
		break;
	    case 2004:
		xtype = "MULTIPOINT";
		xdims = "XYM";
		break;
	    case 3004:
		xtype = "MULTIPOINT";
		xdims = "XYZM";
		break;
	    case 5:
		xtype = "MULTILINESTRING";
		xdims = "XY";
		break;
	    case 1005:
		xtype = "MULTILINESTRING";
		xdims = "XYZ";
		break;
	    case 2005:
		xtype = "MULTILINESTRING";
		xdims = "XYM";
		break;
	    case 3005:
		xtype = "MULTILINESTRING";
		xdims = "XYZM";
		break;
	    case 6:
		xtype = "MULTIPOLYGON";
		xdims = "XY";
		break;
	    case 1006:
		xtype = "MULTIPOLYGON";
		xdims = "XY";
		break;
	    case 2006:
		xtype = "MULTIPOLYGON";
		xdims = "XY";
		break;
	    case 3006:
		xtype = "MULTIPOLYGON";
		xdims = "XY";
		break;
	    case 7:
		xtype = "GEOMETRYCOLLECTION";
		xdims = "XY";
		break;
	    case 1007:
		xtype = "GEOMETRYCOLLECTION";
		xdims = "XYZ";
		break;
	    case 2007:
		xtype = "GEOMETRYCOLLECTION";
		xdims = "XYM";
		break;
	    case 3007:
		xtype = "GEOMETRYCOLLECTION";
		xdims = "XYZM";
		break;
	    default:
		invalid = 1;
		break;
	    };
	  name = results[(i * columns) + 1];
	  len = strlen (name);
	  geomcol = malloc (len + 1);
	  strcpy (geomcol, name);
	  name = results[(i * columns) + 2];
	  xsrid = atoi (name);
      }
    sqlite3_free_table (results);
    if (invalid)
	return;
    *geometry = geomcol;
    len = strlen (xtype);
    *type = malloc (len + 1);
    strcpy (*type, (char *) xtype);
    len = strlen (xdims);
    *dims = malloc (len + 1);
    strcpy (*dims, (char *) xdims);
    *srid = xsrid;
}

static int
do_output_prj_file (struct config *params, const char *shp_path, int srid)
{
/* attempting to export a PRJ file */
    char **results;
    int rows;
    int columns;
    int i;
    char *sql;
    int ret;
    FILE *out;
    int ok = 0;
    char *path_prj;

    sql = sqlite3_mprintf ("SELECT srtext FROM spatial_ref_sys "
			   "WHERE srid = %d AND srtext IS NOT NULL", srid);
    ret =
	sqlite3_get_table (params->db_handle, sql, &results, &rows,
			   &columns, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;
    if (rows < 1)
	return 0;
    for (i = 1; i <= rows; i++)
      {
	  char *srs = results[(i * columns) + 0];
	  path_prj = sqlite3_mprintf ("%s.prj", shp_path);
	  out = fopen (path_prj, "wb");
	  sqlite3_free (path_prj);
	  if (out)
	    {
		/* exporting the PRJ file */
		fprintf (out, "%s\r\n", srs);
		fclose (out);
		ok = 1;
	    }
      }
    sqlite3_free_table (results);
    return ok;
}

static int
do_output_shp_resultset (struct config *params, int *truncation)
{
/* will return the SHAPEFILE resultset zipfile */
    int ret;
    sqlite3_stmt *stmt = NULL;
    char *sql;
    char *item;
    int i;
    zipFile zip = NULL;
    char *temp_path;
    char *file_path;
    time_t now;
    char *timestamp = NULL;
    char *timestamp2 = NULL;
    int bytes = 0;
    FILE *in;
    unsigned char buf[BUFFSIZE];
    void *md5;
    char *md5_checksum;
    struct extra_data_file *extra;
    zip_fileinfo zip_info;
    struct tm *tms;
    int rows;
    int row1 = 0;
    int n_cols = 0;
    int offset = 0;
    int type;
    gaiaShapefilePtr shp = NULL;
    gaiaDbfListPtr dbf_export_list = NULL;
    gaiaDbfListPtr dbf_list = NULL;
    gaiaDbfListPtr dbf_write;
    gaiaDbfFieldPtr dbf_field;
    int *max_length = NULL;
    int *sql_type = NULL;
    char *dummy;
    const void *blob_value;
    int len;
    struct auxdbf_list *auxdbf = NULL;
    char dbfbuf[256];
    int shape;
    int srid;
    char *geometry = NULL;
    char *shp_path;
    int has_prj_file;
    char *x_path;
    int nrow = 0;

    *truncation = 0;
    if (!get_shape_params (params, &geometry, &shape, &srid))
	return 0;

    timestamp = get_current_timestamp (params);
    timestamp2 = make_timestamp2 (timestamp);

/* preparing the SQL Query statement */
    sql = do_prepare_sql (params);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;

/* creating the temporary zipfile */
    time (&now);
    shp_path =
	sqlite3_mprintf ("%s/tmp_%d_%d", params->temp_dir, now, getpid ());
    temp_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.zip", params->temp_dir, now, getpid ());
    zip = zipOpen (temp_path, APPEND_STATUS_CREATE);
    if (zip == NULL)
      {
	  fprintf (stderr, "Unable to create a ZipFile\n");
	  sqlite3_free (temp_path);
	  return 0;
      }

    rows = 0;
    while (1)
      {
	  /*
	     / Pass I - scrolling the result set to compute real DBF attributes' sizes and types 
	   */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* processing a result set row */
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		row1++;
		if (n_cols == 0)
		  {
		      /* this one is the first row, so we are going to prepare the DBF Fields list */
		      n_cols = sqlite3_column_count (stmt);
		      dbf_export_list = gaiaAllocDbfList ();
		      max_length = (int *) malloc (sizeof (int) * n_cols);
		      sql_type = (int *) malloc (sizeof (int) * n_cols);
		      for (i = 0; i < n_cols; i++)
			{
			    /* initializes the DBF export fields */
			    dummy = (char *) sqlite3_column_name (stmt, i);
			    gaiaAddDbfField (dbf_export_list, dummy, '\0',
					     0, 0, 0);
			    max_length[i] = 0;
			    sql_type[i] = SQLITE_NULL;
			}
		  }
		for (i = 0; i < n_cols; i++)
		  {
		      /* update the DBF export fields analyzing fetched data */
		      type = sqlite3_column_type (stmt, i);
		      if (type == SQLITE_NULL || type == SQLITE_BLOB)
			  continue;
		      if (type == SQLITE_TEXT)
			{
			    len = sqlite3_column_bytes (stmt, i);
			    sql_type[i] = SQLITE_TEXT;
			    if (len > max_length[i])
				max_length[i] = len;
			}
		      else if (type == SQLITE_FLOAT
			       && sql_type[i] != SQLITE_TEXT)
			  sql_type[i] = SQLITE_FLOAT;	/* promoting a numeric column to be DOUBLE */
		      else if (type == SQLITE_INTEGER
			       && (sql_type[i] == SQLITE_NULL
				   || sql_type[i] == SQLITE_INTEGER))
			  sql_type[i] = SQLITE_INTEGER;	/* promoting a null column to be INTEGER */
		      if (type == SQLITE_INTEGER && max_length[i] < 18)
			  max_length[i] = 18;
		      if (type == SQLITE_FLOAT && max_length[i] < 24)
			  max_length[i] = 24;
		  }
		nrow++;
	    }
	  else
	      goto shp_error;
      }
    if (dbf_export_list == NULL)
	goto shp_error;
    i = 0;
    offset = 0;
    dbf_list = gaiaAllocDbfList ();
    dbf_field = dbf_export_list->First;
    while (dbf_field)
      {
	  /* preparing the final DBF attribute list */
	  if (sql_type[i] == SQLITE_NULL || sql_type[i] == SQLITE_BLOB)
	    {
		i++;
		dbf_field = dbf_field->Next;
		continue;
	    }
	  if (sql_type[i] == SQLITE_TEXT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'C', offset,
				 max_length[i], 0);
		offset += max_length[i];
	    }
	  if (sql_type[i] == SQLITE_FLOAT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 19, 6);
		offset += 19;
	    }
	  if (sql_type[i] == SQLITE_INTEGER)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 18, 0);
		offset += 18;
	    }
	  i++;
	  dbf_field = dbf_field->Next;
      }
    free (max_length);
    free (sql_type);
    gaiaFreeDbfList (dbf_export_list);
    dbf_export_list = NULL;

/* resetting SQLite query */
    ret = sqlite3_reset (stmt);
    if (ret != SQLITE_OK)
	goto shp_error;
/* trying to open shapefile files */
    shp = gaiaAllocShapefile ();
    gaiaOpenShpWrite (shp, shp_path, shape, dbf_list, "UTF-8", "UTF-8");
    if (!(shp->Valid))
	goto shp_error;
/* trying to export the .PRJ file */
    has_prj_file = do_output_prj_file (params, shp_path, srid);
    nrow = 0;
    while (1)
      {
	  /* Pass II - scrolling the result set to dump data into Shapefile */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		rows++;
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		dbf_write = gaiaCloneDbfEntity (dbf_list);
		auxdbf = alloc_auxdbf (dbf_write);
		for (i = 0; i < n_cols; i++)
		  {
		      if (strcasecmp
			  ((char *) geometry,
			   (char *) sqlite3_column_name (stmt, i)) == 0)
			{
			    /* this one is the internal BLOB encoded GEOMETRY to be exported */
			    if (sqlite3_column_type (stmt, i) != SQLITE_BLOB)
			      {
				  /* this one is a NULL Geometry */
				  dbf_write->Geometry = NULL;
			      }
			    else
			      {
				  blob_value = sqlite3_column_blob (stmt, i);
				  len = sqlite3_column_bytes (stmt, i);
				  dbf_write->Geometry =
				      gaiaFromSpatiaLiteBlobWkb (blob_value,
								 len);
			      }
			}
		      dummy = (char *) sqlite3_column_name (stmt, i);
		      dbf_field = getDbfField (auxdbf, dummy);
		      if (!dbf_field)
			  continue;
		      if (sqlite3_column_type (stmt, i) == SQLITE_NULL
			  || sqlite3_column_type (stmt, i) == SQLITE_BLOB)
			{
			    /* handling NULL values */
			    gaiaSetNullValue (dbf_field);
			}
		      else
			{
			    switch (dbf_field->Type)
			      {
			      case 'N':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_INTEGER)
				      gaiaSetIntValue (dbf_field,
						       sqlite3_column_int64
						       (stmt, i));
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				      gaiaSetDoubleValue (dbf_field,
							  sqlite3_column_double
							  (stmt, i));
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      case 'C':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_TEXT)
				    {
					dummy = (char *)
					    sqlite3_column_text (stmt, i);
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_INTEGER)
				    {
					sprintf (dbfbuf, FRMT64,
						 sqlite3_column_int64 (stmt,
								       i));
					gaiaSetStrValue (dbf_field, dbfbuf);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				    {
					sql = sqlite3_mprintf ("%1.6f",
							       sqlite3_column_double
							       (stmt, i));
					gaiaSetStrValue (dbf_field, sql);
					sqlite3_free (sql);
				    }
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      };
			}
		  }
		free_auxdbf (auxdbf);
		auxdbf = NULL;
		if (!gaiaWriteShpEntity (shp, dbf_write))
		    fprintf (stderr, "dataSeltzer: Shapefile write error\n");
		gaiaFreeDbfList (dbf_write);
		nrow++;
	    }
	  else
	      goto shp_error;
      }
    if (auxdbf != NULL)
	free_auxdbf (auxdbf);
    sqlite3_finalize (stmt);
    gaiaFlushShpHeaders (shp);
    gaiaFreeShapefile (shp);
    goto ok_shp;

  shp_error:
/* unable to export the Shapefile */
    if (auxdbf != NULL)
	free_auxdbf (auxdbf);
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (shp)
	gaiaFreeShapefile (shp);
    fprintf (stderr, "dataSeltzer: Shapefile generation error\n");
    goto error;

  ok_shp:
/* creating the SHP file within the ZipFile */
    file_path =
	sqlite3_mprintf ("%s_%s/%s_%s.shp", params->arg_view, timestamp2,
			 params->arg_view, timestamp2);
    tms = gmtime (&now);
    zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
    zip_info.tmz_date.tm_mon = tms->tm_mon;
    zip_info.tmz_date.tm_mday = tms->tm_mday;
    zip_info.tmz_date.tm_hour = tms->tm_hour;
    zip_info.tmz_date.tm_min = tms->tm_min;
    zip_info.tmz_date.tm_sec = tms->tm_sec;
    zip_info.dosDate = 0;
    zip_info.internal_fa = 0;
    zip_info.external_fa = 0;
    ret =
	zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (file_path);
    if (ret != Z_OK)
      {
	  free (timestamp);
	  fprintf (stderr,
		   "Unable to create a file within ZipFile: error %d\n", ret);
	  goto error;
      }

    x_path = sqlite3_mprintf ("%s.shp", shp_path);
    in = fopen (x_path, "rb");
    if (in == NULL)
	goto error;
    while (1)
      {
	  /* copying the DBF file into the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  zipWriteInFileInZip (zip, buf, nb);
      }
    fclose (in);
    unlink (x_path);
    sqlite3_free (x_path);
/* closing the SHP file in ZipFile */
    zipCloseFileInZip (zip);

/* creating the SHX file within the ZipFile */
    file_path =
	sqlite3_mprintf ("%s_%s/%s_%s.shx", params->arg_view, timestamp2,
			 params->arg_view, timestamp2);
    tms = gmtime (&now);
    zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
    zip_info.tmz_date.tm_mon = tms->tm_mon;
    zip_info.tmz_date.tm_mday = tms->tm_mday;
    zip_info.tmz_date.tm_hour = tms->tm_hour;
    zip_info.tmz_date.tm_min = tms->tm_min;
    zip_info.tmz_date.tm_sec = tms->tm_sec;
    zip_info.dosDate = 0;
    zip_info.internal_fa = 0;
    zip_info.external_fa = 0;
    ret =
	zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (file_path);
    if (ret != Z_OK)
      {
	  free (timestamp);
	  fprintf (stderr,
		   "Unable to create a file within ZipFile: error %d\n", ret);
	  goto error;
      }

    x_path = sqlite3_mprintf ("%s.shx", shp_path);
    in = fopen (x_path, "rb");
    if (in == NULL)
	goto error;
    while (1)
      {
	  /* copying the DBF file into the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  zipWriteInFileInZip (zip, buf, nb);
      }
    fclose (in);
    unlink (x_path);
    sqlite3_free (x_path);
/* closing the SHX file in ZipFile */
    zipCloseFileInZip (zip);

/* creating the DBF file within the ZipFile */
    file_path =
	sqlite3_mprintf ("%s_%s/%s_%s.dbf", params->arg_view, timestamp2,
			 params->arg_view, timestamp2);
    tms = gmtime (&now);
    zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
    zip_info.tmz_date.tm_mon = tms->tm_mon;
    zip_info.tmz_date.tm_mday = tms->tm_mday;
    zip_info.tmz_date.tm_hour = tms->tm_hour;
    zip_info.tmz_date.tm_min = tms->tm_min;
    zip_info.tmz_date.tm_sec = tms->tm_sec;
    zip_info.dosDate = 0;
    zip_info.internal_fa = 0;
    zip_info.external_fa = 0;
    ret =
	zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (file_path);
    if (ret != Z_OK)
      {
	  free (timestamp);
	  fprintf (stderr,
		   "Unable to create a file within ZipFile: error %d\n", ret);
	  goto error;
      }

    x_path = sqlite3_mprintf ("%s.dbf", shp_path);
    in = fopen (x_path, "rb");
    if (in == NULL)
	goto error;
    while (1)
      {
	  /* copying the DBF file into the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  zipWriteInFileInZip (zip, buf, nb);
      }
    fclose (in);
    unlink (x_path);
    sqlite3_free (x_path);
/* closing the DBF file in ZipFile */
    zipCloseFileInZip (zip);

    if (has_prj_file)
      {
/* creating the PRJ file within the ZipFile */
	  file_path =
	      sqlite3_mprintf ("%s_%s/%s_%s.prj", params->arg_view,
			       timestamp2, params->arg_view, timestamp2);
	  tms = gmtime (&now);
	  zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
	  zip_info.tmz_date.tm_mon = tms->tm_mon;
	  zip_info.tmz_date.tm_mday = tms->tm_mday;
	  zip_info.tmz_date.tm_hour = tms->tm_hour;
	  zip_info.tmz_date.tm_min = tms->tm_min;
	  zip_info.tmz_date.tm_sec = tms->tm_sec;
	  zip_info.dosDate = 0;
	  zip_info.internal_fa = 0;
	  zip_info.external_fa = 0;
	  ret =
	      zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL,
				   0, NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
	  sqlite3_free (file_path);
	  if (ret != Z_OK)
	    {
		free (timestamp);
		fprintf (stderr,
			 "Unable to create a file within ZipFile: error %d\n",
			 ret);
		goto error;
	    }

	  x_path = sqlite3_mprintf ("%s.prj", shp_path);
	  in = fopen (x_path, "rb");
	  if (in == NULL)
	      goto error;
	  while (1)
	    {
		/* copying the PRJ file into the zipfile */
		int nb = fread (buf, 1, BUFFSIZE, in);
		if (nb <= 0)
		    break;
		zipWriteInFileInZip (zip, buf, nb);
	    }
	  fclose (in);
	  unlink (x_path);
	  sqlite3_free (x_path);
/* closing the PRJ file in ZipFile */
	  zipCloseFileInZip (zip);
      }

/* inserting download metainfo */
    do_write_metainfo (zip, params, *truncation, timestamp2, &zip_info);

    if (params->extra_data_dir != NULL)
      {
	  /* attempting to retrieve datasource-specific extra-files */
	  struct dirent *entry;
	  DIR *dir;
	  char *path;

	  if (params->extra_data_subdirs)
	    {
		/* scanning the datasource specific subdir (if any) */
		char *view = to_lower (params->arg_view);
		char *dir_path =
		    sqlite3_mprintf ("%s/%s", params->extra_data_dir, view);
		free (view);
		dir = opendir (dir_path);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    path =
				sqlite3_mprintf ("%s/%s", dir_path,
						 entry->d_name);
			    if (!test_regular_file (path, &zip_info))
			      {
				  sqlite3_free (path);
				  continue;
			      }
			    add_file_to_zipfile (zip, params->arg_view,
						 timestamp2, path, &zip_info);
			    sqlite3_free (path);
			}
		      closedir (dir);
		  }
		sqlite3_free (dir_path);
	    }
	  else
	    {
		/* scanning the generic extra_data_dir */
		dir = opendir (params->extra_data_dir);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    if (strncmp
				(entry->d_name, params->arg_view,
				 strlen (params->arg_view)) == 0)
			      {
				  /* attempting to include every file starting with the View name */
				  path =
				      sqlite3_mprintf ("%s/%s",
						       params->extra_data_dir,
						       entry->d_name);
				  if (!test_regular_file (path, &zip_info))
				    {
					sqlite3_free (path);
					continue;
				    }
				  add_file_to_zipfile (zip,
						       params->arg_view,
						       timestamp2, path,
						       &zip_info);
				  sqlite3_free (path);
			      }
			}
		      closedir (dir);
		  }
	    }
      }

    extra = params->first_extra;
    while (extra != NULL)
      {
	  /* inserting any extra-file into the ZipFile */
	  if (!test_regular_file (extra->path, &zip_info))
	    {
		extra = extra->next;
		continue;
	    }
	  add_file_to_zipfile (zip, params->arg_view, timestamp2,
			       extra->path, &zip_info);
	  extra = extra->next;
      }


/* final steps */
    zipClose (zip, NULL);
    zip = NULL;
    in = fopen (temp_path, "rb");
    if (in == NULL)
	goto error;
    md5 = gaiaCreateMD5Checksum ();
    while (1)
      {
	  /* computing Content-Length and MD5 checksum */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  bytes += nb;
	  gaiaUpdateMD5Checksum (md5, buf, nb);
      }
    md5_checksum = gaiaFinalizeMD5Checksum (md5);
    gaiaFreeMD5Checksum (md5);

/* sending HTTP headers */
    cgi_send_header ("Content-Type: application/zip");
    file_path =
	sqlite3_mprintf ("%s_%s_%s.zip", params->arg_view, timestamp2,
			 md5_checksum);
    free (md5_checksum);
    item =
	sqlite3_mprintf ("Content-Disposition: attachment; filename=%s",
			 file_path);
    sqlite3_free (file_path);
    cgi_send_header (item);
    sqlite3_free (item);
    item = sqlite3_mprintf ("Content-Length: %d\n", bytes);
    params->bytes = bytes;
    cgi_send_header (item);
    sqlite3_free (item);

    rewind (in);
    while (1)
      {
	  /* updloading the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  fwrite (buf, 1, nb, stdout);
      }

    fclose (in);
    unlink (temp_path);
    sqlite3_free (temp_path);
    sqlite3_free (shp_path);
    free (timestamp);
    free (timestamp2);
    free (geometry);
/* updating the LOG */
    do_update_log (params);
    return 1;

  error:
    if (stmt != NULL)
	sqlite3_finalize (stmt);
    if (zip != NULL)
	zipClose (zip, NULL);
    unlink (temp_path);
    sqlite3_free (temp_path);
    if (timestamp != NULL)
	free (timestamp);
    if (timestamp2 != NULL)
	free (timestamp2);
    if (geometry != NULL)
	free (geometry);
    if (shp_path != NULL)
      {
	  x_path = sqlite3_mprintf ("%s.shp", shp_path);
	  unlink (x_path);
	  sqlite3_free (x_path);
	  x_path = sqlite3_mprintf ("%s.shx", shp_path);
	  unlink (x_path);
	  sqlite3_free (x_path);
	  x_path = sqlite3_mprintf ("%s.dbf", shp_path);
	  unlink (x_path);
	  sqlite3_free (x_path);
	  if (has_prj_file)
	    {
		x_path = sqlite3_mprintf ("%s.prj", shp_path);
		unlink (x_path);
		sqlite3_free (x_path);
	    }
	  sqlite3_free (shp_path);
      }
    return 0;
}

static int
do_output_dbf_resultset (struct config *params, int *truncation)
{
/* will return the DBF resultset zipfile */
    int ret;
    sqlite3_stmt *stmt = NULL;
    char *sql;
    char *item;
    int i;
    zipFile zip = NULL;
    char *temp_path;
    char *file_path;
    time_t now;
    char *timestamp = NULL;
    char *timestamp2 = NULL;
    int bytes = 0;
    FILE *in;
    unsigned char buf[BUFFSIZE];
    void *md5;
    char *md5_checksum;
    struct extra_data_file *extra;
    zip_fileinfo zip_info;
    struct tm *tms;
    int rows;
    int row1 = 0;
    int n_cols = 0;
    int offset = 0;
    int type;
    gaiaDbfPtr dbf = NULL;
    gaiaDbfListPtr dbf_export_list = NULL;
    gaiaDbfListPtr dbf_list = NULL;
    gaiaDbfListPtr dbf_write;
    gaiaDbfFieldPtr dbf_field;
    int *max_length = NULL;
    int *sql_type = NULL;
    char *dummy;
    int len;
    struct auxdbf_list *auxdbf = NULL;
    char *dbf_path;
    char dbfbuf[256];
    int nrow = 0;

    *truncation = 0;
/* preparing the SQL Query statement */
    sql = do_prepare_sql (params);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;

/* creating the temporary zipfile */
    time (&now);
    dbf_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.dbf", params->temp_dir, now, getpid ());
    temp_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.zip", params->temp_dir, now, getpid ());
    zip = zipOpen (temp_path, APPEND_STATUS_CREATE);
    if (zip == NULL)
      {
	  fprintf (stderr, "Unable to create a ZipFile\n");
	  sqlite3_free (temp_path);
	  return 0;
      }

    rows = 0;
    while (1)
      {
	  /*
	     / Pass I - scrolling the result set to compute real DBF attributes' sizes and types 
	   */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		/* processing a result set row */
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		row1++;
		if (n_cols == 0)
		  {
		      /* this one is the first row, so we are going to prepare the DBF Fields list */
		      n_cols = sqlite3_column_count (stmt);
		      dbf_export_list = gaiaAllocDbfList ();
		      max_length = (int *) malloc (sizeof (int) * n_cols);
		      sql_type = (int *) malloc (sizeof (int) * n_cols);
		      for (i = 0; i < n_cols; i++)
			{
			    /* initializes the DBF export fields */
			    dummy = (char *) sqlite3_column_name (stmt, i);
			    gaiaAddDbfField (dbf_export_list, dummy, '\0',
					     0, 0, 0);
			    max_length[i] = 0;
			    sql_type[i] = SQLITE_NULL;
			}
		  }
		for (i = 0; i < n_cols; i++)
		  {
		      /* update the DBF export fields analyzing fetched data */
		      type = sqlite3_column_type (stmt, i);
		      if (type == SQLITE_NULL || type == SQLITE_BLOB)
			  continue;
		      if (type == SQLITE_TEXT)
			{
			    len = sqlite3_column_bytes (stmt, i);
			    sql_type[i] = SQLITE_TEXT;
			    if (len > max_length[i])
				max_length[i] = len;
			}
		      else if (type == SQLITE_FLOAT
			       && sql_type[i] != SQLITE_TEXT)
			  sql_type[i] = SQLITE_FLOAT;	/* promoting a numeric column to be DOUBLE */
		      else if (type == SQLITE_INTEGER
			       && (sql_type[i] == SQLITE_NULL
				   || sql_type[i] == SQLITE_INTEGER))
			  sql_type[i] = SQLITE_INTEGER;	/* promoting a null column to be INTEGER */
		      if (type == SQLITE_INTEGER && max_length[i] < 18)
			  max_length[i] = 18;
		      if (type == SQLITE_FLOAT && max_length[i] < 24)
			  max_length[i] = 24;
		  }
		nrow++;
	    }
	  else
	      goto dbf_error;
      }
    if (dbf_export_list == NULL)
	goto dbf_error;
    i = 0;
    offset = 0;
    dbf_list = gaiaAllocDbfList ();
    dbf_field = dbf_export_list->First;
    while (dbf_field)
      {
	  /* preparing the final DBF attribute list */
	  if (sql_type[i] == SQLITE_NULL || sql_type[i] == SQLITE_BLOB)
	    {
		i++;
		dbf_field = dbf_field->Next;
		continue;
	    }
	  if (sql_type[i] == SQLITE_TEXT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'C', offset,
				 max_length[i], 0);
		offset += max_length[i];
	    }
	  if (sql_type[i] == SQLITE_FLOAT)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 19, 6);
		offset += 19;
	    }
	  if (sql_type[i] == SQLITE_INTEGER)
	    {
		gaiaAddDbfField (dbf_list, dbf_field->Name, 'N', offset, 18, 0);
		offset += 18;
	    }
	  i++;
	  dbf_field = dbf_field->Next;
      }
    free (max_length);
    free (sql_type);
    gaiaFreeDbfList (dbf_export_list);
    dbf_export_list = NULL;

/* resetting SQLite query */
    ret = sqlite3_reset (stmt);
    if (ret != SQLITE_OK)
	goto dbf_error;
/* trying to open the DBF file */
    dbf = gaiaAllocDbf ();
/* xfering export-list ownership */
    dbf->Dbf = dbf_list;
    dbf_list = NULL;
    gaiaOpenDbfWrite (dbf, dbf_path, "UTF-8", "UTF-8");
    if (!(dbf->Valid))
	goto dbf_error;

    nrow = 0;
    while (1)
      {
	  /* Pass II - scrolling the result set to dump data into DBF */
	  ret = sqlite3_step (stmt);
	  if (ret == SQLITE_DONE)
	      break;		/* end of result set */
	  if (ret == SQLITE_ROW)
	    {
		rows++;
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		    break;
		dbf_write = gaiaCloneDbfEntity (dbf->Dbf);
		auxdbf = alloc_auxdbf (dbf_write);
		for (i = 0; i < n_cols; i++)
		  {
		      dummy = (char *) sqlite3_column_name (stmt, i);
		      dbf_field = getDbfField (auxdbf, dummy);
		      if (!dbf_field)
			  continue;
		      if (sqlite3_column_type (stmt, i) == SQLITE_NULL
			  || sqlite3_column_type (stmt, i) == SQLITE_BLOB)
			{
			    /* handling NULL values */
			    gaiaSetNullValue (dbf_field);
			}
		      else
			{
			    switch (dbf_field->Type)
			      {
			      case 'N':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_INTEGER)
				      gaiaSetIntValue (dbf_field,
						       sqlite3_column_int64
						       (stmt, i));
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				      gaiaSetDoubleValue (dbf_field,
							  sqlite3_column_double
							  (stmt, i));
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      case 'C':
				  if (sqlite3_column_type (stmt, i) ==
				      SQLITE_TEXT)
				    {
					dummy = (char *)
					    sqlite3_column_text (stmt, i);
					gaiaSetStrValue (dbf_field, dummy);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_INTEGER)
				    {
					sprintf (dbfbuf, FRMT64,
						 sqlite3_column_int64 (stmt,
								       i));
					gaiaSetStrValue (dbf_field, dbfbuf);
				    }
				  else if (sqlite3_column_type (stmt, i) ==
					   SQLITE_FLOAT)
				    {
					sql = sqlite3_mprintf ("%1.6f",
							       sqlite3_column_double
							       (stmt, i));
					gaiaSetStrValue (dbf_field, sql);
					sqlite3_free (sql);
				    }
				  else
				      gaiaSetNullValue (dbf_field);
				  break;
			      };
			}
		  }
		free_auxdbf (auxdbf);
		auxdbf = NULL;
		if (!gaiaWriteDbfEntity (dbf, dbf_write))
		    fprintf (stderr, "dataSeltzer: DBF write error\n");
		gaiaFreeDbfList (dbf_write);
		nrow++;
	    }
	  else
	      goto dbf_error;
      }
    if (auxdbf != NULL)
	free_auxdbf (auxdbf);
    sqlite3_finalize (stmt);
    gaiaFlushDbfHeader (dbf);
    gaiaFreeDbf (dbf);
    goto ok_dbf;

  dbf_error:
/* unable to export the DBF */
    if (auxdbf != NULL)
	free_auxdbf (auxdbf);
    if (dbf_export_list)
	gaiaFreeDbfList (dbf_export_list);
    if (dbf_list)
	gaiaFreeDbfList (dbf_list);
    if (dbf)
	gaiaFreeDbf (dbf);
    fprintf (stderr, "dataSeltzer: DBF file generation error\n");
    goto error;

  ok_dbf:
/* creating a file within the ZipFile */
    timestamp = get_current_timestamp (params);
    timestamp2 = make_timestamp2 (timestamp);
    file_path =
	sqlite3_mprintf ("%s_%s/%s_%s.dbf", params->arg_view, timestamp2,
			 params->arg_view, timestamp2);
    tms = gmtime (&now);
    zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
    zip_info.tmz_date.tm_mon = tms->tm_mon;
    zip_info.tmz_date.tm_mday = tms->tm_mday;
    zip_info.tmz_date.tm_hour = tms->tm_hour;
    zip_info.tmz_date.tm_min = tms->tm_min;
    zip_info.tmz_date.tm_sec = tms->tm_sec;
    zip_info.dosDate = 0;
    zip_info.internal_fa = 0;
    zip_info.external_fa = 0;
    ret =
	zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (file_path);
    if (ret != Z_OK)
      {
	  free (timestamp);
	  fprintf (stderr,
		   "Unable to create a file within ZipFile: error %d\n", ret);
	  goto error;
      }

    in = fopen (dbf_path, "rb");
    if (in == NULL)
	goto error;
    while (1)
      {
	  /* copying the DBF file into the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  zipWriteInFileInZip (zip, buf, nb);
      }
    fclose (in);
    unlink (dbf_path);
    sqlite3_free (dbf_path);
/* closing the DBF file in ZipFile */
    zipCloseFileInZip (zip);

/* inserting download metainfo */
    do_write_metainfo (zip, params, *truncation, timestamp2, &zip_info);

    if (params->extra_data_dir != NULL)
      {
	  /* attempting to retrieve datasource-specific extra-files */
	  struct dirent *entry;
	  DIR *dir;
	  char *path;

	  if (params->extra_data_subdirs)
	    {
		/* scanning the datasource specific subdir (if any) */
		char *view = to_lower (params->arg_view);
		char *dir_path =
		    sqlite3_mprintf ("%s/%s", params->extra_data_dir, view);
		free (view);
		dir = opendir (dir_path);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    path =
				sqlite3_mprintf ("%s/%s", dir_path,
						 entry->d_name);
			    if (!test_regular_file (path, &zip_info))
			      {
				  sqlite3_free (path);
				  continue;
			      }
			    add_file_to_zipfile (zip, params->arg_view,
						 timestamp2, path, &zip_info);
			    sqlite3_free (path);
			}
		      closedir (dir);
		  }
		sqlite3_free (dir_path);
	    }
	  else
	    {
		/* scanning the generic extra_data_dir */
		dir = opendir (params->extra_data_dir);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    if (strncmp
				(entry->d_name, params->arg_view,
				 strlen (params->arg_view)) == 0)
			      {
				  /* attempting to include every file starting with the View name */
				  path =
				      sqlite3_mprintf ("%s/%s",
						       params->extra_data_dir,
						       entry->d_name);
				  if (!test_regular_file (path, &zip_info))
				    {
					sqlite3_free (path);
					continue;
				    }
				  add_file_to_zipfile (zip,
						       params->arg_view,
						       timestamp2, path,
						       &zip_info);
				  sqlite3_free (path);
			      }
			}
		      closedir (dir);
		  }
	    }
      }

    extra = params->first_extra;
    while (extra != NULL)
      {
	  /* inserting any extra-file into the ZipFile */
	  if (!test_regular_file (extra->path, &zip_info))
	    {
		extra = extra->next;
		continue;
	    }
	  add_file_to_zipfile (zip, params->arg_view, timestamp2,
			       extra->path, &zip_info);
	  extra = extra->next;
      }


/* final steps */
    zipClose (zip, NULL);
    zip = NULL;
    in = fopen (temp_path, "rb");
    if (in == NULL)
	goto error;
    md5 = gaiaCreateMD5Checksum ();
    while (1)
      {
	  /* computing Content-Length and MD5 checksum */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  bytes += nb;
	  gaiaUpdateMD5Checksum (md5, buf, nb);
      }
    md5_checksum = gaiaFinalizeMD5Checksum (md5);
    gaiaFreeMD5Checksum (md5);

/* sending HTTP headers */
    cgi_send_header ("Content-Type: application/zip");
    file_path =
	sqlite3_mprintf ("%s_%s_%s.zip", params->arg_view, timestamp2,
			 md5_checksum);
    free (md5_checksum);
    item =
	sqlite3_mprintf ("Content-Disposition: attachment; filename=%s",
			 file_path);
    sqlite3_free (file_path);
    cgi_send_header (item);
    sqlite3_free (item);
    item = sqlite3_mprintf ("Content-Length: %d\n", bytes);
    params->bytes = bytes;
    cgi_send_header (item);
    sqlite3_free (item);

    rewind (in);
    while (1)
      {
	  /* updloading the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  fwrite (buf, 1, nb, stdout);
      }

    fclose (in);
    unlink (temp_path);
    sqlite3_free (temp_path);
    free (timestamp);
    free (timestamp2);
/* updating the LOG */
    do_update_log (params);
    return 1;

  error:
    if (stmt != NULL)
	sqlite3_finalize (stmt);
    if (zip != NULL)
	zipClose (zip, NULL);
    unlink (temp_path);
    sqlite3_free (temp_path);
    if (timestamp != NULL)
	free (timestamp);
    if (timestamp2 != NULL)
	free (timestamp2);
    if (dbf_path != NULL)
      {
	  unlink (dbf_path);
	  sqlite3_free (dbf_path);
      }
    return 0;
}

static int
do_output_sqlite_resultset (struct config *params, int *truncation)
{
/* will return the SQLite/SpatiaLite DB zipfile */
    sqlite3 *out;
    int ret;
    char *err_msg = NULL;
    void *cache = NULL;
    sqlite3_stmt *stmt_in = NULL;
    sqlite3_stmt *stmt_out = NULL;
    char *sql;
    char *item;
    int n_columns;
    int i;
    zipFile zip;
    char *temp_path;
    char *file_path;
    char *sqlite_path;
    time_t now;
    char *timestamp = NULL;
    char *timestamp2 = NULL;
    int bytes = 0;
    FILE *in;
    unsigned char buf[BUFFSIZE];
    void *md5;
    char *md5_checksum;
    struct extra_data_file *extra;
    zip_fileinfo zip_info;
    struct tm *tms;
    int nrow = 0;
    struct xml_meta_columns *meta_cols;
    char *rowid_name = NULL;
    int ok;
    char *xtable;
    char *old_sql;
    char *geometry = NULL;
    char *type = NULL;
    char *dims = NULL;
    int srid = -1;

    *truncation = 0;
    get_geom_params (params, &geometry, &type, &dims, &srid);

/* executing the SQL Query statement */
    sql = do_prepare_sql (params);
    ret =
	sqlite3_prepare_v2 (params->db_handle, sql, strlen (sql), &stmt_in,
			    NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
	return 0;

/* creating the temporary zipfile */
    time (&now);
    sqlite_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.sqlite", params->temp_dir, now,
			 getpid ());
    temp_path =
	sqlite3_mprintf ("%s/tmp_%d_%d.zip", params->temp_dir, now, getpid ());
    zip = zipOpen (temp_path, APPEND_STATUS_CREATE);
    if (zip == NULL)
      {
	  fprintf (stderr, "Unable to create a ZipFile\n");
	  sqlite3_free (temp_path);
	  return 0;
      }

/* creating the output DB-file */
    ret =
	sqlite3_open_v2 (sqlite_path, &out,
			 SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "dataSelzer: cannot open '%s': %s\n", sqlite_path,
		   sqlite3_errmsg (out));
	  goto error;
      }
    cache = spatialite_alloc_connection ();
    spatialite_init_ex (out, cache, 0);

    meta_cols = build_xml_meta_cols (params, stmt_in);
    n_columns = sqlite3_column_count (stmt_in);
    rowid_name = sqlite3_mprintf ("uid");
    ok = 1;
    while (ok)
      {
	  int alias = 1;
	  ok = 0;
	  for (i = 0; i < n_columns; i++)
	    {
		/* checking for RowIdName unicity */
		const char *colname = sqlite3_column_name (stmt_in, i);
		if (strcasecmp (colname, rowid_name) == 0)
		  {
		      sqlite3_free (rowid_name);
		      rowid_name = sqlite3_mprintf ("rowid_%d", alias++);
		      ok = 1;
		      break;
		  }
	    }
      }

/* initialize the output DB */
    ret =
	sqlite3_exec (out, "SELECT InitSpatialMetadata(1)", NULL, NULL,
		      &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "InitSpatialMetadata() error: %s\n", err_msg);
	  sqlite3_free (err_msg);
	  goto error;
      }

/* creating the output table */
    xtable = gaiaDoubleQuotedSql (params->arg_view);
    sql = sqlite3_mprintf ("CREATE TABLE \"%s\" ("
			   "%s INTEGER PRIMARY KEY AUTOINCREMENT", xtable,
			   rowid_name);
    free (xtable);
    for (i = 0; i < n_columns; i++)
      {
	  /* defining all columns for the output table */
	  const char *colname = sqlite3_column_name (stmt_in, i);
	  struct xml_meta_col *col = find_xml_meta_col (meta_cols, colname);
	  char *sql_aux;
	  if (geometry != NULL)
	    {
		if (strcasecmp (geometry, colname) == 0)
		  {
		      /* skipping the Geometry column */
		      continue;
		  }
	    }
	  old_sql = sql;
	  xtable = gaiaDoubleQuotedSql (colname);
	  if (col->nullable)
	      sql_aux = sqlite3_mprintf (", \"%s\" %s", xtable, col->type);
	  else
	      sql_aux =
		  sqlite3_mprintf (", \"%s\" %s NOT NULL", xtable, col->type);
	  free (xtable);
	  sql = sqlite3_mprintf ("%s%s", old_sql, sql_aux);
	  sqlite3_free (old_sql);
	  sqlite3_free (sql_aux);
      }
    destroy_xml_meta_cols (meta_cols);
    old_sql = sql;
    sql = sqlite3_mprintf ("%s)", old_sql);
    sqlite3_free (old_sql);
    ret = sqlite3_exec (out, sql, NULL, NULL, &err_msg);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "CREATE TABLE: %s\n", err_msg);
	  sqlite3_free (err_msg);
	  goto error;
      }

    if (geometry != NULL)
      {
	  /* inserting the Geometry column */
	  sql = sqlite3_mprintf ("SELECT AddGeometryColumn(%Q, %Q, %d, %Q, %Q)",
				 params->arg_view, geometry, srid, type, dims);
	  ret = sqlite3_exec (out, sql, NULL, NULL, &err_msg);
	  sqlite3_free (sql);
	  if (ret != SQLITE_OK)
	    {
		fprintf (stderr, "AddGeometryColumn error: %s\n", err_msg);
		sqlite3_free (err_msg);
		goto error;
	    }
	  /* creating a Spatial Index */
	  sql = sqlite3_mprintf ("SELECT CreateSpatialIndex(%Q, %Q)",
				 params->arg_view, geometry);
	  ret = sqlite3_exec (out, sql, NULL, NULL, &err_msg);
	  sqlite3_free (sql);
	  if (ret != SQLITE_OK)
	    {
		fprintf (stderr, "CreateSpatialIndex error: %s\n", err_msg);
		sqlite3_free (err_msg);
		goto error;
	    }
      }

/* creating the INSERT statement */
    xtable = gaiaDoubleQuotedSql (params->arg_view);
    sql = sqlite3_mprintf ("INSERT INTO \"%s\" (\"%s\"", xtable, rowid_name);
    free (xtable);
    sqlite3_free (rowid_name);
    for (i = 0; i < n_columns; i++)
      {
	  /* INSERT columns */
	  const char *colname = sqlite3_column_name (stmt_in, i);
	  char *sql_aux;
	  xtable = gaiaDoubleQuotedSql (colname);
	  sql_aux = sqlite3_mprintf (", \"%s\"", xtable);
	  free (xtable);
	  old_sql = sql;
	  sql = sqlite3_mprintf ("%s%s", old_sql, sql_aux);
	  sqlite3_free (old_sql);
	  sqlite3_free (sql_aux);
      }
    old_sql = sql;
    sql = sqlite3_mprintf ("%s) VALUES (?", old_sql);
    sqlite3_free (old_sql);
    for (i = 0; i < n_columns; i++)
      {
	  /* parameters */
	  old_sql = sql;
	  sql = sqlite3_mprintf ("%s, ?", old_sql);
	  sqlite3_free (old_sql);
      }
    old_sql = sql;
    sql = sqlite3_mprintf ("%s)", old_sql);
    sqlite3_free (old_sql);
    ret = sqlite3_prepare_v2 (out, sql, strlen (sql), &stmt_out, NULL);
    sqlite3_free (sql);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "INSERT INTO error: %s\n", sqlite3_errmsg (out));
	  goto error;
      }

/* beginning a Transaction */
    ret = sqlite3_exec (out, "BEGIN", NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "BEGIN error: %s\n", err_msg);
	  sqlite3_free (err_msg);
	  goto error;
      }

    while (1)
      {
	  /* fetching rows from the resultset */
	  ret = sqlite3_step (stmt_in);
	  if (ret == SQLITE_DONE)
	      break;
	  if (ret == SQLITE_ROW)
	    {
		/* ok, found a valid row */
		if (params->max_resultset_rows > 0
		    && nrow > params->max_resultset_rows)
		  {
		      *truncation = 1;
		      break;
		  }
		sqlite3_reset (stmt_out);
		sqlite3_clear_bindings (stmt_out);
		sqlite3_bind_int (stmt_out, 1, nrow);
		for (i = 0; i < n_columns; i++)
		  {
		      /* exporting every cell */
		      switch (sqlite3_column_type (stmt_in, i))
			{
			case SQLITE_NULL:
			    sqlite3_bind_null (stmt_out, i + 2);
			    break;
			case SQLITE_INTEGER:
			    sqlite3_bind_int64 (stmt_out, i + 2,
						sqlite3_column_int64 (stmt_in,
								      i));
			    break;
			case SQLITE_FLOAT:
			    sqlite3_bind_double (stmt_out, i + 2,
						 sqlite3_column_double (stmt_in,
									i));
			    break;
			case SQLITE_TEXT:
			    sqlite3_bind_text (stmt_out, i + 2,
					       (const char *)
					       sqlite3_column_text (stmt_in, i),
					       sqlite3_column_bytes (stmt_in,
								     i),
					       SQLITE_STATIC);
			    break;
			case SQLITE_BLOB:
			    sqlite3_bind_blob (stmt_out, i + 2,
					       sqlite3_column_blob (stmt_in, i),
					       sqlite3_column_bytes (stmt_in,
								     i),
					       SQLITE_STATIC);
			    break;
			}
		  }
		ret = sqlite3_step (stmt_out);
		if (ret == SQLITE_DONE || ret == SQLITE_ROW)
		    ;
		else
		  {
		      fprintf (stderr, "INSERT sqlite3_step error: %s\n",
			       sqlite3_errmsg (out));
		      goto error;
		  }
		nrow++;
	    }
	  else
	      break;
      }
    sqlite3_finalize (stmt_in);
    sqlite3_finalize (stmt_out);

/* committing the pending Transaction */
    ret = sqlite3_exec (out, "COMMIT", NULL, NULL, &err_msg);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "COMMIT error: %s\n", err_msg);
	  sqlite3_free (err_msg);
	  goto error;
      }
    sqlite3_close (out);
    out = NULL;
    spatialite_cleanup_ex (cache);
    cache = NULL;

/* creating a file within the ZipFile */
    timestamp = get_current_timestamp (params);
    timestamp2 = make_timestamp2 (timestamp);
    file_path =
	sqlite3_mprintf ("%s_%s/%s_%s.sqlite", params->arg_view, timestamp2,
			 params->arg_view, timestamp2);
    tms = gmtime (&now);
    zip_info.tmz_date.tm_year = (1900 + tms->tm_year) - 1980;
    zip_info.tmz_date.tm_mon = tms->tm_mon;
    zip_info.tmz_date.tm_mday = tms->tm_mday;
    zip_info.tmz_date.tm_hour = tms->tm_hour;
    zip_info.tmz_date.tm_min = tms->tm_min;
    zip_info.tmz_date.tm_sec = tms->tm_sec;
    zip_info.dosDate = 0;
    zip_info.internal_fa = 0;
    zip_info.external_fa = 0;
    ret =
	zipOpenNewFileInZip (zip, file_path, &zip_info, NULL, 0, NULL, 0,
			     NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
    sqlite3_free (file_path);
    if (ret != Z_OK)
      {
	  free (timestamp);
	  fprintf (stderr,
		   "Unable to create a file within ZipFile: error %d\n", ret);
	  goto error;
      }

    in = fopen (sqlite_path, "rb");
    if (in == NULL)
	goto error;
    while (1)
      {
	  /* copying the DBF file into the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  zipWriteInFileInZip (zip, buf, nb);
      }
    fclose (in);
    unlink (sqlite_path);
    sqlite3_free (sqlite_path);
/* closing the DBF file in ZipFile */
    zipCloseFileInZip (zip);

/* inserting download metainfo */
    do_write_metainfo (zip, params, *truncation, timestamp2, &zip_info);

    if (params->extra_data_dir != NULL)
      {
	  /* attempting to retrieve datasource-specific extra-files */
	  struct dirent *entry;
	  DIR *dir;
	  char *path;

	  if (params->extra_data_subdirs)
	    {
		/* scanning the datasource specific subdir (if any) */
		char *view = to_lower (params->arg_view);
		char *dir_path =
		    sqlite3_mprintf ("%s/%s", params->extra_data_dir, view);
		free (view);
		dir = opendir (dir_path);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    path =
				sqlite3_mprintf ("%s/%s", dir_path,
						 entry->d_name);
			    if (!test_regular_file (path, &zip_info))
			      {
				  sqlite3_free (path);
				  continue;
			      }
			    add_file_to_zipfile (zip, params->arg_view,
						 timestamp2, path, &zip_info);
			    sqlite3_free (path);
			}
		      closedir (dir);
		  }
		sqlite3_free (dir_path);
	    }
	  else
	    {
		/* scanning the generic extra_data_dir */
		dir = opendir (params->extra_data_dir);
		if (dir)
		  {
		      while (1)
			{
			    /* attempting to include every file from the subdir */
			    entry = readdir (dir);
			    if (!entry)
				break;
			    if (strncmp
				(entry->d_name, params->arg_view,
				 strlen (params->arg_view)) == 0)
			      {
				  /* attempting to include every file starting with the View name */
				  path =
				      sqlite3_mprintf ("%s/%s",
						       params->extra_data_dir,
						       entry->d_name);
				  if (!test_regular_file (path, &zip_info))
				    {
					sqlite3_free (path);
					continue;
				    }
				  add_file_to_zipfile (zip,
						       params->arg_view,
						       timestamp2, path,
						       &zip_info);
				  sqlite3_free (path);
			      }
			}
		      closedir (dir);
		  }
	    }
      }

    extra = params->first_extra;
    while (extra != NULL)
      {
	  /* inserting any extra-file into the ZipFile */
	  if (!test_regular_file (extra->path, &zip_info))
	    {
		extra = extra->next;
		continue;
	    }
	  add_file_to_zipfile (zip, params->arg_view, timestamp2,
			       extra->path, &zip_info);
	  extra = extra->next;
      }


/* final steps */
    zipClose (zip, NULL);
    zip = NULL;
    in = fopen (temp_path, "rb");
    if (in == NULL)
	goto error;
    md5 = gaiaCreateMD5Checksum ();
    while (1)
      {
	  /* computing Content-Length and MD5 checksum */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  bytes += nb;
	  gaiaUpdateMD5Checksum (md5, buf, nb);
      }
    md5_checksum = gaiaFinalizeMD5Checksum (md5);
    gaiaFreeMD5Checksum (md5);

/* sending HTTP headers */
    cgi_send_header ("Content-Type: application/zip");
    file_path =
	sqlite3_mprintf ("%s_%s_%s.zip", params->arg_view, timestamp2,
			 md5_checksum);
    free (md5_checksum);
    item =
	sqlite3_mprintf ("Content-Disposition: attachment; filename=%s",
			 file_path);
    sqlite3_free (file_path);
    cgi_send_header (item);
    sqlite3_free (item);
    item = sqlite3_mprintf ("Content-Length: %d\n", bytes);
    params->bytes = bytes;
    cgi_send_header (item);
    sqlite3_free (item);

    rewind (in);
    while (1)
      {
	  /* updloading the zipfile */
	  int nb = fread (buf, 1, BUFFSIZE, in);
	  if (nb <= 0)
	      break;
	  fwrite (buf, 1, nb, stdout);
      }

    fclose (in);
    unlink (temp_path);
    sqlite3_free (temp_path);
    free (timestamp);
    free (timestamp2);
/* updating the LOG */
    do_update_log (params);
    if (geometry != NULL)
	free (geometry);
    if (type != NULL)
	free (type);
    if (dims != NULL)
	free (dims);
    return 1;

  error:
    if (geometry != NULL)
	free (geometry);
    if (type != NULL)
	free (type);
    if (dims != NULL)
	free (dims);
    if (out != NULL)
	sqlite3_close (out);
    if (cache != NULL)
	spatialite_cleanup_ex (cache);
    if (stmt_in != NULL)
	sqlite3_finalize (stmt_in);
    if (zip != NULL)
	zipClose (zip, NULL);
    unlink (temp_path);
    sqlite3_free (temp_path);
    if (timestamp != NULL)
	free (timestamp);
    if (timestamp2 != NULL)
	free (timestamp2);
    if (sqlite_path != NULL)
      {
	  unlink (sqlite_path);
	  sqlite3_free (sqlite_path);
      }
    return 0;
}

static void
do_output_get_url (struct config *params)
{
/* will return the HTML GetURL page */
    char *path;
    char *url;
    char *query_string;

/* HTML header */
    path = sqlite3_mprintf ("%s/geturl_header.inc", params->include_dir);
    cgi_include (path);
    sqlite3_free (path);

/* building the GetURL */
    query_string = do_build_get_url (params, 1);
    url = sqlite3_mprintf ("%s%s", params->cgi_url, query_string);
    sqlite3_free (query_string);

    myputs (url);
    sqlite3_free (url);

/* HTML footer */
    path = sqlite3_mprintf ("%s/geturl_footer.inc", params->include_dir);
    cgi_include (path);
    sqlite3_free (path);
}

static void
do_output_frameset (struct config *params)
{
/* will return the HTML main frame */
    char *path;

/* HTML Frame */
    path = sqlite3_mprintf ("%s/frame.inc", params->include_dir);
    cgi_include (path);
    sqlite3_free (path);
}

static void
open_db (const char *path, sqlite3 ** handle, void *cache)
{
/* opening the DB */
    sqlite3 *db_handle;
    int ret;

    ret = sqlite3_open_v2 (path, &db_handle, SQLITE_OPEN_READONLY, NULL);
    if (ret != SQLITE_OK)
      {
	  fprintf (stderr, "dataSelzer: cannot open '%s': %s\n", path,
		   sqlite3_errmsg (db_handle));
	  sqlite3_close (db_handle);
	  *handle = NULL;
	  return;
      }
    spatialite_init_ex (db_handle, cache, 0);

    *handle = db_handle;
    return;
}

static void
set_column (struct config *params, const char *value)
{
/* setting the Help Column Name */
    int len;
    if (params->column != NULL)
	free (params->column);
    params->column = NULL;
    if (value == NULL)
	return;
    if (strcasecmp (value, params->label_not_selected) == 0)
	return;
    len = strlen (value);
    params->column = malloc (len + 1);
    strcpy (params->column, value);
}

static void
set_field (struct config *params, const char *value)
{
/* setting the Help Field Name */
    int len;
    if (params->field != NULL)
	free (params->field);
    len = strlen (value);
    params->field = malloc (len + 1);
    strcpy (params->field, value);
}

static void
set_arg_view (struct config *params, const char *value)
{
/* setting the View Name arg */
    int len;
    if (params->arg_view != NULL)
	free (params->arg_view);
    params->arg_view = NULL;
    if (value == NULL)
	return;
    if (strcasecmp (value, params->label_not_selected) == 0)
      {
	  /* ignoring the "not selected" value */
	  return;
      }
    len = strlen (value);
    params->arg_view = malloc (len + 1);
    strcpy (params->arg_view, value);
}

static void
set_arg_query (struct config *params, const char *value)
{
/* setting the Query Mode arg */
    int len;
    if (params->arg_query != NULL)
	free (params->arg_query);
    len = strlen (value);
    params->arg_query = malloc (len + 1);
    strcpy (params->arg_query, value);
    if (strcmp (value, "aggregate") == 0)
      {
	  params->arg_count_groupby = 1;
	  params->arg_count_aggregate = 1;
      }
    else
	params->arg_count = 1;
}

static void
set_arg_count (struct config *params, const char *value)
{
/* setting the Current Count arg */
    params->arg_count = atoi (value);
}

static void
set_arg_count_groupby (struct config *params, const char *value)
{
/* setting the Current GroupBy Count arg */
    params->arg_count_groupby = atoi (value);
}

static void
set_arg_count_aggregate (struct config *params, const char *value)
{
/* setting the Current Aggregate Count arg */
    params->arg_count_aggregate = atoi (value);
}

static void
set_arg_count_orderby (struct config *params, const char *value)
{
/* setting the Current OrderBy Count arg */
    params->arg_count_orderby = atoi (value);
}

static void
alloc_filters (struct config *params)
{
/* allocating all Filters */
    int i;
    int is_filter = 1;
    if (params->arg_query != NULL)
      {
	  if (strcmp (params->arg_query, "aggregate") == 0)
	      is_filter = 0;
      }
    if (is_filter)
      {
	  /* filter query */
	  reset_all_filters (params);
	  reset_all_orderbys (params);
	  /* allocating N filters (initially empty) */
	  for (i = 0; i < params->arg_count; i++)
	    {
		struct filter *pf = malloc (sizeof (struct filter));
		pf->filter_index = i;
		pf->filter_column = NULL;
		pf->filter_comparison = COMPAR_NONE;
		pf->filter_value = NULL;
		pf->next = NULL;
		if (params->first_filter == NULL)
		    params->first_filter = pf;
		if (params->last_filter != NULL)
		    params->last_filter->next = pf;
		params->last_filter = pf;
	    }
	  /* allocating N orderbys (initially empty) */
	  for (i = 0; i < params->arg_count_orderby; i++)
	    {
		struct orderby *po = malloc (sizeof (struct orderby));
		po->orderby_index = i;
		po->orderby_column = NULL;
		po->next = NULL;
		if (params->first_orderby == NULL)
		    params->first_orderby = po;
		if (params->last_orderby != NULL)
		    params->last_orderby->next = po;
		params->last_orderby = po;
	    }
      }
    else
      {
	  /* aggregate query */
	  reset_all_aggregations (params);
	  reset_all_orderbys (params);
	  /* allocating N group by clauses (initially empty) */
	  for (i = 0; i < params->arg_count_groupby; i++)
	    {
		struct groupby *pg = malloc (sizeof (struct groupby));
		pg->groupby_index = i;
		pg->groupby_column = NULL;
		pg->groupby_comparison = COMPAR_NONE;
		pg->groupby_value = NULL;
		pg->next = NULL;
		if (params->first_groupby == NULL)
		    params->first_groupby = pg;
		if (params->last_groupby != NULL)
		    params->last_groupby->next = pg;
		params->last_groupby = pg;
	    }
	  /* allocating N aggregate clauses (initially empty) */
	  for (i = 0; i < params->arg_count_aggregate; i++)
	    {
		struct aggregate *pa = malloc (sizeof (struct aggregate));
		pa->aggregate_index = i;
		pa->aggregate_column = NULL;
		pa->aggregate_alias = NULL;
		pa->aggregate_function = FUNCT_NONE;
		pa->having_comparison = COMPAR_NONE;
		pa->having_value = NULL;
		pa->next = NULL;
		if (params->first_aggregate == NULL)
		    params->first_aggregate = pa;
		if (params->last_aggregate != NULL)
		    params->last_aggregate->next = pa;
		params->last_aggregate = pa;
	    }
      }
}

static void
add_filter (struct config *params, int index)
{
/* adding one more Filter */
    struct filter *pf = malloc (sizeof (struct filter));
    pf->filter_index = index;
    pf->filter_column = NULL;
    pf->filter_comparison = COMPAR_NONE;
    pf->filter_value = NULL;
    pf->next = NULL;
    if (params->first_filter == NULL)
	params->first_filter = pf;
    if (params->last_filter != NULL)
	params->last_filter->next = pf;
    params->last_filter = pf;
}

static void
add_groupby (struct config *params, int index)
{
/* adding one more Group By clause */
    struct groupby *pg = malloc (sizeof (struct groupby));
    pg->groupby_index = index;
    pg->groupby_column = NULL;
    pg->groupby_comparison = COMPAR_NONE;
    pg->groupby_value = NULL;
    pg->next = NULL;
    if (params->first_groupby == NULL)
	params->first_groupby = pg;
    if (params->last_groupby != NULL)
	params->last_groupby->next = pg;
    params->last_groupby = pg;
}

static void
add_aggregate (struct config *params, int index)
{
/* adding one more Aggregate/Having clause */
    struct aggregate *pa = malloc (sizeof (struct aggregate));
    pa->aggregate_index = index;
    pa->aggregate_column = NULL;
    pa->aggregate_alias = NULL;
    pa->aggregate_function = FUNCT_NONE;
    pa->having_comparison = COMPAR_NONE;
    pa->having_value = NULL;
    pa->next = NULL;
    if (params->first_aggregate == NULL)
	params->first_aggregate = pa;
    if (params->last_aggregate != NULL)
	params->last_aggregate->next = pa;
    params->last_aggregate = pa;
}

static void
set_filter_column (struct config *params, int i, const char *value)
{
/* setting the Nth Filter Column */
    int count = 0;
    struct filter *filter = params->first_filter;
    while (filter != NULL)
      {
	  if (i == count)
	    {
		int len = strlen (value);
		if (filter->filter_column != NULL)
		    free (filter->filter_column);
		filter->filter_column = NULL;
		if (value == NULL)
		    return;
		if (strcasecmp (value, params->label_not_selected) == 0)
		    return;
		filter->filter_column = malloc (len + 1);
		strcpy (filter->filter_column, value);
		return;
	    }
	  count++;
	  filter = filter->next;
      }
}

static void
set_filter_comparison (struct config *params, int i, const char *value)
{
/* setting the Nth Filter Comparison */
    int count = 0;
    struct filter *filter;
    int compar = COMPAR_NONE;
    if (strcmp (value, "eq") == 0)
	compar = COMPAR_EQ;
    if (strcmp (value, "ne") == 0)
	compar = COMPAR_NE;
    if (strcmp (value, "gt") == 0)
	compar = COMPAR_GT;
    if (strcmp (value, "ge") == 0)
	compar = COMPAR_GE;
    if (strcmp (value, "lt") == 0)
	compar = COMPAR_LT;
    if (strcmp (value, "le") == 0)
	compar = COMPAR_LE;
    if (strcmp (value, "lk") == 0)
	compar = COMPAR_LIKE;
    if (strcmp (value, "nlk") == 0)
	compar = COMPAR_NOT_LIKE;
    if (strcmp (value, "sw") == 0)
	compar = COMPAR_STARTS;
    if (strcmp (value, "nsw") == 0)
	compar = COMPAR_NOT_STARTS;
    if (strcmp (value, "ew") == 0)
	compar = COMPAR_ENDS;
    if (strcmp (value, "new") == 0)
	compar = COMPAR_NOT_ENDS;
    if (strcmp (value, "in") == 0)
	compar = COMPAR_NULL;
    if (strcmp (value, "inn") == 0)
	compar = COMPAR_NOT_NULL;
    filter = params->first_filter;
    while (filter != NULL)
      {
	  if (i == count)
	    {
		filter->filter_comparison = compar;
		return;
	    }
	  count++;
	  filter = filter->next;
      }
}

static void
set_filter_value (struct config *params, int i, const char *value)
{
/* setting the Nth Filter Value */
    int count = 0;
    struct filter *filter = params->first_filter;
    while (filter != NULL)
      {
	  if (i == count)
	    {
		int len = strlen (value);
		if (filter->filter_value != NULL)
		    free (filter->filter_value);
		filter->filter_value = malloc (len + 1);
		strcpy (filter->filter_value, value);
		return;
	    }
	  count++;
	  filter = filter->next;
      }
}

static void
set_groupby_column (struct config *params, int i, const char *value)
{
/* setting the Nth Group By Column */
    int count = 0;
    struct groupby *groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  if (i == count)
	    {
		int len = strlen (value);
		if (groupby->groupby_column != NULL)
		    free (groupby->groupby_column);
		groupby->groupby_column = NULL;
		if (value == NULL)
		    return;
		if (strcasecmp (value, params->label_not_selected) == 0)
		    return;
		groupby->groupby_column = malloc (len + 1);
		strcpy (groupby->groupby_column, value);
		return;
	    }
	  count++;
	  groupby = groupby->next;
      }
}

static void
set_groupby_comparison (struct config *params, int i, const char *value)
{
/* setting the Nth Group By Comparison */
    int count = 0;
    struct groupby *groupby;
    int compar = COMPAR_NONE;
    if (strcmp (value, "eq") == 0)
	compar = COMPAR_EQ;
    if (strcmp (value, "ne") == 0)
	compar = COMPAR_NE;
    if (strcmp (value, "gt") == 0)
	compar = COMPAR_GT;
    if (strcmp (value, "ge") == 0)
	compar = COMPAR_GE;
    if (strcmp (value, "lt") == 0)
	compar = COMPAR_LT;
    if (strcmp (value, "le") == 0)
	compar = COMPAR_LE;
    if (strcmp (value, "lk") == 0)
	compar = COMPAR_LIKE;
    if (strcmp (value, "nlk") == 0)
	compar = COMPAR_NOT_LIKE;
    if (strcmp (value, "sw") == 0)
	compar = COMPAR_STARTS;
    if (strcmp (value, "nsw") == 0)
	compar = COMPAR_NOT_STARTS;
    if (strcmp (value, "ew") == 0)
	compar = COMPAR_ENDS;
    if (strcmp (value, "new") == 0)
	compar = COMPAR_NOT_ENDS;
    if (strcmp (value, "in") == 0)
	compar = COMPAR_NULL;
    if (strcmp (value, "inn") == 0)
	compar = COMPAR_NOT_NULL;
    groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  if (i == count)
	    {
		groupby->groupby_comparison = compar;
		return;
	    }
	  count++;
	  groupby = groupby->next;
      }
}

static void
set_groupby_value (struct config *params, int i, const char *value)
{
/* setting the Nth Group By Value */
    int count = 0;
    struct groupby *groupby = params->first_groupby;
    while (groupby != NULL)
      {
	  if (i == count)
	    {
		int len = strlen (value);
		if (groupby->groupby_value != NULL)
		    free (groupby->groupby_value);
		groupby->groupby_value = malloc (len + 1);
		strcpy (groupby->groupby_value, value);
		return;
	    }
	  count++;
	  groupby = groupby->next;
      }
}

static void
set_aggregate_column (struct config *params, int i, const char *value)
{
/* setting the Nth Aggregate Column */
    int count = 0;
    struct aggregate *aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  if (i == count)
	    {
		int len = strlen (value);
		if (aggregate->aggregate_column != NULL)
		    free (aggregate->aggregate_column);
		aggregate->aggregate_column = NULL;
		if (value == NULL)
		    return;
		if (strcasecmp (value, params->label_not_selected) == 0)
		    return;
		aggregate->aggregate_column = malloc (len + 1);
		strcpy (aggregate->aggregate_column, value);
		return;
	    }
	  count++;
	  aggregate = aggregate->next;
      }
}

static void
set_aggregate_function (struct config *params, int i, const char *value)
{
/* setting the Nth Aggregate Function */
    int count = 0;
    struct aggregate *aggregate;
    int fnct = FUNCT_NONE;
    if (strcmp (value, "sum") == 0)
	fnct = FUNCT_SUM;
    if (strcmp (value, "avg") == 0)
	fnct = FUNCT_AVG;
    if (strcmp (value, "count") == 0)
	fnct = FUNCT_COUNT;
    if (strcmp (value, "max") == 0)
	fnct = FUNCT_MAX;
    if (strcmp (value, "min") == 0)
	fnct = FUNCT_MIN;
    aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  if (i == count)
	    {
		aggregate->aggregate_function = fnct;
		return;
	    }
	  count++;
	  aggregate = aggregate->next;
      }
}

static void
set_having_comparison (struct config *params, int i, const char *value)
{
/* setting the Nth Having Comparison */
    int count = 0;
    struct aggregate *aggregate;
    int compar = COMPAR_NONE;
    if (strcmp (value, "eq") == 0)
	compar = COMPAR_EQ;
    if (strcmp (value, "ne") == 0)
	compar = COMPAR_NE;
    if (strcmp (value, "gt") == 0)
	compar = COMPAR_GT;
    if (strcmp (value, "ge") == 0)
	compar = COMPAR_GE;
    if (strcmp (value, "lt") == 0)
	compar = COMPAR_LT;
    if (strcmp (value, "le") == 0)
	compar = COMPAR_LE;
    if (strcmp (value, "lk") == 0)
	compar = COMPAR_LIKE;
    if (strcmp (value, "nlk") == 0)
	compar = COMPAR_NOT_LIKE;
    if (strcmp (value, "sw") == 0)
	compar = COMPAR_STARTS;
    if (strcmp (value, "nsw") == 0)
	compar = COMPAR_NOT_STARTS;
    if (strcmp (value, "ew") == 0)
	compar = COMPAR_ENDS;
    if (strcmp (value, "new") == 0)
	compar = COMPAR_NOT_ENDS;
    if (strcmp (value, "in") == 0)
	compar = COMPAR_NULL;
    if (strcmp (value, "inn") == 0)
	compar = COMPAR_NOT_NULL;
    aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  if (i == count)
	    {
		aggregate->having_comparison = compar;
		return;
	    }
	  count++;
	  aggregate = aggregate->next;
      }
}

static void
set_having_value (struct config *params, int i, const char *value)
{
/* setting the Nth Having Value */
    int count = 0;
    struct aggregate *aggregate = params->first_aggregate;
    while (aggregate != NULL)
      {
	  if (i == count)
	    {
		int len = strlen (value);
		if (aggregate->having_value != NULL)
		    free (aggregate->having_value);
		aggregate->having_value = malloc (len + 1);
		strcpy (aggregate->having_value, value);
		return;
	    }
	  count++;
	  aggregate = aggregate->next;
      }
}

static void
set_orderby_column (struct config *params, int i, const char *value)
{
/* setting the Nth Order By Column */
    int count = 0;
    struct orderby *orderby = params->first_orderby;
    while (orderby != NULL)
      {
	  if (i == count)
	    {
		int len = strlen (value);
		if (orderby->orderby_column != NULL)
		    free (orderby->orderby_column);
		orderby->orderby_column = NULL;
		if (value == NULL)
		    return;
		if (strcasecmp (value, params->label_not_selected) == 0)
		    return;
		orderby->orderby_column = malloc (len + 1);
		strcpy (orderby->orderby_column, value);
		return;
	    }
	  count++;
	  orderby = orderby->next;
      }
}

static void
set_arg_format (struct config *params, const char *value)
{
/* setting the Output Format arg */
    int len;
    if (params->arg_format != NULL)
	free (params->arg_format);
    len = strlen (value);
    params->arg_format = malloc (len + 1);
    strcpy (params->arg_format, value);
}

static void
set_arg_select (struct config *params, const char *value)
{
/* setting the Select arg */
    int len;
    if (params->arg_select != NULL)
	free (params->arg_select);
    len = strlen (value);
    params->arg_select = malloc (len + 1);
    strcpy (params->arg_select, value);
}

static void
set_html_offset (struct config *params, const char *value)
{
/* setting the HTML offset arg */
    params->current_html_offset = atoi (value);
}

static void
test_view_has_changed (struct config *params)
{
/* testing if the current view or current query have changed */
    if (params->arg_view == NULL)
      {
	  if (cgi_param ("cvw"))
	      params->changed_reset_all = 1;
	  goto end;
      }
    if (cgi_param ("cvw"))
      {
	  if (strcmp (params->arg_view, cgi_param ("cvw")) != 0)
	      params->changed_reset_all = 1;
      }
    else
	params->changed_reset_all = 1;
    if (cgi_param ("cqy"))
      {
	  if (strcmp (params->arg_query, cgi_param ("cqy")) != 0)
	      params->changed_reset_all = 1;
      }
  end:
    if (params->changed_reset_all)
      {
	  params->arg_count = 1;
	  params->arg_count_groupby = 1;
	  params->arg_count_aggregate = 1;
	  params->arg_count_orderby = 0;
      }
}

static void
arrange_orderby (struct config *params)
{
/* properly arranging all ORDER BY clauses */
    int i;
    struct orderby *base = params->first_orderby;
    struct orderby *orderby;
    struct orderby *orderby_out;

    params->first_orderby = NULL;
    params->last_orderby = NULL;
/* allocating N orderbys (initially empty) */
    for (i = 0; i < params->arg_count_orderby; i++)
      {
	  struct orderby *po = malloc (sizeof (struct orderby));
	  po->orderby_index = i;
	  po->orderby_column = NULL;
	  po->next = NULL;
	  if (params->first_orderby == NULL)
	      params->first_orderby = po;
	  if (params->last_orderby != NULL)
	      params->last_orderby->next = po;
	  params->last_orderby = po;
      }

/* reinserting valid clauses */
    orderby = base;
    while (orderby != NULL)
      {
	  int already_defined = 0;
	  if (orderby->orderby_column == NULL)
	    {
		/* skipping: undefined clause */
		orderby = orderby->next;
		continue;
	    }
	  orderby_out = params->first_orderby;
	  while (orderby_out != NULL)
	    {
		/* testing for duplicates */
		if (orderby_out->orderby_column == NULL)
		  {
		      orderby_out = orderby_out->next;
		      continue;
		  }
		if (strcasecmp
		    (orderby_out->orderby_column, orderby->orderby_column) == 0)
		  {
		      /* already defined: skipping */
		      already_defined = 1;
		      break;
		  }
		orderby_out = orderby_out->next;
	    }
	  if (already_defined)
	    {
		/* skipping - already defined */
		orderby = orderby->next;
		continue;
	    }
	  orderby_out = params->first_orderby;
	  while (orderby_out != NULL)
	    {
		if (orderby_out->orderby_column == NULL)
		  {
		      /* inserting */
		      orderby_out->orderby_column = orderby->orderby_column;
		      orderby->orderby_column = NULL;
		      break;
		  }
		orderby_out = orderby_out->next;
	    }
	  orderby = orderby->next;
      }
/* memory cleanup - destroying old entries */
    orderby = base;
    while (orderby != NULL)
      {
	  orderby_out = orderby->next;
	  if (orderby->orderby_column != NULL)
	      free (orderby->orderby_column);
	  free (orderby);
	  orderby = orderby_out;
      }
}

static void
do_get_args (struct config *params)
{
/* retrieving the current arguments */
    int i;
    char *name;
    if (cgi_param ("hoff"))
	set_html_offset (params, cgi_param ("hoff"));
    if (cgi_param ("f"))
      {
	  /* special request: help Field Values */
	  if (cgi_param ("vw"))
	      set_arg_view (params, cgi_param ("vw"));
	  if (cgi_param ("c"))
	      set_column (params, cgi_param ("c"));
	  set_field (params, cgi_param ("f"));
	  if (cgi_param ("cnt"))
	      set_arg_count (params, cgi_param ("cnt"));
	  alloc_filters (params);
	  for (i = 0; i < params->arg_count; i++)
	    {
		name = sqlite3_mprintf ("c%d", i);
		if (cgi_param (name))
		    set_filter_column (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("x%d", i);
		if (cgi_param (name))
		    set_filter_comparison (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("v%d", i);
		if (cgi_param (name))
		    set_filter_value (params, i, cgi_param (name));
		sqlite3_free (name);
	    }
	  params->is_field_help = 1;
	  return;
      }
    if (cgi_param ("gu"))
	params->is_get_url = 1;
    if (cgi_param ("go"))
	params->is_submit = 1;
    if (cgi_param ("q"))
	set_arg_query (params, cgi_param ("q"));
    else
	params->arg_count = 1;
    if (cgi_param ("rst"))
      {
	  params->reset = 1;
	  alloc_filters (params);
	  return;
      }
    if (cgi_param ("target"))
      {
	  if (strcmp (cgi_param ("target"), "results") == 0)
	      params->is_target_results = 1;
	  else
	      params->is_target_params = 1;
      }
    if (cgi_param ("vw"))
	set_arg_view (params, cgi_param ("vw"));
    if (cgi_param ("cnt"))
	set_arg_count (params, cgi_param ("cnt"));
    if (cgi_param ("cntg"))
	set_arg_count_groupby (params, cgi_param ("cntg"));
    if (cgi_param ("cnta"))
	set_arg_count_aggregate (params, cgi_param ("cnta"));
    if (cgi_param ("cnto"))
	set_arg_count_orderby (params, cgi_param ("cnto"));
    alloc_filters (params);
    if (cgi_param ("addf.x"))
      {
	  add_filter (params, params->arg_count);
	  params->arg_count += 1;
      }
    if (cgi_param ("addg.x"))
      {
	  add_groupby (params, params->arg_count_groupby);
	  params->arg_count_groupby += 1;
      }
    if (cgi_param ("adda.x"))
      {
	  add_aggregate (params, params->arg_count_aggregate);
	  params->arg_count_aggregate += 1;
      }
    if (params->changed_reset_all == 0)
      {
	  for (i = 0; i < params->arg_count; i++)
	    {
		name = sqlite3_mprintf ("c%d", i);
		if (cgi_param (name))
		    set_filter_column (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("x%d", i);
		if (cgi_param (name))
		    set_filter_comparison (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("v%d", i);
		if (cgi_param (name))
		    set_filter_value (params, i, cgi_param (name));
		sqlite3_free (name);
	    }
	  for (i = 0; i < params->arg_count_groupby; i++)
	    {
		name = sqlite3_mprintf ("gc%d", i);
		if (cgi_param (name))
		    set_groupby_column (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("gx%d", i);
		if (cgi_param (name))
		    set_groupby_comparison (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("gv%d", i);
		if (cgi_param (name))
		    set_groupby_value (params, i, cgi_param (name));
		sqlite3_free (name);
	    }
	  for (i = 0; i < params->arg_count_aggregate; i++)
	    {
		name = sqlite3_mprintf ("ac%d", i);
		if (cgi_param (name))
		    set_aggregate_column (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("af%d", i);
		if (cgi_param (name))
		    set_aggregate_function (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("hc%d", i);
		if (cgi_param (name))
		    set_having_comparison (params, i, cgi_param (name));
		sqlite3_free (name);
		name = sqlite3_mprintf ("hv%d", i);
		if (cgi_param (name))
		    set_having_value (params, i, cgi_param (name));
		sqlite3_free (name);
	    }
	  for (i = 0; i < params->arg_count_orderby; i++)
	    {
		name = sqlite3_mprintf ("o%d", i);
		if (cgi_param (name))
		    set_orderby_column (params, i, cgi_param (name));
		sqlite3_free (name);
	    }
	  arrange_orderby (params);
      }
    if (cgi_param ("fmt"))
	set_arg_format (params, cgi_param ("fmt"));
    if (cgi_param ("select"))
	set_arg_select (params, cgi_param ("select"));
    if (params->changed_reset_all)
	set_arg_format (params, "html");
}

static void
do_build_logfile_path (struct config *params)
{
/* building the logfile path */
    struct tm *now;
    if (params->timestamps_localtime)
	now = localtime (&(params->now));
    else
	now = gmtime (&(params->now));
    if (params->logfile_insulation == LOG_INSULATION_SEPARATE)
      {
	  if (params->logfile_rotation == LOG_ROTATE_DAILY)
	      params->logfile_path =
		  sqlite3_mprintf ("%s/dataSeltzerLog_%s_%04d%02d%02d.log",
				   params->logfile_dir_path,
				   params->instance_name,
				   now->tm_year + 1900, now->tm_mon + 1,
				   now->tm_mday);
	  else
	      params->logfile_path =
		  sqlite3_mprintf ("%s/dataSeltzerLog_%s_%04d%02d.log",
				   params->logfile_dir_path,
				   params->instance_name,
				   now->tm_year + 1900, now->tm_mon + 1);
      }
    else
      {
	  if (params->logfile_rotation == LOG_ROTATE_DAILY)
	      params->logfile_path =
		  sqlite3_mprintf ("%s/dataSeltzerLog_%04d%02d%02d.log",
				   params->logfile_dir_path,
				   now->tm_year + 1900, now->tm_mon + 1,
				   now->tm_mday);
	  else
	      params->logfile_path =
		  sqlite3_mprintf ("%s/dataSeltzerLog_%04d%02d.log",
				   params->logfile_dir_path,
				   now->tm_year + 1900, now->tm_mon + 1);
      }
}

int
main (void)
{
/* initialization tasks and main logic flow */
    struct config params;
    sqlite3 *handle = NULL;
    void *cache = NULL;
    char *path;
    int truncation;
    time_t tim;

/* initializing an empty configuration */
    params.now = time (&tim);
    gettimeofday (&(params.start), NULL);
    params.logfile_dir_path = NULL;
    params.logfile_path = NULL;
    params.logfile_rotation = LOG_ROTATE_DAILY;
    params.logfile_insulation = LOG_INSULATION_SEPARATE;
    params.instance_name = NULL;
    params.instance_config_path = NULL;
    params.enable_multi_instances = 0;
    params.enable_text_xml = 1;
    params.full_blob_xml = 0;
    params.full_xml_column_infos = 1;
    params.timestamps_localtime = 1;
    params.db_handle = NULL;
    params.db_path = NULL;
    params.http_request_method = METHOD_GET;
    params.max_resultset_rows = -1;
    params.max_help_resultset_rows = -1;
    params.max_rows_html_page = -1;
    params.include_dir = NULL;
    params.temp_dir = NULL;
    params.cgi_url = NULL;
    params.icon_help_url = NULL;
    params.icon_add_url = NULL;
    params.icon_first = NULL;
    params.icon_last = NULL;
    params.icon_prev = NULL;
    params.icon_next = NULL;
    params.css_stylesheet_url = NULL;
    params.label_data_truncation = NULL;
    params.label_not_selected = NULL;
    params.label_instance = NULL;
    params.label_datasource = NULL;
    params.label_query_mode = NULL;
    params.label_mode_filter = NULL;
    params.label_mode_aggregate = NULL;
    params.label_filter_by = NULL;
    params.label_and_filter_by = NULL;
    params.label_order_by = NULL;
    params.label_group_by = NULL;
    params.label_and_group_by = NULL;
    params.label_aggregate = NULL;
    params.label_and_aggregate = NULL;
    params.label_any_value = NULL;
    params.label_equal = NULL;
    params.label_not_equal = NULL;
    params.label_greater = NULL;
    params.label_greater_equal = NULL;
    params.label_lesser = NULL;
    params.label_lesser_equal = NULL;
    params.label_like = NULL;
    params.label_not_like = NULL;
    params.label_starts_with = NULL;
    params.label_not_starts_with = NULL;
    params.label_ends_with = NULL;
    params.label_not_ends_with = NULL;
    params.label_is_null = NULL;
    params.label_is_not_null = NULL;
    params.label_output_format = NULL;
    params.label_output_html = NULL;
    params.label_output_xml = NULL;
    params.label_output_xml_zip = NULL;
    params.label_output_csv = NULL;
    params.label_output_dbf = NULL;
    params.label_output_shapefile = NULL;
    params.label_output_spatialite = NULL;
    params.label_button_reset = NULL;
    params.label_button_go = NULL;
    params.label_button_get_url = NULL;
    params.label_function_none = NULL;
    params.label_function_sum = NULL;
    params.label_function_avg = NULL;
    params.label_function_count = NULL;
    params.label_function_min = NULL;
    params.label_function_max = NULL;
    params.label_help_header = NULL;
    params.label_help_truncation = NULL;
    params.current_view_name = NULL;
    params.current_geometry_column = NULL;
    params.first_hidden = NULL;
    params.last_hidden = NULL;
    params.extra_data_dir = NULL;
    params.extra_data_subdirs = 0;
    params.first_extra = NULL;
    params.last_extra = NULL;
    params.arg_view = NULL;
    params.arg_query = NULL;
    params.arg_count = 0;
    params.arg_count_groupby = 0;
    params.arg_count_aggregate = 0;
    params.arg_count_orderby = 0;
    params.first_filter = NULL;
    params.last_filter = NULL;
    params.first_groupby = NULL;
    params.last_groupby = NULL;
    params.first_aggregate = NULL;
    params.last_aggregate = NULL;
    params.first_orderby = NULL;
    params.last_orderby = NULL;
    params.arg_format = NULL;
    params.arg_select = NULL;
    params.is_target_params = 0;
    params.is_target_results = 0;
    params.is_get_url = 0;
    params.is_submit = 0;
    params.changed_reset_all = 0;
    params.reset = 0;
    params.column = NULL;
    params.field = NULL;
    params.is_field_help = 0;
    params.current_html_offset = 0;
    params.first_instance = NULL;
    params.last_instance = NULL;
    params.CharDataLen = 0;
    params.CharDataMax = BUFFSIZE;
    params.CharData = malloc (BUFFSIZE);
/* initialing CGI */
    cgi_init ();
    cgi_process_form ();
/* loading the internal configuration */
    if (cgi_param ("muin"))
	set_instance_name (&params, cgi_param ("muin"));
    else if (cgi_param ("in"))
	set_instance_name (&params, cgi_param ("in"));
    if (!parse_main_config (&params))
      {
	  fprintf (stderr, "dataSeltzer: Invalid Main config-file\n");
	  goto error;
      }
    if (params.instance_config_path == NULL)
      {
	  fprintf (stderr,
		   "dataSeltzer: Undefined Instance config-file [in=%s]\n",
		   params.instance_name);
	  goto error;
      }
    if (!parse_instance_config (&params))
      {
	  fprintf (stderr,
		   "dataSeltzer: Invalid Instance config-file [in=%s]\n",
		   params.instance_name);
	  goto error;
      }
    do_build_logfile_path (&params);

/* opening the DB-file */
    cache = spatialite_alloc_connection ();
    open_db (params.db_path, &handle, cache);
    if (!handle)
	goto error;
    params.db_handle = handle;

/* processing the request */
    do_get_args (&params);
    if (params.reset)
      {
	  /* RESET: initializing the frameset */
	  cgi_init_headers ();
	  do_output_frameset (&params);
	  goto stop;
      }
    if (params.arg_view == NULL)
      {
	  params.is_field_help = 0;
	  params.is_get_url = 0;
	  params.is_submit = 0;
      }
    if (params.is_field_help)
      {
	  /* Field Help */
	  cgi_init_headers ();
	  do_output_field_help (&params);
	  goto stop;
      }
    if (params.is_get_url)
      {
	  /* GetURL request */
	  cgi_init_headers ();
	  do_output_get_url (&params);
	  goto stop;
      }
    if (params.is_submit)
      {
	  /* processing a data request */
	  if (strcmp (params.arg_format, "xml") == 0)
	    {
		if (params.enable_text_xml)
		  {
		      if (!do_output_xml_resultset (&params, &truncation))
			  goto export_failure;
		  }
		else
		    goto unsupported_text_xml;
	    }
	  else if (strcmp (params.arg_format, "xmlzip") == 0)
	    {
		if (!do_output_xmlzip_resultset (&params, &truncation))
		    goto export_failure;
	    }
	  else if (strcmp (params.arg_format, "csv") == 0)
	    {
		if (!do_output_csv_resultset (&params, &truncation))
		    goto export_failure;
	    }
	  else if (strcmp (params.arg_format, "dbf") == 0)
	    {
		if (!do_output_dbf_resultset (&params, &truncation))
		    goto export_failure;
	    }
	  else if (strcmp (params.arg_format, "shp") == 0)
	    {
		if (!do_output_shp_resultset (&params, &truncation))
		    goto export_failure;
	    }
	  else if (strcmp (params.arg_format, "sqlite") == 0)
	    {
		if (!do_output_sqlite_resultset (&params, &truncation))
		    goto export_failure;
	    }
	  else
	    {
		cgi_init_headers ();
		do_output_html_resultset (&params);
	    }
	  goto stop;
      }
    if (params.is_target_params)
      {
	  /* initializing the "params" frame */
	  cgi_init_headers ();
	  do_output_params_frame (&params);
	  goto stop;
      }
    if (params.is_target_results)
      {
	  /* initializing the "results" frame (empty) */
	  cgi_init_headers ();
	  myputs
	      ("<!DOCTYPE html PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n");
	  myputs ("<html>\n<head>\n");
	  myputs
	      ("<meta http-equiv='content-type' content='text/html; charset=UTF-8'>\n");
	  myputs ("<title>void</title>\n");
	  myputs ("</head>\n<body>\n</body>\n</html>\n");
	  goto stop;
      }
    test_view_has_changed (&params);
    if (params.arg_view == NULL && params.changed_reset_all == 0)
      {
	  /* initializing the frameset */
	  cgi_init_headers ();
	  do_output_frameset (&params);
	  goto stop;
      }
/* updating the UI ("params" frame) */
    cgi_init_headers ();
    test_view_has_changed (&params);
    do_output_params_frame (&params);
    goto stop;

/* some absolutely unexpected error occurred */
  error:
    cgi_init_headers ();
    if (params.include_dir != NULL)
      {
	  path = sqlite3_mprintf ("%s/misconfigured.inc", params.include_dir);
	  cgi_include (path);
	  sqlite3_free (path);
      }
    else
      {
	  /* just in case the current configuration in completely bogus */
	  myputs
	      ("<!DOCTYPE html PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n");
	  myputs ("<html>\n<head>\n");
	  myputs
	      ("<meta http-equiv='content-type' content='text/html; charset=UTF-8'>\n");
	  myputs ("<title>dataSeltzer configuration error</title>\n");
	  myputs
	      ("</head>\n<body>\n<h1>dataSeltzer configuration error</h1>\n");
	  myputs
	      ("<p>this CGI is misconfigured and cannot correctly work !!!<br><br>\n");
	  myputs
	      ("Please urgently contact the Web Administator reporting this issue.\n");
	  myputs ("</p>\n</body>\n</html>\n");
      }
    goto stop;

/* unsupported text/xml output */
  unsupported_text_xml:
    cgi_init_headers ();
    path = sqlite3_mprintf ("%s/unsupported_xml.inc", params.include_dir);
    cgi_include (path);
    sqlite3_free (path);
    goto stop;

/* for some reason an attempt to prepare a zip-download failed */
  export_failure:
    cgi_init_headers ();
    path = sqlite3_mprintf ("%s/zip_error.inc", params.include_dir);
    cgi_include (path);
    sqlite3_free (path);

/* memory cleanup and final steps */
  stop:
    cgi_end ();
    free_config (&params);
    if (handle != NULL)
	sqlite3_close (handle);
    if (cache != NULL)
	spatialite_cleanup_ex (cache);
    spatialite_shutdown ();
    return 0;
}
